
;
; Atari XE Space Harrier Conversion Project (c) 1987-2003 Chris Hutt
; - Cart Version
;

;
; To do list
;

;
; Stage 1:
; 1) Better dragon patterns?
;
; Stage 2:
; 1) Music for end boss
; 2) Stone pillar needs re-drawing
;
; Stage 3:
; 1) Music for end boss
;
; Stage 4:
; 1) top_grid_lo, hi into temp mem rather than perm data
; 2) background is rough only
; 3) Music
; 4) Robot sound fx
;
; Stage 5:
; 1) Music
;
; Stages 6-18:
; ?) Everything!
;
;
; Source code tidy up:
; 1) Maybe make all subroutines start with a verb that describes their purpose. ie move_dlis, make_table
; 	Variables should then not include a verb at start to make it clear they are variables 
;
;
; Optimization:
; 1) Better way to find right object frames?
; 2) Looped code optimization, and general code tweaking.
; 3) smaller code for memory management/loading? (Not speed critical)
; 4) VBI cleanup - must be some way to not check man_status twice AND set dlist address early.
; 5) Do we need the hide property now -ve z is used for that?
;	Maybe. The explosions or dead are harder to distinguish otherwise from aliens.
; 6) using full size as width rather than 1/2 size would be slightly quicker
; 7) player draw in VBI can be unrolled
; 8) with around 25+ dlis it is quicker to re-set the whole dlist rather than just certain lines
; 9) sample play routine
; 10) look at DLI routines again. The tacked on code for DLI on 1st line can be done better.
;
;
; End stage objects:
; 1) Stage 17 boss - Wi-wi Jumbo. The scenery objects rise up and kamikaze you
;	could be some kind of repeat pattern objects that fire missiles that look like themselves,
;	and then disappear themselves? Or, modification to scenery handler?
;	wi-wi dogs die only when he does. linked objects.
; 2) Objects make no sound when they initialize as bosses. Presently have separate objects for
;	these "no sound" objects. Better to have an end_stage active flag to check for sound required
;
;
; Uriah mini-game:
; 1) Special "chained" scenery routine rather than totally random x position
;	could use normal pattern handler?
;
;
; Object Sizes:
; 1) Real size may be too small on smaller frames to hit objects easily enough.
; 2) using full size as width rather than 1/2 size would be slightly quicker
;
;
; Music:
; 1) maybe convert timers for correct PAL playback speed, since can't get it right just with beat timer.
;	only to be done at setup time. EG beat of 5 changes to 4. 7 to 6. 30 to 24, 2 to 2, etc.
;	Make beat timer happen every VBI?
;
; Title screen:
; 1) Use game objects. Animation of objects with player/missile?
;	Overlay with other object animations rather than player/missile seems better
;
;
;
; Tester feedback:
;
; colours dull:
; 1) Palette on atari800winplus3 is dull. Check on real atari to get best results 
;	I tested on PAL 130xe and cheap TV - the emu "real" palette is very similar
;
; 3rd party memory upgrade detection:
; 1) Try make it work with non-standard 130xe bank switching
;	tested ok in emulator with compy and rambo 320kb and all other emulated RAM upgrades
;	Mathy Nisselroy's 1Mb doesn't work. other large upgrades mess with other bits in portb
;	NB What about switching out OS while allocating memory, as the OS bit is critical to selecting
;	another bank, then switching back after? Would have to turn off interrupts, and put it back
;	for the next load. But it might work for Mathy's I think.
;
;
; Flicker quite bad (especially PAL):
; 1) Make option to select only 4-colour version? A lot of extra changes though...
;	extra version of draw routine, loading changes, extra 4-colour palette byte for each object
;	self-modifying code on jump of clear screen to select 1/2 amount only to clear.
;	bonus stage could use this 4-colour only version for speed...
; 2) Investigate other extra colour methods that flicker less.
;	None that change colours every scan line are any good for in-game - too much cpu time is used
;
; Allow to work with other processor than 6502c?:
; 1) "LAX" only available on standard Atari 6502c. 16bit upgrades do not support
;	Load different draw routine if LAX doesn't work? Need loading code changes too..
;
; Autofire doesn't work?:
; 1) Well, maybe not for one person? Seems ok on mine.


;
; Current bugs/glitches:
;
; 1) bug where going back to title screen, background didn't clear. Left stage 1 background.
;	Don't know what could have caused it. Only noticed once on 9/1/03
;
; 2) start from stage 3 go onto 4, as grid coming down die. top grid remains on restart of
;	stage 3, even though it is reset on every game restart in init_game(???)
;	Check game_status and game_command are clear
;
; 3) top score update stopped partway on playthrough from stage 1 to 3 on 6/4/03. Guess it's
;	something corrupting the top_score_status to off(0)??? Stopped at 2999090.
;	Can't reproduce problem
;
; 4) alien missiles move before showing first frame (deliberately). would look better if they
;	displayed the originating position before moving them


;
; Notes, Bugs and Fixes
; 

;
; 2/8/03	Start changeover of development to cartridge rather than disk
;
; 29/7/03	Changed Uriah z freeze to zero z position on all vector buffers relating to Uriah -
;		freezing only last vector buffer did not give enough flexibility in pattern design
;
; 28/7/03	Pattern command to total the uriah counters and add to score at the end of stage
;
;		Put back code to make PAL computers do one less VBI per frame than NTSC. Game is
;		noticeably too slow on Uriah stage especially, when running in PAL otherwise.
;
; 27/7/03	Game command to reset sprite priority for man jump off 
; 
; 26/7/03	Pattern command to centre uriah ready for man to jump off - do_centre_object
;
; 20/7/03	Overlay counter code
;
; 19/7/03	Initialize stage changes to setup players for counter overlay
;
; 11/7/03	Skip rotate check, move missiles and move bullets during uriah stage
;
; 7/7/03	Code for stopping player/missile man draw while uriah stage 
;
; 6/7/03	Finished some code to deal with wraps of vector buffer when when using the chase
;		object pattern command. Previously had assumed won't wrap - there's a 1 in 10
;		chance of it just hitting a wrap, depending on the pattern data.
;
; 1/7/03	Redone collision routine with scenery being checked against uriah, rather
;		than the other way around. Each piece of scenery can only be on one z position
;		so it will be easy to skip the collision check with uriah if there is only
;		1 object in that z position. Uriah is more complicated in its use of z position
;
; 28/6/03	Uriah collision routine started again. Had to go back to backup from last week
;		due to corruption somewhere.
;
;		Small optimize to check_object_collisions. Use x reg instead of zp first_object
;
;		Man right limits were one pixel off intended position
;
; 21/6/03	Past couple of days doing pattern commands for beginning of uriah stage -
;		code for man chasing uriah, ready to jump. code to control man prior to this
;		done earlier. Consolidation of best way to arrange these pattern commands
;
; 14/6/03	last_alien and last_handler initialized to 0 in init_stage for easy uriah setup
;
;		As a game command does not run until end of game loop, and pattern command gets
;		configured by a game command, but gets called mid-loop by patterns - check for
;		use of the pattern command before it has been configured. Pattern command could
;		point to any data before it is configured, making corruption highly likely if it
;		is called too early. game_status now checked before pattern command call
;
; 1/6/03	Made event sequencer able to start straight away at stage start, for man
;		initialization on uriah stage. Stage data changed IAW.
;
; 15/5/03	Missing plp in decode_pattern in the new frame animation was causing crashes
;
; 12/5/03	Change to object frame animation. Needed to be made more flexible so man animation
;		can be done correctly before the jump onto Uriah. Could only move one frame back
;		or forwards (or mirror) before. Now has ability to move back 16 or forward 15
;		frames at expense of x vector not being used at the same time.
;
; 11/5/03	Change to sprite format file. Qty of zeros to write for missing frames changed to
;		qty of missing frames. There are 2 zeros for each missing frame. Reason for this
;		change is that the man graphics before jumping on Uriah is only one frame and there
;		aren't enough bits to represent all the zeros needed. create_object code changed to
;		multiply the figure by 2. All object files changed.
;
;		object_to_slot amount of objects was too low and routines referencing it were using
;		wrong qty of objects causing problems of animating to incorrect objects.
;		constant created to avoid this problem.
;
; 10/5/03	Code to allow loading of raw graphic data in ASH format. To allow more colours
;		than 4 per object. This is necessary for Uriah with the man sitting on him.
;		Changed load_object, create_object, decode_frame. frame_no used to denote this
;		situation in the file. Also had to move read_joystick from $3000 area to make room.
;		Changes uglify some already ugly code. A cart version with none of this loading and
;		memory management code, or any 4 colour limitations would be very nice...
;
; 9/5/03	Code insert for init_alien to change uriah head into uriah segment for all
;		following aliens - using bit 7 of pat_object to denote. This was done because the
;		joystick follow routine required all of Uriah to be in the same pattern to work,
;		and it seemed the easiest way to allow more than one object type per pattern
;
; 8/5/03	Moved check for pattern command after decode of x movement bits so Uriah
;		x movement can be done as a pattern command and override pattern x movement
;
; 6/5/03	Was not removing mirror bit on properties of object in move_objects(aliens).
;		Code change would not fit, but made room by optimizing pat_kill reduction.
;
; 30/4/03	Code to flip to mirror version of sprite put into move_objects(aliens). To make
;		room, restoration of accumulator and stripping of fire bit put into init_missile
;
; 24/4/03	Change labels starting with "_" to "@" so code will assemble with Atasm 1.03
;
; 22/4/03	Made missiles fire from centre of aliens, not where their y position happens to be.
;		Was previously firing from y position, be it top, centre or bottom of object.
;
; 16/4/03	Added lookup table for global timer (timer_wait) now fps can change 
;
; 15/4/03	Added entries to x11_table now there can be 22 objects per stage
;
; 14/4/03	Add in code to set speed of each stage from overlay variable stage_speed
;
; 13/4/03	Created background_y variable to save doing horizon-9. "tilt" sets it in the *3 code
;
; 12/4/03	Increased objects allowed to 22 to use up spare bytes in overlay. This required moving
;		some variables in $600 area to make room for other tables which increase in size the
;		more objects there are. Re-did all stage overlays
;
; 6/4/03	Added code to allow DLI on first line of screen since it looks a bit ugly on some
;		stages by not allowing it. The code needs work though as it is still a bit cludgy
;
; 5/4/03	Changed position co-ordinates on sky colour to allow full freedom of positioning
;		DLIS anywhere. Previously position 57 was off limits due to -57 adjustment pre-
;		calculated into co-ordinates; zero is used to denote end of data. Sky_scroll
;		now used instead of horizon to store scroll value(always 57 less than horizon)
;
; 4/4/03	Still doing code for gradual grid colour cycling - changed previous method and
;		changed overlay area layout again
;
; 31/3/03	Work began on code for gradual grid colour colour cycling in move_dlis
;		
; 29/3/03	Tweaking DLIS again, as I'm ramping up the amount of them for nicer colour shading.
;		Moved 1st DLI to blank line. Now there is time to finish the inc before the 
;		sta WSYNC in the DLI
;
;		An experiment to test how quickly the colour can be changed on acknowledging an NMI
;		shows there is not quite enough time to change the colour before the scanline is
;		visible, even with normal rather than wide screen. Should be feasible with a narrow
;		screen though. Since nearly 100 cycles are "wasted" per DLI this could be well worth
;		looking at for other games. Using NOP's instead of WSYNC doesn't gain much time for
;		an early exit to DLI's either.
;
;		Put clear of game_status into init_game, in case there is an outstanding request
;		after game over, Luckily this actually happened, so highlighting the problem, despite
;		it being incredibly unlikely.
;
; 28/3/03	Use a constant for maximum amount of DLIS to make changing dli buffers easier
;
; 27/3/03	Removed temp_mask_table code - no longer required
;
;		Moved grid colour data after objects_required in overlay so stage 0 takes 1 sector
;
; 26/3/03	Extended overlay area an extra sector, for grid colour animation. Moving around
;		variables to make room. For some reason I hadn't been using page 6. Now used
;		for mask_table to save having to create a duplicate when the OS is in. Moved
;		old position data from $700 area into the $d800 area.
;		Extension of overlay used to increase number of DLI positions and patterns
;		available per stage too, as well as the initial game speed for the stage.
;
;		Renamed sample_done to sample_status
;
; 22/3/03	Top grid down and up changed to use position check to know when to stop
;		rather than a simple counter, which can't work for knowing when up finished
;
; 17/3/03	Top grid up crash due to temp2 in move_dlis not being initialized to zero
;
; 14/3/03	Merged top grid position and colour buffer code over past couple of days.
;		Tested grid up for first time. Surprize - it doesn't work. Crashes part way up
;
; 10/3/03	Fix top grid start so initial position has tilt removed. Was starting too far
;		down screen, or too far off top of screen.
;
;		If the top grid position of the next DLI was actually less than the current DLI 
;		the top grid position was set incorrectly to the next position.
;
; 9/3/03	Pattern commands do_grid_up, do_grid_down, and overlap of top grid and background
;
; 2/3/03	Top grid initial DLI positions set to 2 rather than 1; to avoid glitch on upward
;		tilt on initial downward movement of top grid
;
;		Changes to sky_index usage - all code in move_dlis now to sync with top grid moves
;
; 25/2/03	Moved inc in DLI to after WSYNC since there doesn't seem to be time on NTSC to
;		always do this before the end of line (according to AtariWinPlus 3.1)
;
; 24/2/03	Fixed z off screen (z clipping) problem. Had forgotten that if not bothering
;		to update the z clipped objects sort index in sort table, they would need to
;		be separated in the sort table from ordinary objects somehow else. Portion at front
;		of sort table set aside for the clipped objects, so no need to do the adds to give
;		a final position in table. collision and draw routines changed to take this extra
;		portion into account.
;
;		Attempt to fix active pattern count. There is an intermittent problem where the
;		pat_kill or active_pat_count is not reduced properly, but it is very rare, and the
;		cause is still not certain. Multi-hit objects now have bounce bit set after
;		initialize for kill, to stop countdown of pat_kill again by another bullet hit
;		in the same screen frame, which in theory could mess up the kill.
;
; 20/2/03	pat_kill changes do not work because sum pattern does not have the qty of aliens
;		in it(!). Change init_pattern to set force no object bit when there is a qty of
;		zero in the pat_qty rather than checking for a sum pattern as well - Only time
;		there will be a zero is for a sum pattern.
;
; 19/2/03	Made sum pattern end leave the active pattern count alone if it has an object to
;		kill (which will do that job for it). Also made to strip off the force no object
;		bit from pat_kill, so the pat_kill counts down to zero when the object does get
;		killed. NB Force no object bit is required when the sum pattern has no object of
;		its own to control.
;
; 18/2/03	Hang problem is basically confusion on my part about how I've written the sum
;		pattern handler. Well, it had to be my fault somewhere along the line.
;		The sum pattern waits, ends and reduces its object count. The move_objects also
;		watches and kills the object, and reduces the object count also...so twice killed.
;		This means the pattern qty is not 0, and so the pattern is still active. This
;		hasn't been noticed before because the sum pattern on stage 2 does not have an
;		object of its own
;
; 16/2/03	Typically, the potential problems listed yesterday are not what is causing the
;		game to hang. The off-screen z is a serious problem though. Tested dragon without
;		rotate, without z offscreen, without handler and music reset. Went back to older
;		versions of program to no avail. What else is different about this pattern?
;
;		OK, some progress. Without the fireball speedup pattern command the game doesn't
;		hang. Why is this different to the stage 1 dragon though?
;
;		The sum pattern is still running, after the pattern is supposed to be killed.
;		The pattern command is being called after it has been set to an illegal address
;		by the sequencer.
;
; 15/2/03	Past few days spent trying to track down a nasty bug that has somehow escaped
;		notice before. When moving on from stage 3 to stage 4 the game locks soon after
;		the start of stage 4 (but not always the same place). It is something to do with the
;		2 head dragon pattern, as when this is removed, everything works fine.
;
;		NB Sum pattern should not be set up first if there is a delay between it being started
;		and the patterns it sums with. This is because the sum pattern once running, always  
;		checks to see if it is the last pattern running, and if it is, it will kill itself.
;		so the other patterns must already be running or start at the same time, in order
;		for the sum pattern not to kill itself straight away.
;
;		The extra off-screen z positions will corrupt memory upto a page after sort_table
;		as their position in the sort_table is not calculated, but they are written into
;		the sort_table....somewhere.
;
; 14/2/03	init_objects was not resetting scenery object 0.
;
; 12/2/03	Changes to dli off screen checks. Uses sky_index to keep track of on-screen
;		dlis in an incremental way, rather than running through whole list every time
;		Requires initializing in init_stage for the new sky positions on every stage
;
; 9/2/03	Correction to off-screen dli position check in move_dlis, changed to 1 from 2
;
; 29/1/03	Optimize move_dlis. Reduced compares on next grid position. Merged colour and position
;		code in sky routine. No need for buffer write varibles. Optimized top_grid_pos check
;		Fixed bug in sky to top grid compare - Not skipping sky when grid dli on same position
;
; 25/1/03	Used equates file to give proper names of hardware registers rather than addresses
;
;		Made write_data save the buffer size so write_buffer didn't have to do some adding
;
;		Began renaming routines so they begin with an action, for standardisation in naming 
;
; 23/1/03	Merged "sky" and "grid" routines with top grid dli code into one routine - move_dlis
;		Title screen given its own routine - title_dlis - was using "sky"
;
; 20/1/03	Finished mirror_sprite, integrated with create_object and file_load
;
; 19/1/03	Previous week has been spent on drawing objects for stage 4
;		Started work on creating program to mirror objects.
;
;		Redone load_object so doesn't need file_same flag (moved load_file)
;		last_properties created instead for checking whether mirror has been done or not
;
;		Frame buffer and optimize buffers incorrectly in $a200, moved to screen 2a area
;		Left like that they would overwrite code that is going to be there later.
;
; 11/1/03	Sample playback. NTSC has odd number of VCOUNT, so different value every frame
;		Don't think it really matters to sound quality with my routines though.
;
;		Bug in play_sample the initial wait for VCOUNT was not using VCOUNT! Made it wait
;		for a specific scan-line now, rather than add 2 to current line to start
;
;		Made sequencer wait for all explosions to finish as well as active patterns.
;
; 10/1/03	Changed Display Lists. Removed extra blank line in Display List, and change the
;		single mode e line back to mode d line. Put fixed DLI onto mode 2 line at top instead
;		of the mode e single line. The colour change is then in exactly the right place.
;		Put the other fixed DLI at bottom onto the last mode d line, rather than the mode 2
;		line under it. Takes less cycles that way. Able to move restore of y register in DLI
;		before the WSYNC because of this change. Still not time on a mode 2 DLI or a mode e
;		or mode d first line with LMS to move the INC index in the DLI. 
;
; 8/1/03	Changed how game command is setup. Additional fields are taken from the stream to setup
;		the pattern command required and the music required on the game command. This allows
;		fewer game commands to be needed. New table of pattern_command_lo and hi.
;		Only Stage 3 implemented so far. Must still do stage 1 and 2
;
;		Experiment with extra line in Display List to see if affects the sound sample
;		playback. 48 (4 blank lines) was original, 64 (5 blank lines) was tried. 48 seems
;		to be more crackly than 64. NB Crackle is more noticable in NTSC for some reason.
;
; 5/1/03	Made init_sound run in VBI at screen flip time to sync sound fx with graphics better.
;		Main code routines now set sound_queue variable instead of calling init_sound
;
;		Add init_sound to scenery for objects like loopers and hexagons
;
; 3/1/03	Delay of 2 seconds at beginning of all stages - init_stage sequence and stream timers.
;		fixed control - needed initializing to zero in init_objects
;
; 24/12/02	No action on game_command. 1 second pause is not too bad. Decided to up the
;		resolution of timer to once every 1/2 second so more fine control
;		wait_second changed to wait_timer
;
;		Changed rotate_dragon alien numbers, so dragon heads can start simultaneously
;
; 23/12/02	Problem with when game_command is run. Only runs at end of code loop. But for 2 head
;		dragon to reset the last_handler with game_command it is too late if the setup
;		patterns for the dragon are run immediately. There has to be a 1 second pause. That
;		adds up to quite a delay with all the other pauses getting the 2 head dragon right.
;
; 12/12/02	Added some extra bytes onto back of sort_index to allow some objects to remain hidden
;		at back of playfield.
;		Created sprite_count variable instead of using sort_index+21
;		Stop man bullets reaching into the back of playfield - limited to z=18 now
;
; 10/12/02	Sum pattern was still checking wrong bit to identify the sum object
;		init_alien shifts the whole properties byte left, so sum_bit*2 is correct bit
;
;		Moved rotate_dragon code after all other alien movement code
;
; 8/11/02	Rotate_dragon was trying to set the x position of a head rather than a segment.
;		Sum pattern was checking wrong bit to identify the sum object
;
; 6/11/02	Bug where gremlins initialized to wrong position or ships go missing, occasionally
;		Have been looking for past 2 weeks, but have found no reason yet. Looks like some
;		kind of initialize problem with the aliens. Don't know when bug was introduced.
;		Changes of 6/10/02 where frame animation is done before fire check removed.
;		The fire check must be done first or it is possible the frame change buffer will
;		put incorrect data into an object. This is because fire check detects the end of
;		pattern. This was not the cause of the bug though...GRRRRR!
;
;		*Think* the bug is to do with rotate_dragon, With the call to that routine gone
;		I haven't got it to go wrong yet.
;
; 27/10/02	sum_pat changed to sum_status. rotate_status initialize changed in init_pattern
;		last pattern in a rotate series should turn on the rotate else gets reset
;
; 22/10/02	Change to how patterns are linked. pattern link_bit denotes the pattern links to
;		the previous pattern, rather than the control_bit specifying the start of a link.
;		control_bit removed - renamed to link_bit. control pattern is now called link pattern
;		pattern_bit removed - used for new hit_count_bit instead, which specifies the object
;		in a pattern requires multiple hits to kill and maybe dying animation frames
;
; 20/10/02	Change to pattern stream properties - use stream_object to detect whether a pattern
;		is required or a command. This gets an extra bit back for the properties.
;
; 16/10/02	Changes to only reduce pat_kill as control linked alien is exploded iaw 15/10/02
;
; 15/10/02	There is a problem with the pattern commands, now that the dragon is actually
;		2 patterns using same data. The priority swap command doesn't work! It does swap
;		but the other pattern swaps back again.
;		Fixed apply die animation yesterday - a missing tax.
;		Made code simpler by using different pattern data for the dragon and segments
;		Removed check to stop bounce objects firing, now that different pattern data is used.
;		Change to "aliens" so uses vector=0 rather than pat_kill=0 to determine whether
;		object is to be control killed. This has the benefit of allowing destructible
;		objects to be control killed, there is no need to risk their pat_kill getting
;		reduced to below zero by a normal hit; as pat_kill was being set to zero before
;		they had died to flag a kill. Bonus of being slightly optimized as well.
;
;		init_missile changed to not lookup missile properties, but to use immediate mode
;
; 13/10/02	First trials of new pattern handler. Changes are not great but there are major
;		problems of course. Main change has been concept of linking to next pattern if
;		the pattern is a control pattern. The control pattern is then responsible to kill
;		and animate the objects of that linked pattern. The cludgy multi_hit_bit was removed
;		and the lead_bit. In their place the animate_bit and rotate_bit have been created.
;		The animate_bit is necessary for the wi-wi jumbo dogs which are not animated, but
;		wi-wi is. pat_lead removed as now each pattern can only have one object type. This
;		gets round problem of having to create several instances of segment as described in
;		23/9/02 bugs. multi-hit is handled by having the hit count and the amount of frames
;		stored in the spacing - since the control pattern is allowed only 1 alien, the
;		spacing was redundant.
;		red_control_aliens changed to apply_die_frames
;		init_kill_control_aliens changed to init_kill_aliens
;		kill_control_aliens changed to kill_alien
;
; 7/10/02	Tiny optimizes in bullet_collision and decode_segment - reduced clear carrys
;		Small fix to move_alien optimize - must keep z pos in y reg.		
;
; 6/10/02	Changes to not allow indestructible bounce objects fire has stopped animated triads
;		from firing. Their frame properties don't get set until after the firing which messes
;		things up. Moved code in "aliens" around to fix
;		Removed 29/9/02 changes. A sum alien can't fire if it works like that
;		Worked on new handler in init_pattern and init_alien. Optimized some variable restores
;		Optimized move_alien slightly. 3d check changed to use ldy control,x
;
; 5/10/02	Put in new handler code to not allow indestructible objects to fire in "init_missile"
;
; 29/9/02	Started change to rationalize pattern handler. If there is a sum pattern object
;		then it gets pointed to vector 0 (no movement). There is no need to check to stop
;		sum pattern object getting moved twice. Changes to init_alien and sum pattern
;		"aliens" changed to not change vector pointer when it is vector 0
;
; 23/9/02	init_alien can't handle different dragon head unless another instance of segment is
;		created as another object. The pat_lead can only point to next object along. EG At
;		moment segment is next along from stage 1 dragon head, not stage 3 dragon.
;
; 11/6/02	Changed init_alien to set any aliens that are not lead or sum ones to not move when
;		the rotate_status is on. Their movement must be determined by the rotate code. They
;		just get pointed to the special vector buffer 0.
;
; 7/6/02	Changed workings of control for scenery objects. control=0 means no object required
;		rather than a -ve number. After experimenting with bit 7, bit 0 will be used to
;		represent 3-in-a-row. Multi-object bit added to scenery control. To allow 3 different
;		objects to be randomly selected in the same scenery event, eg 3 different mushrooms.
;
; 3/6/02	Noticed bullets never bounce downwards on arcade version. Changed iaw.
;
; 1/6/02	Problem where stage 3 test was crashing on end game return to title screen was found
;		to be due to the heads (which share a slot) not being next to each other in the
;		object_to_slot table. Because they were separated in this table the remove_object
;		routine thought they were different objects and removed them twice, corrupting object
;		stack. Very important lesson - shared slot objects must be next to each other.
;
; 31/5/02	Stage 1 object data now all converted to new sprite format. Changed file_same flag
;		to skip file_load and new create_object if the object is the same - it should not
;		be running create_object when they're the same (machine crashes). Made dragon with
;		red colour use same file as green colour one. Palette needed changing to match the
;		frame bits used for the green one. Different bits get chosen on the old red one.
;		Fixed line_lo-1 to be in byte before $2000 as was intended rather than $a000
;
; 28/5/02	All graphic data eventually changed again 5 days ago. Then mammoth coding and
;		debugging sessions on the internal object convert the rest of the time. Today actually
;		has worked with real data. The bush works OK (I think). Now to move the objects over
;		to the new format, one at a time. load_object slot*11 code moved around a bit to
;		avoid a nasty variable conflict. create_object creates data that is exactly the same
;		as the original load file data from a smaller data file (it calculates the shifts)
;
; 20/5/02	Last several days spent trying to do object convert internally. Not going well. Sprite
;		width increase and shifting done, but convert still glitchy. Just found the way of
;		checking whether mask is required can't work due to the stupid way I've packed the
;		4-colour object data - I have bak as 11 (and black as 00), but also uses zero byte
;		to denote blank data - which can't possibly work in sprite format. All graphic data
;		must be changed now to use bak as 00, before I resume convert program
;
; 14/5/02	Work on addition to scenery handler to do y sine wave for oranges. Had to change how
;		low and high bits work on scenery as head uses these bits and is scenery too.
;
;		Put in clear of all GRAFP registers to try prevent PM glitch during stage loading
;
; 12/5/02	Sum pattern was not being reset after game over, causing it to stop itself at the
;		beginning of next stage and messing the active pat count.
;
;		Man flash not being reset at init_stage causing PM glitch if die just before end stage
;
;		Fix sum pattern animate to check for sum pattern
;
; 11/5/02	Stopped decode_segment from reducing active pat count. Done in aliens now to give
;		accurate pat_kill figure to stop pattern ending too soon. Missing sec in bullet_hit
;		could have given wrong pat_kill figure sometimes as well.
;
;		Stop sum pattern from having animate on multi-hit because using lead object -
;		now no sum animate allowed at all
;
;		kill aliens that go -ve z rather than just clip them - for stage 2 boss heads
;
; 9/5/02	When no object in Sum pattern it doesn't work because pat_kill is zero.
;		Set it as non-zero in init_pattern when there is a sum pattern, should be ok?
;
;		Bug in sum pattern setting 2d position messing up firing position because it was
;		not including the y tilt. turned out to be no need for setting 2d position.		
;
;		Remove optimize from day before on sum pattern - doesn't work due to bit clash
;
; 8/5/02	Remove clear carry at end of y_divide. It has to be cleared on return anyway.
;
;		*Attempt* to fix serious problem with pattern handler. If the decoder finishes
;		before the last object launched then the remaining objects won't get launched.
;		Removed reliance on pat_hi being 0 to detect handler done - use pat_hit instead.
;
;		Slight optimize of sum pattern check for lead object and for dead object
;
; 5/5/02	Assumed a clear carry incorrectly in move_aliens. bcc changed to jmp
;
;		Crashes on removing shared slot object. Put code in remove_object and init_stage
;		to use slot_no as a check for trying to remove twice. Also was a bug in load_object
;		which was not setting the shared slot correctly causing a crash too.
;
;		Moved man bullet fire position down slightly to be more accurate to arcade not
;		being able to hit low pattern objects from ground
;
; 30/4/02	Top grid overlay code on background draw. much more self-modifying code needed.
;		More routines had to be moved to $8000 to make room.
;
; 28/4/02	Optimized draw_background setup slightly. Unrolled and used registers better.
;		Then redid whole routine with self-modifing code. Slightly slower setup, but
;		got back over 500 cycles in draw. Had to move copy_background from $3000 area
;		to $8000 to make room. Noticed that top grid overlay won't work. Re-think clear.
;
; 22/4/02	Use vbi_count as music beat timer instead
;
; 20/4/02	New division routines - non-restoring divide is quicker. Optimized for signed nos.
;
; 9/4/02	Demo version for testers
;
; 6/4/02	Dragon music would not initialize in game commands. Was found to be bug with game
;		command not resetting game_status to zero, making it continually initialize.
;
;		Intermittent bug when killing dragon - didn't die right. Maybe due to ftimer_buffer
;		zero entry not set to zero. Bug only seen once though. Can't recreate.
;
;		On init_stage player garbage appears when loading sometimes. Set 559 to no players
;
; 1/4/02	Re-arrange disk overlay to include required_song for stage. Take off 1 byte from
;		sky_pos, that couldn't be used anyway. stop_sound created. make sound_hi zero
;		when end of stage so sound fx not resumed after end stage. 
;
; 28/3/02	Temporary pause command put in for screen shots etc. when press option key
;
; 27/3/02	On real atari, setting SKCTL to 0, rather than 3 causes RANDOM to return 255!
;		Causing non-random scenery when samples played. New AtariWinPlus 3 highlighed problem.
;
;		Move man_die player draw continue into play_sample routine for neatness
;
; 26/3/02	Bug fix in music player - next treble pattern must reset bass and mid timers to 1
;		for problem of syncing timers on a pattern channel that has been off.
;
; 23/3/02	Bug fix in music player - next treble pattern must reset bass and mid loop counters.
;
;		Make addr of music pattern=0 mean ignore - to stop problem where otherwise to not
;		use the channel means to keep having to set it to 0 volume in the pattern, which
;		would mess up a sound effect that could be using the channel.
;
;		Make man_die resume sound work. play_sounds now zeros sound_hi when fx done. 
;
; 19/3/02	Bug fix in music player - not stopping sound fx. sound_stop-1,y not sound_stop,y.
;
; 16/3/02	Added extra voice to music player - must be used sparingly, as shared with sound fx.
;		Changed name of background draw labels to not conflict with music note labels.
;
; 15/3/02	Sound glitch on warble - changed wraps of pitch offset to use 12 rather than 13.
;
; 10/3/02	Bug fix in init_sound. VBI play could interrupt setup so sound must not be enabled
;		until all the sound parameters have been configured.
;		Music player proceeds poorly - can't debug, and getting screen glitches coz VBI
;		is overrunning too.
;
;		VBI code rearranged to stop glitches. dlist address not getting set early enough
;		in VBI. Moved move_man, music_patterns, play_sound to end of vbi. The check on
;		man_status now has to be done again at end of VBI - a bit messy.
;
;		Maybe finished music player. loops work. next pattern works. song loop works.
;		Whether any good music can be done with it is another matter though.
;
; 2/3/02	Feeble attempt at music player started.
;		Pattern data moved to after $4000-$7fff code to make room for music code
;		and data which has to run in non-bank ram as it is used in VBI while main code
;		could have it switched. 
;
; 24/2/02	init_pattern changed to not use a handler till pat_kill is zero to stop problem
;		with handler decoder finished, getting reassigned quickly and then all the aliens
;		being killed from 1st pattern assigned. Since handlers are rotated as assigned
;		this is a highly unlikely problem, but it was still just possible.
;		"aliens" changed to reduce pat_kill as aliens finish pattern.
;		reset for pat_kill put in init_game
;
;		Problem with fire straight away on pattern getting wrong y pos. This is to do
;		with init_missile having to be done before move_alien (animation frames are
;		the reason why done first). Accept it as a limitation of the pattern handler.
;		A slight pause segment can be put in start of a pattern to work around.
;		Remove initialization of 2d position in init_alien as the previous workaround.
;		It doesn't include the grid tilt which is why it gets wrong y pos, and it's not
;		worth puting it in.
;
;		Optimize pat_lead so -ve when not in use rather than 0, and having to add 12.
;
; 23/2/02	Bug in bullet_hit if all aliens in pattern were killed and decoder finished the
;		pattern - bullet_hit would kill pattern again resulting in wrong pattern count.
;		Bullet_hit changed to check if pattern finished already.
;		Reset active_pat_count at init_game rather than setup.
;
; 22/2/02	Reset last_alien in do_init_drag1 for correct priorities.
;		Reset fireball_speed after man_die
;		Remove step_count in "missiles" use z -ve to check for missile stop. This
;		because sometimes not enough steps to get missile all the way to front.
;		Had to add offscreen bottom check for fireball missiles.
;		Make fireball speedup a pattern command. Remove from pattern decoder loopback.
;
; 20/2/02	Fix to "missiles" not remembering missile speed due to call to init_sound
;
; 17/2/02	Missing clc in "missiles" causing wrong frame of missile.
;		pat_lead object number s/be number less 12 in "init_missiles"
;
; 16/2/02	Bug with pat_lead. Was getting reset by follow objects in init_alien.
;		Bug in red_pattern_aliens was not using control to detect lead alien.
;		Add sound fx for explosions in kill_control_alien
;
; 10/2/02	Add 3d conversion for death of 2d bounce (indestructible) objects.
;		renamed init_alien_explosion to kill_control_alien.
;		renamed kill_pattern_aliens to init_kill_control_aliens.
;
; 3/2/02	Added pattern command code to run things such as swap priorities.
;		Added priority and fireball reset to init_pattern for stage 18 bosses.
;
; 29/1/02	Added priority_status to sort to allow swap sprite priorities
;
; 27/1/02	Changed man x limits. must be a gap on side of screen, or patterns going
;		offscreen look wrong. move score displays inward slightly.
;
; 26/1/02	To allow deduction of (red frame) multi-hit object number init_alien
;		changed to not increase pat_object if there are no follow objects -
;		because wi-wi jumbo must use lead_bit to allow red frame. Dragons are
;		only other multi-hit objects to need animate red frame. bullet_hit now
;		changed iaw above to make all a patterns objects next object number up.
;
; 23/1/02	Used last pattern properties bit for multi_hit - choice of 2 countdown
;		values before multi_hit object dies. Was always same number before
;
;		Pat_multi_hit created for amount of hits left after multi_hit object
;		has changed colour before it dies. Changed init_pattern. More work on
;		bullet_hit is required to change all patterns objects frames to red
;
; 22/1/02	Swapped around bullet_hit code to make more room for pat_kill routine
;		changes soon. Slightly optimized bullet_hit - no high/low bit mask needed
;
;		Put scenery scoring round right way after 31/12/01 swapping high/low bits. 
;
; 19/1/02	Removed un-needed z_scroll_hi table, always zero.
;		Recoded move_aliens top/bottom/middle into smaller code.
;
; 16/1/02	Add init_sound to init_missile routine.
;
;		Bug in bullet_hit. Can't tell aliens from scenery since 31/12/01 changes.
;		Fixed by checking object number, rather than top+low properties.
;
; 9/1/02	Done bounce bullet except for sound fx. Vector values may need tweaking.
;
; 8/1/02	Start on bounce bullet routine. Amendment to how property set for bullets.
;		Changed nokill_bit to bounce_bit, to clarify meaning for bullets.
;
; 7/1/02	Changed missile y jump to 4 from 8. Gives smoother result, but still quick.
;		Also gives more accurate final position, to avoid off screen bottom errors.
;		Mod on init_missiles to allow different speeds for types of missile.
;
;		Made "triad" pattern smaller managing to use repeat function of handler. 
;
; 5/1/02	Working on triad animation, bug in "aliens" was not setting the properties of
;		the new animated object to the alien - triad as unkillable when closed to 
;		vunerable when open was not working.
;
;		Bug in init_missile. Was using handler+12 when should have used handler.
;		Causing missile not to fire always because getting wrong pat_lead value.
;
; 31/12/01	Put in code for y position as top, middle or bottom of alien. Swapped top_bit
;		and low_bit to make code more efficient - meant had to swap scenery_y data too.
;
; 29/12/01	Putting back upto 20 object types per stage meant moving things around to make
;		room; vector buffers and old positions swapped around, *48 and *11 tables
;		extended. Surprise bug in *48 data on byte 15 - was +14 instead of *14.
;
;		Added routine to init_missile and decode_segment to be able to speed up
;		missiles as player takes longer to kill infinite repeat patterns.
;		Default speed of missiles can now also be set. x & y move upto 8 pix/s now.
;
; 28/12/01	Small optimize in init_missile. sound player done without extra audctl stuff
;		a couple of days ago. Test sound effects created for ships, bullet and triads.
;
; 24/12/01	Bug - wasn't stopping pattern handlers after game over.
;
; 20/12/01	Past few days spent working on sounds player. Pitch and volume auto add/subtract
;		segments done, but had to add looping pitch offset table to be able to give
;		various vibrato and warble effects.
;
; 11/12/01	Bugs due to sort index-1 byte being wrongly ammended somewhere between
;		01/09/01 and 12/11/01. The missiles weren't always firing because of
;		pat_lead being overwritten with wrong values.
;
; 10/12/01	Using top of object as y position for ships looks better than centre or bottom -
;		try it for all alien objects.
;
;		Problem with pattern address offset - 2k won't be enough. 4k would be, I expect.
;		Can we use some of the other bits to encode more offset? No. Too ugly. Expanded
;		the stage overlay to use 6 command bytes instead of 5. Of course this meant
;		recoding the "init_pattern" decoder. Can also have only 47 patterns per stage now
;		without giving the overlay file another 128 bytes.
;
;		Redone top of display list. DLI can't start on 1st line of graphics so a gap of
;		one line with no colour was there. Making the 1st line 1/2 height looks better. 
;
; 8/12/01	At last, missile stuff finished; smoothing of Bresenham line draw, selection of 
;		different kinds of missile by properties of firing object, and lead object firing
;		for following objects.
;
; 4/12/01	Just noticed screen clip stopped working, crashes when offscreen. Eventually found
;		to be a mistaken optimization on "priority". The initial jump address for sprite
;		must be set back to the clip routine address for every sprite. Can't be set once.
;
; 2/12/01	Fix to init_alien to set 2d position as well as position - needed if missile fired
;		right at pattern start
;
; 24/11/01	Optimizing sort routine, made z pos -ve to mean hide as well as the hide property.
;
; 18/11/01	Removed explosion routine. Incorporated explosions into aliens and scenery for speed
;
;		Start replacing jsr/rts (12 cycles) with jmp/jmp (6 cycles) in code loops
;
; 17/11/01	2d co-ords changes completed. New table for explosion ground check was required.
;		Centre of screen in 2d co-ords is x=128,y=90.
;
; 14/11/01	Remove y mirror command from aliens. it is not used much in real game, and would
;		need an additional y offset to be specified to work like real game. Changes were made
;		to the values of pat_properties and control bits to remove the option.
;
;		Start work on making x get converted to 2d co-ords before draw so that a non-negative
;		more efficient collision check can be done (x co-ords get too big when +/-)
;
; 12/11/01	Fast line draw being investigated for missile firing is holding up development.
;
;		Optimize slot address lookups in frames - have each slot address predefined
;
;		Moved "frames" code into "Priority". no need for frame addr and bank for objects
;
; 2/11/01	Small optimize in "priority" and removal of extraneous "dey".
;		line up tabbed comments again - messed up after upgrade to winXP which uses notepad.
;
; 28/10/01	Start work on alien firing routine.
;		z_pos_lo size changed from 24 bytes to 18 - not required by bullets. offsets changed
;
; 24/10/01	Small fix to turn off volume only on the sound channel after playing sample.
;		Small change in making scenery more random. Put 1 in 2 chance of no setup
;
; 23/10/01	Change in palette data and all object data, due to new palette sharing method.
;
; 20/10/01	Serious bug on remove object. Locks up often, despite previous testing.
;		Eventually found to be problem in slot_insert. Stack0 object address being updated
;		incorrectly, due to a typo.
;
; 18/10/01	y mirror patterns are going to need a y offset. y=0 is not far from the top of the
;		mountains here, but on real game is much higher in sky. Not fixed yet. Need to think.
;
; 17/10/01	Bug in y position of aliens. Should not make y bottom of object, but the centre.
;		Still looks kind of wrong compared to original - Tilt of zero is when man on ground
;		in original game. This would be perverse the way I've written my code. Try work round.
;
; 16/10/01	Split disk file data off into separate file for stage referrence. Ship graphics done
;
; 8/10/01	Changed organisation of disk overlay. Reduced size of stage 0 overlay.
;
; 7/10/01	Setup code for title screen. Added routine to load_background to create a blank
;		background for the title screen
;
; 29/9/01	Bug in pattern decoder fixed where loop counter not reset after first use in pattern.
;
;		Start re-organizing setup code into first time only setup, game intialize code and stage
;		initialize code. General rationalisation of setup code.
;
; 26/9/01	Testing on spread_bit pattern function successful
;
;		Changed .incbin to use relative file names. Found it can do it if no slash is put on the
;		front of the directory. A ".\" is not recognised though. The working directory of the
;		shortcut to atasm is very important - the "atari space harrier" directory is the working 
;		directory and everything is relative to this now.
;
;		Changed the disk data objects such as backgrounds, stage data and objects to have their
;		own directory. They were starting to clutter up the main directory. Also changed them
;		to take advantage of the relative file names with .incbin.
;
; 25/9/01	Master sequencer mostly ready for testing.
;
; 22/9/01	Work started on master sequencer.
;
;		Moved wait_second reliant code (scenery, and now sequencer), into one routine in main
;		loop.
;
; 17/9/01	fixed bug with stripping fire bit in pattern decode.
;
;		Added pat_lead to store object number of the lead object. Changed sum pattern to use
;		the lead object so knows which object to set the timer of, rather than checking the
;		sum bit of all the objects - A bit more elegant maybe. NB Sum pattern must now always
;		use lead_bit if it has an object of its own.
;
; 16/9/01	Merged Notes and Bug fixes (these comments).
;
;		There will be a bug when alien firing is done. The decoder doesn't strip the fire bit
;		from the timer for the pattern (it must leave it for the buffer though).
;
;		Bug in sum pattern where centre alien won't die quick coz its timer not reset. Fixed,
;		but maybe not in a very elegant way. Maybe sum pattern death should go in bullet_hit
;		and try and use the same routine as the control pattern kill?
;
; 12/9/01	Preliminary alien hit and pattern routines still being developed. Testing going better 
;		than might be expected - Most functions seem to be mostly working. Control patterns
;		going quite well. Just problems with the die timing. But, still have sum pattern to test, so
;		there may be a whole bucketload of problems to resolve.
;
; 6/9/01	Optimize completed on tilt. Back to pattern handler now. Look at alien hit stuff
;
; 5/9/01	Completed a working self-modifying clear screen routine, as per earlier notes. I think
;		it can be optimized slightly though, to take advantage of the constant screen offsets
;		between the a and b screens now. NB, it is part of the "tilt" routine
;
;		Good News. Another 4k memory mislabel has given me back $9000-$9fff. I'm back to the
;		amount of memory I thought I had originally.
;
;		Changing memory map because of this, and to get all a and b screens the same offset
;		from one another, which has resulted in some small code improvements.
;		$2000-$2fff changes from program to screen1b, so it can follow screen1a in $1000-$1fff
;		$2000-$2fff code moves to $3000-3fff which had yet to be filled.
;		$a000-$afff changes from screen1b to program/data
;
; 4/9/01	Back of grid didn't always move up or down a whole pixel every frame. Adjusted the
;		initial value of tilt_lo to be 128, so that tilt by 255/256 gives a movement of 1 in
;		the tilt_hi for the full up/down scroll range. Had to tweak initial z_scoll values
;		in order for the horizon of grid to look right when at lowest range.
;
; 3/9/01	Tilt routine moved from after new_screen, to just before clear_screen. This to enable
;		knowing where background is going to be, to not waste time clearing its screen area
;
;		More elegant way of resuming game after man dies. Just need to resync the dlist_screen
;		and the draw_screen, to reset any outstanding frames, rather than trying to move back a
;		frame, erasing and redrawing. The old method had problem when frame rate misses caused
;		the displayed screen to be erased and redrawn in plain view, glitching badly. New way
;		fixes that.
;
; 2/9/01	16 frames per second is not realistic. It may hit this with just scenery going, but it
;		will slow when just one or two large frames of aliens or missiles are about.
;		This is a major limitation with having 2 screens for extra colours. Try 12 fps.
;
;		Bug of man dying when bullet over offending object fixed (again). Was due to move of
;		sort table to an area that is normally ROM. sort_table-1 has to be set at 0, and
;		this had been done with .byte. When program loaded, the area is still ROM, so it
;		doesn't get the value. That meant the collision detection for objects with z=0
;		checked the wrong stuff "behind" them. Changed to make initial setup write the value.
;
;		Bug with object flicking into front of screen from nowhere. Happens even when only
;		scenery is running. Must be something to do with scenery then. Maybe x range.
;
;		Bug is actually to do with priority routine changes. object position includes
;		its centreup now, and it can go beyond the hidden left edge of the
;		screen (-95). Before only the centre of the object got dealt with, so the -95 limit
;		would occur less frequently. To get to the centre of screen 96 is added so anything
;		below this really screws up. To fix it, have allowed x to go off screen by 36 pixels
;		(9 bytes). A new table has been created for times 48 minus 9 to get the screen
;		positions quickly. The existing times 48 table reduced in size. Changes to  background.
;
; 1/9/01	Still doing pattern handler. Even infinite repeat Patterns now should kill themselves
;		when their objects are all dead. This code need putting in the object hit routine. It
;		just counts down each object killed until zero left. This allows the sum pattern to be
;		able to check easily if any objects are alive. It just needs to check if any patterns
;		are left. The sum pattern should be allowed an object of it's own, which it's
;		responsible for killing.
;
; 31/8/01	Old position buffer for man collision check had wrong offsets. Caused crashes when
;		more than 4 objects at z=0. Thought it was pattern handler problem initially
;
; 28/8/01	Moved background data to bank 3 to free memory. Done some optimizing of priority routine
;
; 27/8/01	Bad news. 4k I thought I had does not exist! There was a typo on the memory position
;		markers for $0700-0fff. It was set to show $0700-1fff. This was the space where I was
;		going to put alien pattern data. Will have to move background data to one of the memory
;		banks. This is 3k.
;
; 25/8/01	Idea for improving clear screen. Already know there's no need to clear space where 
;		background is drawn. But if self modifying code is used, the clear screen can be 
;		smaller as well. Make code able to clear screen_height-background_height, and alter
;		the location in clear_screen either side of where background is. Has to be reset for
;		next clear of that screen
;
; 19/8/01	Change of plan for patterns. Instead of every alien decoding its pattern, have a
;		pattern handler to do that. Saves overhead of decoding. Only need 4 pattern
;		handlers, I reckon. Oh, and a special one for Sum patterns 
;
; 11/8/01	Scolling background completed a few days ago. Toughest part was to do the 2 pixel
;		shift version of the background for smoother scroll, in a small routine.
;		Started the alien pattern routines. Needs a bit of a rethink from earlier routine.
;		The sequence list for the stage should use more memory as it is overlayed each stage.
;		Memory can then be freed up in the patterns themselves, which are memory resident.
;
; 6/8/01	Re-draw background after man dies, as per objects. Changed to use stack to save
;		current screen pointers, as the zero page variables were used for background draw
;
; 3/8/01	Background wasn't scrolling soon enough when man moved left/right. Changed screen
;		back to divided evenly into 5 sections per x_scroll and man frame. Was previously
;		in 6 sections with 2 for the middle.
;		Scroll looks right, but man middle frame doesn't seem to be used enough. Needs work.
;
; 29/7/01	The end of game code needs writing to reset the game variables correctly.
;		The setup routine is run at the moment and that doesn't clear the dlist dlis.
;		However it may be quite possible to not have to re-initialize everything.
;
; 28/7/01	first notes apart from the stuff about the size of the objects compared to the
;		original and the memory map.
;		Scoring routines and display mostly finished last week. Work to be done when
;		scoring hits of spinning head objects and dragon segments. They are worth 100,000.
;		When the pattern handler is written, check if object has a control object
;		Current task is the scrolling background.
;
; 28/7/01	Amount of storage for old position check reduced to 12 objects at z=0.
;		Put in constant to easily change and allocate memory space
;
; 27/7/01	Bug in man_die on game end. PMG glitch in dead man sometimes. Due to rest of main
; 		loop being run before game exited - allowing dead man player_b area to be written
;		with screen data and PMG area not to be reset to normal.
;		Solution was to make code exit main loop straight after last man die.
;
; 26/7/01	Bug in man_die. PMG glitch on 1st frame of man death.
;		Initialization of player_index was wrong. Was not set to do a code fetch first.
;		Was 9 which sequencer counts down to 8. Should be 0 which sequencer wraps to 9.
;
; 25/7/01	Started trying to share zero page variables where possible. This may lead to bugs
;		particularly in complex routines and VBI where shares could be used twice. Take care.
;


;
; graphic sizes compared to original:
;

; original has double the resolution, and I have made the Atari screen smaller
; with the objects slightly relatively smaller for a faster refresh rate.
; E.G. A full size tree is 179 pixels. Atari tree is 72 pixels. 
; This is 72/179 or about 40-41% of the original size.

; carefully calculated "real 3d" frame sizes
; 100%, 80%, 67%, 57%, 50%, 44%, 38%, 33%, 27%, 22%, 18%
; but try these - this should make pop up less apparent on tall scenery
; and the graphics are so chunky the slight size differences don't matter.
;
; 100%, 80%, 66%, 57%, 50%, 44%, 37%, 29%, 23%, 19%, 15.5%
;
; 96*96 sizes scaled:- 96, 76.8, 63.4, 54.7, 48, 42.2, 35.5, 27.8, 22.1, 18.2, 15
; there are 11 frame sizes for each object
;


;
; xl/xe memory map
;

; (with dos/dup $0200-$3306 is used)
; (with dos $0200-$1fff is used. you can't load lower than $2000 with dos routines)
; (with just the os $0200-$700 is used)
; (without the os you can do what you like anywhere!)

; assuming no os:

; $0000	-	$00ff		zero page
; $0100	-	$01ff		stack
; $0200	-	$07ff		free
; $0800	-	$0fff		screen1a clip area
; $1000	-	$2fff		screen1a
; $2000	-	$27ff		free
; $2800 -	$2fff		screen1b clip area
; $3000	-	$3fff		screen1b
; $4000	-	$47ff		free
; $4800 -	$4fff		screen2a clip area
; $5000 -	$5fff		screen2a
; $6000	-	$7fff           free	
; $8000	-	$bfff		free/16K Megacart Bank
; $c000	-	$cfff		free (but normally rom)	
; $d000	-	$d7ff		hardware registers
; $d800	-	$dfff		screen2b clip area
; $e000	-	$efff		screen2b
; $f000	-	$ffef		free


;
; program start
;


	.opt no err


;
; Atari Equates
;

ATARI=0					; 8-bit series computer (1 = 5200 console)

	.include equates.asm


;
; constants
;


gamma_adjust=4				; brightness default
drive_number=3				; drive number to load data from. (development=3, real game=1) 
start_from_stage=5			; stage number to start from (for testing purposes)
max_lives=7				; initial amount of lives man given

max_objects=100				; max objects in whole game
max_stage_objects=22			; max objects allowed per stage (if changed, check overlay)
vector_buffer_size=10			; size of pattern handler vector buffers
max_handlers=5				; most simultaneous pattern handlers
max_collision_objects=12		; most objects to check in collision with man (z=0)
z_clip=10				; maximum z clipped objects
max_dlis=50				; maximum dlis allowed
overlay_size=5				; overlay size in disk sectors (128 bytes per sector)

norm_fireball_speed=16			; normal fireball speed (lower nos are faster)

man_die_bit=~10000000

top_grid_on=~00111111			; top grid control
top_grid_up=~10000001
top_grid_down=~01000001


;
; zero page variables
;

	*=$80

; permanent zero page variables


dlist_index		.ds 1		; video
draw_screen_index	.ds 1
flip_count		.ds 1
frame_count		.ds 1
vbi_count		.ds 1
game_speed		.ds 1		; target number of VBI's per game frame
draw_screen_a		.ds 1
clip_screen_a		.ds 1
sprite_count		.ds 1

man_x_pos		.ds 1		; man
man_y_pos		.ds 1
man_y_stop		.ds 1
man_run_frame		.ds 1
last_button		.ds 1
bullet_count		.ds 1
man_status		.ds 1
man_lives		.ds 1
lives_status		.ds 1
man_flash_timer		.ds 1

colour_buffer_read	.ds 1		; colour buffer pointer

scenery1_pointer	.ds 1		; scenery
scenery2_pointer	.ds 1
scenery1_timer		.ds 1
scenery2_timer		.ds 1

last_alien		.ds 1		; patterns
rotate_status		.ds 1
active_pat_count	.ds 1
sum_status		.ds 1
pattern_command		.ds 2
pattern_temp		.ds 1
game_command_argument	.ds 1
pattern_command_reqd	.ds 1

sequencer_pointer	.ds 1		; event sequencer
sequencer_timer		.ds 1
last_handler		.ds 1
game_status		.ds 1

last_missile		.ds 1		; missiles

x_scroll		.ds 1		; misc
wait_timer		.ds 1		; time to sync down to sequencer clock
horizon			.ds 1		; y position of horizon
background_x		.ds 1
background_y		.ds 1
score_status		.ds 1		; flag for score over high score
stack_pointer		.ds 1		; stack pointer for memory banks
grid_index		.ds 1		; grid animation index
stage_no		.ds 1
max_vcount		.ds 1		; holds maximum vcount for PAL or NTSC machine
sample_status		.ds 1		; sample playing flag. Only critical code has time to be run
player_index		.ds 1		; man die time slice index. Splits critical code on sample
priority_status		.ds 1		; flag for sprite priorities swap.

music_pat_index		.ds 1		; music
music_source		.ds 2
sound_queue		.ds 1		; sound fx

top_grid_status		.ds 1		; grid/sky
sky_scroll		.ds 1
sky_index		.ds 1
top_grid_pos		.ds 1
grid_colour_index	.ds 1
grid_colour_timer	.ds 1
grid_colour_swap	.ds 1

uriah_status		.ds 1		; uriah
counters_small		.ds 1
counters_big		.ds 1


; temporary zero page variables used sometimes in VBI and DLI - take care trying to share them

vbi_temp
dli_temp		.ds 1

sprite_width
free_mem
pix_a
man_die_index
man_left		.ds 1
sprite_shift
pix_b
man_right
player_timer		.ds 1

sprite_height
mem_remain
page_refresh
pix_change_count
man_top
player_offset		.ds 1
add
y_refresh
pix_index
man_bottom
player_add		.ds 1


; other temporary zero page variables

line_offset
player_source
mem_reqd
jump			.ds 1
			.ds 1

temp			.ds 1

next_vcount		.ds 1

colour1
sample_nibble
dividend
multiplicand
temp2			.ds 1

counter_y_stop
colour2
multiplier
temp3
data_buffer_size
sample_index
divisor			.ds 1

source_a
last_object		.ds 1
			.ds 1

source_b		.ds 1
			.ds 1

dest_a
bullet_left
scenery_left		.ds 1
bullet_right
scenery_right		.ds 1

dest_b
player_dest
bullet_top
scenery_top		.ds 1
bullet_bottom
scenery_bottom		.ds 1

sample
table1			.ds 1
			.ds 1

table2
player_frame		.ds 1
player_pos		.ds 1

frame_no		.ds 1
object_no		.ds 1

slot_no			.ds 1
bank_no			.ds 1

zero_count
sect_count		.ds 1
last_properties		.ds 1

last_sector_lo		.ds 1
last_sector_hi		.ds 1

last_palette		.ds 1




mem_position_ZP


;
; variables/generated tables/buffers/overlay area
;

	*=$600

mask_table 		.ds 256


;
; stage overlay area - exactly 640 bytes (5 sectors)
;

	*=$700

stage_overlay


; song position required at stage start

song_required		.ds 1

; default VBI ticks per frame

stage_speed		.ds 1

; uriah bonus stage (when non zero). Also determines overlay set required on uriah stage

uriah_stage		.ds 1

; sky dli positions and colours

sky_pos			.ds 40
sky_colour		.ds 40

; objects required for stage

objects_required	.ds max_stage_objects

; grid colours and colour cycling

; start should be 1 more than table entry required as a DEC is done on the fetch

grid_colour_start	.ds 1
grid_colour_wrap	.ds 1
grid_colour_pause	.ds 24
grid_colour1		.ds 24
grid_colour2		.ds 24

; scenery stream 1

stream1_control		.ds 40
stream1_timer		.ds 40

; scenery stream 2

stream2_control		.ds 40
stream2_timer		.ds 40

; sequencer stream

; byte 1 - timer. timer is in seconds/2 (129-255 means wait until active patterns dead)
; byte 2 - properties (when no object then pattern command required)
; byte 3 - pattern offset lo (when no object then game command required)
; byte 4 - bit 0-3 - pattern offset hi
; byte 4 - bit 4-7 - object qty (NB sum patterns are allowed 0 objects) 
; byte 4 - (when no object then game command variable)
; byte 5 - object type (0 means no objects - a non-pattern game command)
; byte 6 - object spacing 
; byte 6 - also bit 2-7 - frame die hit count (when a hit count pattern)
; byte 6 - also bit 0-1 - qty of die animation frames (when a hit count pattern)

; byte 1

stream_timer		.ds 50

; byte 2

stream_properties
stream_pat_command	.ds 50

; byte 3

stream_pat_lo
stream_game_command	.ds 50

; byte 4

stream_pat_hi
stream_qty
stream_game_argument	.ds 50

; byte 5

stream_object		.ds 50

; byte 6

stream_spacing		.ds 50



;
; end of stage overlay
;

stage_overlay_end


	*=$700+[overlay_size*128]		; allow x disk sectors space for overlay		; 


; grid_colour can be within overlay, it is only a temporary table 

grid_colour		.ds 3


; times 48 minus 9 table lo

x48min9_table_lo	.ds 142


; sprite objects frame addresses

frame_addr

; NB hi address. To allow address to fit into 2 bytes including mem bank
; b object address is derived, the a address only takes 0-8k of bankm, and b
; object is put at the same address in the 8-16k part of the same bank. Therefore
; the a address only is 0-8k which requires 8+5 bits to describe. there are
; 5 banks requiring 3 bits to describe. b object address is just a object address
; plus 8k
; bits 0-4 hi address less mem bank offset ($40).
; bits 5-7 memory bank. main bank is ~100. extras are ~000,~001,~010,~011 

; EG up to 20 object would be  2 (lo,hi addr) * 20 * 24 (frames)

	.ds 2*max_stage_objects*24




; free memory tracker for each bank

next_mem_lo		.ds 4
next_mem_hi		.ds 4


; address of memory in each bank after every object load
; slot number of each stack entry

stack_slot		.ds max_stage_objects

stack0_lo		.ds max_stage_objects
stack0_hi		.ds max_stage_objects

stack1_lo		.ds max_stage_objects
stack1_hi		.ds max_stage_objects

stack2_lo		.ds max_stage_objects
stack2_hi		.ds max_stage_objects

stack3_lo		.ds max_stage_objects
stack3_hi		.ds max_stage_objects

; slot usage

slot_usage		.ds max_stage_objects
	
; object to slot mapping

object_to_slot		.ds max_objects


; sound variables

sound_timer		.ds 8
sound_index		.ds 8
sound_stop		.ds 8
sound_lo		.ds 8
sound_hi		.ds 8
sound_offset		.ds 8


; music player

music_lo		.ds 5
music_hi		.ds 5
music_timer		.ds 5
music_index		.ds 5
music_loops		.ds 5


;
; mem remaining help. put all 0600-0fff data before this
;

mem_position_0600-0fff


;
; screen data and loading buffer
;

	*=$1000

screen1a
object_buffer				; temp buffer whilst decoding frame data


	*=$1fff

	.byte 0				; to allow line_lo-1 is less than or equal line_lo


	*=$2000

screen1b

file_buffer		.ds 3072	; temp buffer whilst decoding object file data

line_lo					; used as temporary object optimize space
colour_table_a		.ds 256		; and temporary colour decode space

line_hi
colour_table_b		.ds 256

object_size_lo
pix_swap_a		.ds 4

object_size_hi				; object_size is a temp used when deleting objects
pix_swap_b		.ds 4		; pix_swap is a temp used when building colour table



	*=$b000

screen2a

line_width				; used as temporary object optimize space
frame_buffer_a		.ds 1536	; and temp buffer for result of frame decoding

line_remain
frame_buffer_b		.ds 1536

mirror_table		.ds 256		; used for quick mirroring of sprite data
data_buffer		.ds 14		; line data buffer for frame encoding


;
; variables/generated tables/buffers
;

	*=$d800

;
; buffers for dli positions and colours
;

; NB triple buffered
 
dli_buffer		.ds max_dlis*3
colour_buffer		.ds max_dlis*3


; pattern handler variables

pat_lo			.ds max_handlers; address of pattern
pat_hi			.ds max_handlers
pat_index		.ds max_handlers; index in pattern address
pat_timer		.ds max_handlers; timer till next pattern segment
pat_loops		.ds max_handlers; pattern loop counter
pat_vector		.ds max_handlers; current vector buffer position
pat_kill		.ds max_handlers; gets set to dead_bit when control object dies
pat_properties		.ds max_handlers; properties of pattern (x mirror)

pat_qty			.ds max_handlers; quantity of objects to start in pattern
pat_object		.ds max_handlers; object required for pattern
pat_spacing		.ds max_handlers; spacing of objects in pattern. high number - far apart
pat_start_timer		.ds max_handlers; used for object start timing
pat_link		.ds max_handlers; pattern number that is to be controlled, linked and killed
pat_die_frames		.ds max_handlers; how many frames of animation there are for dying
pat_frame_hits		.ds max_handlers; countdown of hits required till next animation frame


			.ds 1		; extra byte required for assisting collision detect
sort_index		.ds 21+z_clip	; 21 z positions, + clipped z positions at back of playfield
sort_table		.ds 32+z_clip	; 32 objects, + maximum z clipped objects   


; general sprite object control variables

; num		type
;
; 0-5		Bullets
; 6-11		Missiles
; 12-23		Aliens
; 24-31		Scenery

x_pos	 		.ds 32
x_pos_2d		.ds 32
x_size			.ds 32
y_pos	 		.ds 32
y_pos_2d		.ds 32
y_top			.ds 32
y_bottom		.ds 32
z_pos			.ds 32
slot			.ds 32
properties		.ds 32

; required for bullets and missiles

x_vector		.ds 12
y_vector		.ds 12

; required for missiles

max_length		.ds 6
x_pos_lo		.ds 6
x_length		.ds 6
y_pos_lo		.ds 6
y_length		.ds 6
z_length		.ds 6

; required for missiles and aliens

z_pos_lo		.ds 18		; required to use z vector lo buffer

; only required by aliens

vector			.ds 12
handler			.ds 12
object			.ds 12

; required for aliens and scenery
; NB control different depending on scenery or alien

control			.ds 20
timer			.ds 20

; only required by scenery

y_sine			.ds 8		; position in y sine wave


; pattern handler vector buffers

x_vector_buffer		.ds 1
x_vector_buffer0	.ds vector_buffer_size
x_vector_buffer1	.ds vector_buffer_size
x_vector_buffer2	.ds vector_buffer_size
x_vector_buffer3	.ds vector_buffer_size
x_vector_buffer4	.ds vector_buffer_size

y_vector_buffer		.ds 1
y_vector_buffer0	.ds vector_buffer_size
y_vector_buffer1	.ds vector_buffer_size
y_vector_buffer2	.ds vector_buffer_size
y_vector_buffer3	.ds vector_buffer_size
y_vector_buffer4	.ds vector_buffer_size

z_vector_lo_buffer	.ds 1
z_vector_lo_buffer0	.ds vector_buffer_size
z_vector_lo_buffer1	.ds vector_buffer_size
z_vector_lo_buffer2	.ds vector_buffer_size
z_vector_lo_buffer3	.ds vector_buffer_size
z_vector_lo_buffer4	.ds vector_buffer_size

z_vector_buffer		.ds 1
z_vector_buffer0	.ds vector_buffer_size
z_vector_buffer1	.ds vector_buffer_size
z_vector_buffer2	.ds vector_buffer_size
z_vector_buffer3	.ds vector_buffer_size
z_vector_buffer4	.ds vector_buffer_size

frame_change_buffer	.ds 1
frame_change_buffer0	.ds vector_buffer_size
frame_change_buffer1	.ds vector_buffer_size
frame_change_buffer2	.ds vector_buffer_size
frame_change_buffer3	.ds vector_buffer_size
frame_change_buffer4	.ds vector_buffer_size

ftimer_buffer		.ds 1
ftimer_buffer0		.ds vector_buffer_size
ftimer_buffer1		.ds vector_buffer_size
ftimer_buffer2		.ds vector_buffer_size
ftimer_buffer3		.ds vector_buffer_size
ftimer_buffer4		.ds vector_buffer_size


; z zero collision check objects

old_pos_index		.ds 3

; NB maximum possible amount of objects at z=0 would be 12 aliens, 6 missiles, 4 scenery.
; This can never happen in reality. Also, hidden and harmless objects aren't stored. 
; A compromise of about half this number is easily enough.

old_left		.ds max_collision_objects*3
old_right		.ds max_collision_objects*3
old_top			.ds max_collision_objects*3
old_bottom		.ds max_collision_objects*3


; z tilt tables

tilt_lo			.ds 21
tilt_hi 		.ds 21


;
; mem remaining help. put all d800-ddff data before this
;

mem_position_d800-ddff


;
; player/missile area
;
	*=$dc00

pmbase_addr

player0=pmbase_addr+512
player1=pmbase_addr+640
player2=pmbase_addr+768
player3=pmbase_addr+896


;
; screen data
;

	*=$c000

screen2b

	*=$e000

screen3a

	*=$f000

screen3b


;
; switch in memory banks
;

	*=$4000

bankA

; background overlay (bank 3)

back_overlay

; background A

back_0a			.ds 96
back_1a			.ds 96
back_2a			.ds 96
back_3a			.ds 96
back_4a			.ds 96
back_5a			.ds 96
back_6a			.ds 96
back_7a			.ds 96

; background B

back_0b			.ds 96
back_1b			.ds 96
back_2b			.ds 96
back_3b			.ds 96
back_4b			.ds 96
back_5b			.ds 96
back_6b			.ds 96
back_7b			.ds 96

; background A shifted 2 pixels

back_0as		.ds 96
back_1as		.ds 96
back_2as		.ds 96
back_3as		.ds 96
back_4as		.ds 96
back_5as		.ds 96
back_6as		.ds 96
back_7as		.ds 96

; background B shifted 2 pixels

back_0bs		.ds 96
back_1bs		.ds 96
back_2bs		.ds 96
back_3bs		.ds 96
back_4bs		.ds 96
back_5bs		.ds 96
back_6bs		.ds 96
back_7bs		.ds 96

; mem start of bank 3 after background data loaded

bank3_start


	*=$6000

bankB


;
; data/tables/bank switch sensitive routines
;

	*=$8000

;
; character set
;

charset

; 0
	.byte ~00111100
	.byte ~01000010
	.byte ~10011001
	.byte ~10011001
	.byte ~10011001
	.byte ~10011001
	.byte ~01000010
	.byte ~00111100
; 1
	.byte ~00111100
	.byte ~01100100
	.byte ~01000100
	.byte ~01100100
	.byte ~00100100
	.byte ~01100110
	.byte ~01000010
	.byte ~01111110
; 2
	.byte ~00111100
	.byte ~01000010
	.byte ~10011001
	.byte ~10011001
	.byte ~01110010
	.byte ~11000111
	.byte ~10000001
	.byte ~11111111
; 3
	.byte ~00111100
	.byte ~01000010
	.byte ~10011001
	.byte ~01110010
	.byte ~01111001
	.byte ~10011001
	.byte ~01000010
	.byte ~00111100
; 4
	.byte ~00001110
	.byte ~00010010
	.byte ~00100010
	.byte ~01010010
	.byte ~10110011
	.byte ~10000001
	.byte ~11110011
	.byte ~00011110
; 5
	.byte ~11111110
	.byte ~10000010
	.byte ~10011110
	.byte ~10000011
	.byte ~11111001
	.byte ~10011001
	.byte ~01000010
	.byte ~00111100
; 6
	.byte ~00111110
	.byte ~01000010
	.byte ~10011110
	.byte ~10000011
	.byte ~10011001
	.byte ~10011001
	.byte ~01000010
	.byte ~00111100
; 7
	.byte ~11111111
	.byte ~10000001
	.byte ~10011001
	.byte ~11110110
	.byte ~00100100
	.byte ~00100100
	.byte ~00100100
	.byte ~00111100
; 8
	.byte ~00111100
	.byte ~01000010
	.byte ~10011001
	.byte ~01000010
	.byte ~10011001
	.byte ~10011001
	.byte ~01000010
	.byte ~00111100
; 9
	.byte ~00111100
	.byte ~01000010
	.byte ~10011001
	.byte ~10011001
	.byte ~11000001
	.byte ~01111001
	.byte ~01000010
	.byte ~01111100
; A
	.byte ~01111110
	.byte ~11000011
	.byte ~10011101
	.byte ~10011101
	.byte ~10011101
	.byte ~10000001
	.byte ~10011101
	.byte ~11110111
; C
	.byte ~01111110
	.byte ~11000011
	.byte ~10011001
	.byte ~10011111
	.byte ~10011111
	.byte ~10011001
	.byte ~11000011
	.byte ~01111110
; E
	.byte ~11111111
	.byte ~10000001
	.byte ~10011111
	.byte ~10000010
	.byte ~10011110
	.byte ~10011111
	.byte ~10000001
	.byte ~11111111
; G
	.byte ~01111110
	.byte ~11000011
	.byte ~10011101
	.byte ~10011111
	.byte ~10010001
	.byte ~10011101
	.byte ~11000011
	.byte ~01111110
; O
	.byte ~01111110
	.byte ~11000011
	.byte ~10011101
	.byte ~10011101
	.byte ~10011101
	.byte ~10011101
	.byte ~11000011
	.byte ~01111110
; P
	.byte ~11111110
	.byte ~10000011
	.byte ~10011101
	.byte ~10011101
	.byte ~10000011
	.byte ~10011110
	.byte ~10010000
	.byte ~11110000
; " "
	.byte ~00000000
	.byte ~00000000
	.byte ~00000000
	.byte ~00000000
	.byte ~00000000
	.byte ~00000000
	.byte ~00000000
	.byte ~00000000
; R
	.byte ~11111110
	.byte ~10000011
	.byte ~10011101
	.byte ~10011101
	.byte ~10000011
	.byte ~10011011
	.byte ~10011101
	.byte ~11110111
; S
	.byte ~01111110
	.byte ~11000011
	.byte ~10011101
	.byte ~01000110
	.byte ~01111011
	.byte ~10011101
	.byte ~11000011
	.byte ~01111110
; T
	.byte ~11111111
	.byte ~10000001
	.byte ~11100111
	.byte ~00100100
	.byte ~00100100
	.byte ~00100100
	.byte ~00100100
	.byte ~00111100
; extra life symbol
	.byte ~00111000
	.byte ~00111000
	.byte ~00000000
	.byte ~01111100
	.byte ~01000100
	.byte ~00111000
	.byte ~00111000
	.byte ~01101100


;
; display lists
;

; NB character set: 0-9 is 0-9, " " = 16
; A = 10, C = 11, E = 12, G = 13, O = 14, P = 15, R = 17, S = 18, T = 19 
; extra life = 20

score_line

	.byte 16,16,16,16,16,16					; spacing to visible display
	.byte 16,16						; spacing
	.byte 19,14,15,16					; the word "TOP "

top_score

	.byte 16,1,0,0,0,0,0,0					; the top score itself
	.byte 16,16,16,16,16,16					; spacing
	.byte 18,11,14,17,12,16					; the word "SCORE "
score

	.byte 16,16,16,16,16,16,16,0				; the score itself

lives_line

	.byte 16,16,16,16,16,16					; spacing to visible display
	.byte 16,16						; spacing

lives

	.byte 16,16,16,16					; the extra lives
	.byte 16,16,16,16,16,16					; spacing
	.byte 128+15,128+17,128+12,128+18,128+18,16		; the hidden word "PRESS "
	.byte 128+18,128+19,128+10,128+17,128+19,16		; the hidden word "START "
	.byte 18,19,10,13,12,16					; the word "STAGE "
stage
	.byte 16,16						; the stage number
	.byte 16,16,16,16,16,16,16				; spacing to visible end display


; in-game display lists

dlist
dlist1

	.byte BLANK8,BLANK8,BLANK8,BLANK4+INT
	.byte RLDMSC+2,<score_line,>score_line
	.byte RLDMSC+13,<screen1a,>screen1a
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13+INT
	.byte RLDMSC+2,<lives_line,>lives_line
	.byte JMPWT,<dlist1,>dlist1

dlist2

	.byte BLANK8,BLANK8,BLANK8,BLANK4+INT
	.byte RLDMSC+2,<score_line,>score_line
	.byte RLDMSC+13,<screen2a,>screen2a
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13+INT
	.byte RLDMSC+2,<lives_line,>lives_line
	.byte JMPWT,<dlist2,>dlist2

dlist3

	.byte BLANK8,BLANK8,BLANK8,BLANK4+INT
	.byte RLDMSC+2,<score_line,>score_line
	.byte RLDMSC+13,<screen3a,>screen3a
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13,13,13,13,13,13,13
	.byte 13,13,13,13+INT
	.byte RLDMSC+2,<lives_line,>lives_line
	.byte JMPWT,<dlist3,>dlist3


; stage loading display list

dlist_load

	.byte BLANK8,BLANK8,BLANK8
	.byte BLANK8,BLANK8,BLANK8,BLANK8
	.byte BLANK8,BLANK8,BLANK8,BLANK8
	.byte BLANK8,BLANK8,BLANK8,BLANK8
	.byte RLDMSC+6,<load_msg,>load_msg
	.byte 112,112,112,112,112,112,112,112,112,112,112
	.byte JMPWT,<dlist_load,>dlist_load

load_msg

	.byte 16,16,16,16,16,16,16,16				; 8 blanks
	.byte 18,19,10,13,12,16					; the word "STAGE"
	.byte 16						; 1 blank

load_msg_stage

	.byte 16,16						; loading message stage number
	.byte 16,16,16,16					; trailing blanks


; blank display list

dlist_blank

	.byte BLANK8,BLANK8,BLANK8
	.byte BLANK8,BLANK8,BLANK8,BLANK8
	.byte BLANK8,BLANK8,BLANK8,BLANK8
	.byte BLANK8,BLANK8,BLANK8,BLANK8
	.byte BLANK8,BLANK8,BLANK8,BLANK8
	.byte BLANK8,BLANK8,BLANK8,BLANK8
	.byte BLANK8,BLANK8,BLANK8,BLANK8
	.byte JMPWT,<dlist_blank,>dlist_blank


; x 48 table for frames
; table of slot addresses (base addr + n*48)

slot_addr_lo

	.byte <frame_addr,<[48+frame_addr],<[48*2+frame_addr],<[48*3+frame_addr]
	.byte <[48*4+frame_addr],<[48*5+frame_addr],<[48*6+frame_addr],<[48*7+frame_addr]
	.byte <[48*8+frame_addr],<[48*9+frame_addr],<[48*10+frame_addr],<[48*11+frame_addr]
	.byte <[48*12+frame_addr],<[48*13+frame_addr],<[48*14+frame_addr],<[48*15+frame_addr]
	.byte <[48*16+frame_addr],<[48*17+frame_addr],<[48*18+frame_addr],<[48*19+frame_addr]
	.byte <[48*20+frame_addr],<[48*21+frame_addr]

slot_addr_hi

	.byte >frame_addr,>[48+frame_addr],>[48*2+frame_addr],>[48*3+frame_addr]
	.byte >[48*4+frame_addr],>[48*5+frame_addr],>[48*6+frame_addr],>[48*7+frame_addr]
	.byte >[48*8+frame_addr],>[48*9+frame_addr],>[48*10+frame_addr],>[48*11+frame_addr]
	.byte >[48*12+frame_addr],>[48*13+frame_addr],>[48*14+frame_addr],>[48*15+frame_addr]
	.byte >[48*16+frame_addr],>[48*17+frame_addr],>[48*18+frame_addr],>[48*19+frame_addr]
	.byte >[48*20+frame_addr],>[48*21+frame_addr]


; vector buffer offsets
; NB offset starts at 1
; vector buffer=0 used for freezing movement

vector_buffer_limits

	.byte [frame_change_buffer0-frame_change_buffer]+1
	.byte [frame_change_buffer1-frame_change_buffer]+1
	.byte [frame_change_buffer2-frame_change_buffer]+1
	.byte [frame_change_buffer3-frame_change_buffer]+1
	.byte [frame_change_buffer4-frame_change_buffer]+1
	
;vector_buffer_start
;
;	.byte [frame_change_buffer1-frame_change_buffer]
;	.byte [frame_change_buffer2-frame_change_buffer]
;	.byte [frame_change_buffer3-frame_change_buffer]
;	.byte [frame_change_buffer4-frame_change_buffer]
;	.byte [frame_change_buffer0-frame_change_buffer]


; times 48 minus 9 table hi

x48min9_table_hi	.ds 142


; Sound effects and instruments

	.include sounds.asm


; Objects and Disk Files

	.include objects.asm


; Music

	.include music.asm


; scoring values for high/low property of object

object_score_value

	.byte 1,1,5,1,1

object_score_digit

	.byte 3,3,4,3,2


; jump byte lookup tables for object decode routine

; for 1st part of sprite routine. up to 12 bytes drawn in a line

jump_byte1

	.byte 0, 110, 91, 82, 73, 64, 55, 46, 37, 28, 19, 10, 1

; for 2nd part of sprite routine - negative add, y refresh and end object.
; with up to 12 byte draw now. special on the 12th - a 3 byte jmp

jump_byte2 

	.byte +151, 104, 85, 76, 67, 58, 49, 40, 31, 22, 13, 4, 1


; this allows for z positions 0-20

z_to_framex4

	.byte 0,4,8,12,16,20,24,24,28,28,28,32,32,32,36,36,36,40,40,40,40


; frame map to allow 24 frames per object.
; grouping is by pixel position. 11 frame sizes are available and the smaller
; ones are available as x pre-shifted to 2 pixel, and the smallest to 1 pixel 
; for more accurate x positioning

frame_mapx2

	.byte 0,0,0,0
	.byte 2,2,4,4
	.byte 6,6,6,6
	.byte 8,8,10,10
	.byte 12,12,14,14
	.byte 16,16,18,18
	.byte 20,20,22,22
	.byte 24,24,26,26
	.byte 28,28,30,30
	.byte 32,34,36,38
	.byte 40,42,44,46


; table for helping reference size of frames.

x11_table

	.byte 0,11,22,33,44,55,66,77,88,99,110,121,132,143,154,165,176,187,198,209,220,231


; size of frames

height_table

	.ds 11*max_stage_objects

width_table

	.ds 11*max_stage_objects


; size mapping of z to frame 

size_map

	.byte 0,1,2,3,4,5,6,6,7,7,7,8,8,8,9,9,9,10,10,10,10


; address of screens and display lists for flipping purposes.
; important that b screen comes after a screen in mem or offset screwed

screen_addr_a

	.byte >screen1a,>screen2a,>screen3a

screen_addr_b

	.byte >screen1b,>screen2b,>screen3b

dlist_addr_lo

	.byte <dlist1,<dlist2,<dlist3

dlist_addr_hi

	.byte >dlist1,>dlist2,>dlist3

dlist_addr_offset

	.byte 0,dlist2-dlist1,dlist3-dlist1


; position in dli buffer for each of the triple buffered screens

buffer_addr

	.byte 0,max_dlis,[max_dlis*2]


; z frame offset

old_pos_offset

	.byte 0,max_collision_objects,max_collision_objects*2


; z scroll speed

z_scroll_lo

	.byte 0,61,102,131,153,170,184,195,204,212,219,224,230,234,238,243,245,248,251,253,255


; table of bit settings for bank selection.
; NB main (standard) bank is bank 4

bank_table

	.byte ~00000000,~00000100,~00001000,~00001100,~00010000


; man frame addresses

man_frame_addr

; end of frames (frame+23). run frames 4,3,2,1. 
; then far left, left, centre, centre, right, far right.

	.byte 5*24+3,4*24+3,3*24+3,2*24+3,9*24+3,8*24+3,1*24-1,7*24+3,6*24+3


; man x scroll settings

man_x_scroll

	.byte 2,1,0,255,254


; 43/z table for positioning objects on ground. also basis for grid positions

div43z_table

	.byte 43,34,29,25,22,19,17,16,14,13,12,12,11,10,10,9,9,8,8,8,7


; 43/z table in 2d co-ordinates for explosion drop check

div43z_2d_table

	.byte +90,43,34,29,25,22,19,17,16,14,13,12,12,11,10,10,9,9,8,8,8,7


grid_init_pos

	.byte 85,76,70,66,63,61,59,57,56,55,54,53,52,51,51,51,50,50,49,49,49

grid_add

	.byte 18,19,17

	
top_grid_lo

	.ds 21

top_grid_hi

	.byte 2
	.ds 20


scenery_x

	.byte 256-4-8,256-4,4,4+8,256-4-8,256-4,4,4+8

scenery_y

; NB 1st 2 bytes are with sine bit, end 2 are with sine bit

	.byte 0,0,43,0


; man die sequencer data

player_dest_lo

	.byte <player3,<player2,<player1,<player0,0
	.byte 128,0,128,0,0

player_dest_hi

	.byte >player3,>player2,>player1,>player0,0
	.byte 0,0,0,0,0

player_source_lo

	.byte <man_data_3b,<man_data_2b,<man_data_1b,<man_data_0b,0
	.byte <man_data_3b,<man_data_2b,<man_data_1b,<man_data_0b,0

player_source_hi

	.byte >man_data_3b,>man_data_2b,>man_data_1b,>man_data_0b,0
	.byte >man_data_3b,>man_data_2b,>man_data_1b,>man_data_0b,0

player_code

	.byte 0,0,0,0,128,0,0,0,0,128

player_pmbase_addr

	.byte 0,0,0,0,0,0,0,0,0,>pmbase_addr

man_die_add

	.byte 256-3,3,6,0,0,0

man_die_frame

	.byte 0,28,56,56,84,84

man_die_timer

	.byte 5,4,127,15,1,255


; background draw branch table

background_branch

	.byte BD0-[BD_Branch+2]+256
	.byte BD1-[BD_Branch+2]+256
	.byte BD2-[BD_Branch+2]+256
	.byte BD3-[BD_Branch+2]+256
	.byte BD4-[BD_Branch+2]+256
	.byte BD5-[BD_Branch+2]+256
	.byte BD6-[BD_Branch+2]+256
	.byte BD7-[BD_Branch+2]+256


; background clear branch table

clear_branch

	.byte BC0-[BC_Branch+2]+256
	.byte BC1-[BC_Branch+2]+256
	.byte BC2-[BC_Branch+2]+256
	.byte BC3-[BC_Branch+2]+256
	.byte BC4-[BC_Branch+2]+256
	.byte BC5-[BC_Branch+2]+256
	.byte BC6-[BC_Branch+2]+256
	.byte BC7-[BC_Branch+2]+256


; game command addresses

game_command_lo

	.byte <do_init_stage
	.byte <do_init_boss
	.byte <do_grid_down
	.byte <do_grid_up
	.byte <do_init_pattern_command
	.byte <do_reset_last_alien
	
game_command_hi

	.byte >do_init_stage
	.byte >do_init_boss
	.byte >do_grid_down
	.byte >do_grid_up
	.byte >do_init_pattern_command
	.byte >do_reset_last_alien

; pattern command addresses

pattern_command_lo

	.byte <do_swap_priority
	.byte <do_fireball_speedup
	.byte <do_control_object
	.byte <do_chase_object
	.byte <do_centre_object
	.byte <do_total_counters

pattern_command_hi

	.byte >do_swap_priority
	.byte >do_fireball_speedup
	.byte >do_control_object
	.byte >do_chase_object
	.byte >do_centre_object
	.byte >do_total_counters


; missile speed

missile_speed

	.byte 12			; oval speed
	.byte 0				; fireball speed (initialized in init_pattern), can speedup
	.byte 6				; robot missile speed


; y "sine" positions for scenery. NB y bottom of object. reverse data start at bottom moving up

sine_table

	.byte 0				; when no sine required point here

	.byte 246
	.byte 248
	.byte 252
	.byte 4
	.byte 16
	.byte 24
	.byte 28
	.byte 30

	.byte 30
	.byte 28			; -2
	.byte 24			; -4
	.byte 16			; -8
	.byte 4				; -12
	.byte 252			; -8
	.byte 248			; -4
	.byte 246			; -2


; PAL frames per second

fps_table

	.byte 50,25,17,13,10,8


; counter overlay colour (for Uriah stages)

counter_colour

	.byte 192+6,0+4


; address of draw routine for each counter

counter_draw_addr_lo

	.byte <draw_P0_counter
	.byte <draw_P0_counter
	.byte <draw_P1_counter
	.byte <draw_P1_counter
	.byte <draw_P2_counter
	.byte <draw_P2_counter
	.byte <draw_P3_counter
	.byte <draw_P3_counter

counter_draw_addr_hi

	.byte >draw_P0_counter
	.byte >draw_P0_counter
	.byte >draw_P1_counter
	.byte >draw_P1_counter
	.byte >draw_P2_counter
	.byte >draw_P2_counter
	.byte >draw_P3_counter
	.byte >draw_P3_counter


; counter shapes offset

counter_frames_offset

	.byte 0
	.byte counter2_frames-counter_frames
	

; counter overlay shapes

counter_frames
	
;tree_small_right

	.byte ~00000110
	.byte ~00001110
	.byte ~00001110
	.byte ~00000100

;tree_small_left

	.byte ~01100000
	.byte ~11100000
	.byte ~11100000
	.byte ~01000000

;tree_big_right

	.byte ~00000110
	.byte ~00001110
	.byte ~00001110
	.byte ~00000110
	.byte ~00001110
	.byte ~00001110
	.byte ~00001110
	.byte ~00000100

;tree_big_left

	.byte ~01100000
	.byte ~11100000
	.byte ~11100000
	.byte ~01100000
	.byte ~11100000
	.byte ~11100000
	.byte ~11100000
	.byte ~01000000

counter2_frames

;pylon_small_right

	.byte ~00000100
	.byte ~00001110
	.byte ~00001110
	.byte ~00000100
	
;pylon_small_left

	.byte ~01000000
	.byte ~11100000
	.byte ~11100000
	.byte ~01000000

;pylon_big_right

	.byte ~00000100
	.byte ~00001110
	.byte ~00001110
	.byte ~00001110
	.byte ~00000100
	.byte ~00000100
	.byte ~00000100
	.byte ~00000100
	
;pylon_big_left

	.byte ~01000000
	.byte ~11100000
	.byte ~11100000
	.byte ~11100000
	.byte ~01000000
	.byte ~01000000
	.byte ~01000000
	.byte ~01000000







;test


draw_test
	.include "Converted graphics/robot1.0.asm"

























;
; bank intolerant routines
;


;
; man movement and draw routine
;

move_man

	.local

;	jsr read_joystick		; get controller position

	ldx #0
	ldy #0

	lda PORTA

	lsr a
	bcs ?0				; branch if not up
	ldy #254			; set up
?0
	lsr a
	bcs ?1				; branch if not down
	ldy #2				; set down
?1
	lsr a
	bcs ?2				; branch if not left
	ldx #254			; set left
?2
	lsr a
	bcs ?3				; branch if not right
	ldx #2				; set right
?3
	lda TRIG0
	cmp last_button	
	beq ?4				; branch if button has not been pushed or released

	sta last_button			; save button status for next check

	cmp #0
	bne ?4				; branch if button not pushed

	inc bullet_count		; another bullet required
?4

	.local

	txa
	clc
	adc man_x_pos			; add to man x position. NB man x pos is left players x.
			
	cmp #56+8+8			; check left side(+ 1/2 width of man)
	bcs ?2				; branch if not past boundary

	lda #56+8+8			; reset to boundary limit
	bne ?3				; always branch out
?2
	cmp #200-8-8			; check right boundary (less 1/2 width of man)
	bcc ?3				; branch if not past boundary

	lda #200-8-8			; reset to boundary limit
?3
	sta man_x_pos			; save x position

	sec
	sbc #49+16			; get x into range 0-127
	sta vbi_temp
	lsr a

	lsr a				; add x/4 to give range 0-159
	clc
	adc vbi_temp

	lsr a
	lsr a
	lsr a
	lsr a
	lsr a				; divide by 32 to give number from 0-5
					; 0-4 for commented out stuff
	clc
	adc #4				; make frames start after man run frames (0-3)
	tax

	lda man_x_scroll-4,x		; lookup man x scroll value based on frame
	sta x_scroll


	tya 				; y direction
	sta vbi_temp			; save direction of y movement for sprite clearing routine
	clc
	adc man_y_pos			; add to man y position. NB man y pos is bottom of man.

	cmp #64+42
	bcc ?5				; branch if above ground
	lda #64+42			; reset to ground boundary

	ldx man_run_frame		; get current run animation frame to x register

	bpl ?6				; always branch
	
?5
	cmp #64-43+24			; check top boundary
	bcs ?6				; branch if not past boundary
	lda #64-43+24			; reset to top boundary

?6
	sta man_y_pos			; save y position

	ldy uriah_status
	bne ?10				; don't move player/missile man during uriah stage

	tay				; y start position into y register			
	sec
	sbc #24				; y stop position (graphics 24 pixels high)
	sta man_y_stop

	lda man_x_pos			; x position
	sta HPOSP2			; position players 2 & 3 to the right of 0 & 1
	sta HPOSP3
	sbc #8				; make left side start 8 pixels less than centre (c is set)
	sta HPOSP1			; players 0 & 1
	sta HPOSP0

; position the player missile graphics. NB frame is in x reg

	lda vbi_temp			; y direction
	beq ?8				; branch if man not moving up or down
	bmi ?7				; branch if man moving up

	lda #0				; setup for clearing player

	sta player0-24,y
	sta player1-24,y
	sta player2-24,y
	sta player3-24,y		; man moving down
	sta player0-25,y		; clear trail of pm graphics from previous frame above him
	sta player1-25,y
	sta player2-25,y	
	sta player3-25,y
	beq ?8				; always branch
?7
	lda #0				; setup for clearing player

	sta player0+1,y
	sta player1+1,y
	sta player2+1,y
	sta player3+1,y			; man moving up
	sta player0+2,y			; clear trail of pm graphics from previous frame below him
	sta player1+2,y
	sta player2+2,y
	sta player3+2,y
?8
	lda man_frame_addr,x		; get address of frame (frame in x reg)
	tax
?9
	lda man_data_0a,x		; bank a line data for player 0
	sta player0,y
	lda man_data_1a,x		; bank a line data for player 1
	sta player1,y
	lda man_data_2a,x		; bank a line data for player 2
	sta player2,y
	lda man_data_3a,x		; bank a line data for player 3
	sta player3,y

	dex				; get indexes to next line of data
	dey
	cpy man_y_stop			; do until all lines of man drawn
	bne ?9
?10
	rts


;
; create object
;

create_object

; make the object data from sprite format data in the file buffer

	.local

	ldy #0
?1
	lda file_buffer,y
	sta object_buffer,y		; next 22 bytes for height then width table
	iny
	cpy #11+11
	bne ?1

	sty table2			; save next position in object buffer
	lda #>object_buffer
	sta table2+1			; hi byte

	lda file_buffer,y		; get count of sprite frames in file
	sta frame_no

	bmi ?8				; if -ve then this isn't sprite format object

	iny				; onto next position in file buffer only

	sty source_a			; save next position in file buffer
	lda #>file_buffer
	sta source_a+1			; hi byte

?2
	ldy #0

	lda (source_a),y		; get header data of sprite frame

	tax				; save byte

	and #~11100000
	sta sprite_shift		; bit 7 means 1-shift required, 6 is 2-shift, 5 is 3-shift

	txa				; return save byte
	and #~00011111			; lower bits give qty of exluded frames required
	asl a				; multiply by 2 
	sta zero_count			; to give qty of zeros to write (2 zeros for each frame)

	inc source_a			; onto next header data
	bne ?3

	inc source_a+1
?3
	lda (source_a),y		; get byte width of sprite frame
	sta sprite_width

	inc source_a
	bne ?3a

	inc source_a+1
?3a

	lda (source_a),y		; get (height-1) of this sprite frame
	sta sprite_height

	inc source_a			; get past header data onto real sprite data
	bne ?4

	inc source_a+1
?4
	jsr copy_sprite			; copy sprite data from file buffer to frame buffer (and widen)

	lda last_properties
	bpl ?5				; if mirror property off branch past mirror
	
	jsr mirror_sprite		; create x mirror of sprite
?5
	jsr encode_frame		; encode sprite data into ASH draw format


	ldx zero_count
	beq ?6				; skip if no zeros to do (denotes same frame as last)

	jsr write_zeros			; write any trailing zeros required after main frame

	stx zero_count			; stop trailing zeros being written for shifted frames
?6
	lda sprite_shift
	beq ?7				; branch if finished or no shifts at all required for frame

	jsr shift_sprite		; shift sprite 1 bit right
	jsr shift_sprite		; shift sprite another bit right for 1 pixel shift

	asl sprite_shift		; check if shifted sprite is needed to be encoded
	bcs ?5				; branch if it is
	bcc ?6				; branch if it isn't
?7
	dec frame_no
	bne ?2				; do until all sprite frames in file
?8
	rts


;
; copy sprite
;

copy_sprite

; copy sprite data from file buffer to frame buffer, and widen it
; add an extra 2 columns to the right hand side of the sprite format data to allow shifting right
; and easy width detection

	.local

	jsr init_write_frame_buffer	; frame buffer start into dest_a. height into temp. C clear

?1
	ldx sprite_width		; get sprite width

?2
	lda (source_a),y		; get file buffer sprite data
	sta (dest_a),y

	iny				; next lo data
	bne ?3				; branch if not onto next page

	inc source_a+1			; next page 
	inc dest_a+1
?3
	dex				; do until sprite width copied
	bne ?2

	txa				; 0 into accumulator

	sta (dest_a),y			; into the extra byte

	inc dest_a			; add an extra byte into source_a only
	bne ?4				; branch if not onto next page

	inc dest_a+1			; next page
?4
	sta (dest_a),y			; add another extra byte into source_a only

	inc dest_a
	bne ?4a				; (must be a blank byte at end of line for convert to work)

	inc dest_a+1
?4a

	dec temp
	bpl ?1				; do until full height of sprite frame

	tya
	adc source_a			; save next position in file buffer
	sta source_a
	bcc ?5

	inc source_a+1
	clc
?5
	inc sprite_width		; denote that sprite is now 2 bytes wider
	inc sprite_width

	rts


;
; mirror sprite
;

mirror_sprite

; create an x mirrored version of sprite in frame buffer

	.local

	jsr init_write_frame_buffer	; frame buffer start into dest_a. height into temp. C clear

	ldy sprite_width
	dey
	dey				; sprite width before widen into y
	dey				; less 1 to adjust

	sty temp2
?2	
	lda (dest_a),y
	tax
	
	lda mirror_table,x		; lookup mirror version of data

	pha				; sprite line data into stack for reading in reverse
	
	dey
	bpl ?2				; do until whole line


	ldy temp2			; return sprite width less 2 to y
?3
	pla				; get sprite line data (in reverse)
	sta (dest_a),y			; save back into line data
	
	dey
	bpl ?3				; do until whole line


	ldy temp2			; return sprite width less 2 to y

	lda dest_a
	adc sprite_width		; position to start of next line of sprite
	sta dest_a
	bcc ?4

	inc dest_a+1
	clc
?4
	dec temp
	bpl ?2				; do until height of sprite

	rts


;
; shift sprite
;

shift_sprite

; create 1-bit shifted version of sprite

	.local

	jsr init_write_frame_buffer	; frame buffer start into dest_a. height into temp. C clear
?1
	ldx sprite_width		; x is width counter (has already been adjusted for extra byte)
?2
	lda (dest_a),y
	ror a				; shift data right picking up previous byte's shift bit
	sta (dest_a),y

	iny				; next data
	bne ?3				; branch if not next page

	inc dest_a+1			; next page
?3
	dex
	bne ?2				; do until width of sprite

	dec temp
	bpl ?1				; do until height of sprite

	rts


;
; initialize write frame buffer
;

init_write_frame_buffer

	ldy #0

	tya				; reset dest_a to start of frame buffer area
	sta dest_a
	lda #>frame_buffer_a
	sta dest_a+1

	lda sprite_height		; setup height counter
	sta temp

	clc				; ensure clear for all routines using this

	rts


;
; encode frame
;

encode_frame

; encode sprite format to ASH format psuedo code

	.local

	ldy #0

	tya				; reset source to start of frame buffer area
	sta source_b			; NB Don't use source_a which is for file position
	lda #>frame_buffer_a
	sta source_b+1

	lda sprite_height
	sta temp			; initialize height counter


	lda #48				; screen width
	sec
	sbc sprite_width		; subtract sprite width to get
	sta line_offset			; extra offset to add due to moving down a line

	lda sprite_width
	sta temp2			; initialize width counter

	sbc #1				; subtract only 1 extra shift byte (C is set) for round up 1/2
	lsr a				; 1/2 sprite width for centre-up data

	eor #255
	clc
	adc #1				; make it -ve

	sta add				; -1/2 sprite width in add to start with

	jsr find_data			; find next data in sprite format data
?1
	ldx add
	stx data_buffer+0		; store add in 1st byte of data buffer

	jsr write_data			; write data to data_buffer

	jsr find_data			; find start of next data (if any)
	beq ?2				; branch if end of sprite data

	lda add
	bpl ?3				; branch if not a negative add
?2
	lda data_buffer+1
	ora #128
	sta data_buffer+1		; set bit 7 of byte count to denote negative add or end of frame
?3
	jsr write_buffer		; write data buffer to object buffer

	lda temp
	bpl ?1				; branch if more of sprite height to go

	ldx #2
	jsr write_zeros			; write 2 zeros to mark end of frame

	rts


;
; find data
;

find_data

; find next data in sprite format frame. keep add value from last data
; width counts only blank data, write counts solid data.

	.local

	ldy #0
?1
	dec temp2			; reduce width of object
	bne ?2				; branch past if not end of line

	dec temp			; reduce height of object
	bmi ?4				; branch if finished object

	lda sprite_width
	sta temp2			; reset width

	lda add
	clc
	adc line_offset
	sta add				; add line offset to add for moving down to next line
?2
	lda (source_b),y
	bne ?5				; branch if data found

	inc add				; increase count to next data

	inc source_b			; next data
	bne ?1				; branch if not next page

	inc source_b+1			; next page
	bne ?1				; always branch
?4
	tya				; ensure Accumulator's 0 status is restored
?5
	rts


;
; write data
;

write_data

; write "solid" sprite data into data buffer until mask required or blank data
; because of extra shift bytes the end of line can be detected by these checks too. 
; NB This should width count data, and only find should width count blanks

	.local

	ldy #0

	lda (source_b),y
	sta data_buffer+2,y		; store data in buffer
?1
	iny
	lda (source_b),y
	beq ?3				; branch if reached blank data

	dec temp2			; reduce width counter (NB data only - blank not counted)

	sta data_buffer+2,y		; store data in buffer

	cpy #11				; check if reached maximum object width
	beq ?2				; branch if we have

	tax
	cmp mask_table,x
	beq ?1				; branch till mask is required
?2
	iny				; move to next data
?3
	lda #255
	sta add				; initialize add to -1

	sty data_buffer+1		; store bytes of data count into buffer
	tya

	clc
	adc source_b			; update position in sprite data
	sta source_b
	bcc ?4

	inc source_b+1
?4
	iny
	iny
	sty data_buffer_size		; save size of buffer including add and count bytes


	rts


;
; write buffer
;

write_buffer

; write data buffer to object buffer

	.local

	ldx #0
	ldy #0
?1
	lda data_buffer,x		; add byte is first, then count, finally data
	sta (table2),y			; store in next object buffer position

	inc table2			; next object buffer position
	bne ?2				; branch if not next page

	inc table2+1			; onto next page
?2
	inx

	cpx data_buffer_size		; check if finished data in buffer
	bne ?1

	rts


;
; write zeros
;

write_zeros

; write x zeros into object buffer. (already in 1st write location)

	.local

	lda #0
?1
	sta (table2),y

	inc table2
	bne ?2

	inc table2+1
?2
	dex
	bne ?1

	rts


;
; clear background
;

clear_background

	.local

	ldy #3				; switch to bank 3 where background graphics are
	jsr switch_bank

	lda #0
	tay
?1
	sta back_overlay,y		; clear background data area for title screen
	sta back_overlay+256,y
	sta back_overlay+256*2,y

	sta back_overlay+256*3,y
	sta back_overlay+256*4,y
	sta back_overlay+256*5,y

	dey
	bne ?1

	ldy #4				; switch back to main bank
	jsr switch_bank

	rts


;
; initialize music
;

init_music

; start pattern index of music passed in y

	.local

	ldx music_pat-1,y

	lda music_patM_lo-1,x
	sta music_lo
	lda music_patM_hi-1,x
	sta music_hi			; save Mid (channel 2) address of pattern

	lda music_patB_lo-1,x
	sta music_lo+2
	lda music_patB_hi-1,x
	sta music_hi+2			; save Bass (channel 3) address of pattern

	lda music_patT_lo-1,x
	sta music_lo+4
	lda music_patT_hi-1,x
	sta music_hi+4			; save Treble (channel 4) address of pattern


	lda #1
	sta music_timer
	sta music_timer+2		; timer till first note
	sta music_timer+4

	lda #0				; reset position in
	sta music_index			; Mid pattern
	sta music_index+2		; Bass pattern
	sta music_index+4		; Treble pattern

	sta music_loops
	sta music_loops+2		; reset any loop counters from previous music
	sta music_loops+4

	iny				; index for next music pattern
	sty music_pat_index		; start music patterns

	rts


;
; music pattern player
;

play_music

	.local

	ldx #4				; 3 part music (x=4, x=2 and x=0 only are used)
?1
	dec music_timer,x		; timer till next note due
	bne ?2

	lda music_hi,x
	beq ?2				; skip if part not in use

	sta music_source+1		; part address into zp for indirect
	lda music_lo,x
	sta music_source

	ldy music_index,x		; get position within pattern

	jmp decode_music		; get next note to play
?2
decode_music_rtn

	dex
	dex				; next part (2 separation due to audio register layout)
	bpl ?1

	jmp play_music_rtn


;
; decode music segment
;

decode_music

; NB channel in x. x=0 is channel 3 x=2 is channel 4 (channel 0 and 1 used for fx)
; channel 4 only has control over setting up next pattern

	.local

?1
	iny				; get 1st data
?1a
	lda (music_source),y		; branch past if not end of pattern
	bne ?5

	iny				; 2nd data

	lda (music_source),y		; get number of repeats required
	bne ?3				; branch if not end of pattern

	ldy music_pat_index		; get index of next music pattern

	lda music_pat-1,y		; get next music pattern
	bne ?2				; branch past if real pattern

	iny				; pattern 0 means wrapback pattern next
	lda music_pat-1,y		; get wrapback pattern
	tay				; re-index to wrapback pattern
	lda music_pat-1,y		; get next music pattern
?2
	iny 
	sty music_pat_index		; update index for next music pattern 

	tay				; next pattern required into y

	lda music_patM_lo-1,y
	sta music_lo
	lda music_patM_hi-1,y		; save Mid (channel 2) address of next pattern
	sta music_hi			; NB shared with sound fx & overrides them - minimise  use.

	lda music_patB_lo-1,y
	sta music_lo+2
	lda music_patB_hi-1,y
	sta music_hi+2			; save Bass (channel 3) address of next pattern

	lda music_patT_lo-1,y
	sta music_lo+4
	sta music_source
	lda music_patT_hi-1,y
	sta music_hi+4			; save Treble (channel 4) address of next pattern
	sta music_source+1		; make current music addr channel 4 pattern.

	ldy #0				; reset position in Bass and Mid pattern
	sty music_index			; (no need to reset position in Treble pattern - current)
	sty music_index+2
	sty music_loops			; reset music loop counter in Bass and Mid pattern
	sty music_loops+2		; they can't get reset as treble pattern ends first

	iny
	sty music_timer			; reset timers for Bass and Mid pattern
	sty music_timer+2

	bne ?1a				; always branch back to start new pattern
?3
	cmp music_loops,x
	bne ?4				; branch if pattern has not reached loop count

	iny				; skip over loopback segment

	lda #0
	sta music_loops,x		; reset loop counter

	beq ?1				; always branch back to get next segment
?4
	inc music_loops,x		; increase loop count of pattern

	iny				; next segment

	lda (music_source),y		; get position in pattern to loop back to
	tay				; go to loop back position

	lda (music_source),y
?5
	sta music_timer,x		; data 1 is timer

	iny
	lda (music_source),y

	sta sound_hi+2,x		; data 2 is initial pitch

	iny

	tya
	sta music_index,x		; save index position in pattern

	lda (music_source),y		; data 3 is instrument fx
	tay				; sound fx required into y

	sta sound_stop+3,x		; start sound (instrument 0 to stop)
	bne ?6

	sta AUDF2,x			; switch off sound if instrument 0
?6
	lda volume_start-1,y		; get volume index start (-1)
	sta sound_index+3,x		; save to volume index

	lda volume_stop-1,y
	sta sound_stop+2,x		; volume index stop (when to stop entire sound)

	lda pitch_start-1,y		; get pitch index start (-1)
	sta sound_index+2,x		; save to pitch index

	lda #1				; can't delay sound. will be pointing to wrong segment
	sta sound_timer+2,x		; start straight away for pitch timer
	sta sound_timer+3,x		; and volume timer

	lda pitch_offset_start-1,y
	sta sound_offset+2,x		; start pitch loop offset

	clc
	adc #12
	sta sound_offset+3,x		; add 12 to give loop end

	lda #0
	sta sound_lo+2,x
	sta sound_lo+3,x		; reset volume and pitch auto increase/decrease

	jmp decode_music_rtn


;
; inititialize sound effect or instrument
;

init_sound

; sound effect number passed in y

	.local

	ldx #0				; sound channel 0

	stx sound_queue			; empty sound queue

	cpy sound_stop+1		; channel 0 stop priority (sound effect number)
	bcs ?1 				; branch if required sound has higher priority (0 is lowest)

	ldx #2				; sound channel 1

	cpy sound_stop+3		; channel 1 stop priority
	bcc ?2				; branch out if required sound lower priority - can't play 
?1

	lda volume_start-1,y		; get volume index start (-1)
	sta sound_index+1,x		; save to volume index

	lda volume_stop-1,y
	sta sound_stop,x		; volume index stop (stops entire sound)

	lda pitch_start-1,y		; get pitch index start (-1)
	sta sound_index,x		; save to pitch index

	lda #1				; can't delay sound. will be pointing to wrong segment
	sta sound_timer,x		; for pitch timer
	sta sound_timer+1,x		; and volume timer

	lda pitch_offset_start-1,y
	sta sound_offset,x		; start pitch loop offset

	adc #11
	sta sound_offset+1,x		; add 12 to give loop end (c is set)

	lda #0
	sta sound_lo,x
	sta sound_lo+1,x		; reset volume and pitch auto increase/decrease

	tya				; start sound and
	sta sound_stop+1,x		; save required sound's priority in the available channel
?2
	rts


;
; play sounds
;

play_sounds

; play upto 4 sounds - each sound is a sound effect or instrument

	.local

	ldx #7				; 4 sounds *2 for pitch and volume registers interleaved
?1
	lda sound_stop,x
	beq ?7				; branch if sound off

	ldy sound_index,x		; get current volume segment index

	dec sound_timer,x		; timer till next volume segment
	beq init_volume 

?2 
	lda sound_lo,x			; auto volume rise/decrease
	clc
	adc volume_add_lo,y
	sta sound_lo,x			; add to lo byte of volume

	lda sound_hi,x
	adc volume_add_hi,y
?3
	sta sound_hi,x			; add to hi byte of volume

	sta AUDC1-1,x			; save actual volume/distortion in audio control of channel


	dex				; pitch data

	ldy sound_index,x		; get current pitch segment index

	dec sound_timer,x		; timer till next pitch segment
	beq init_pitch

?4			
	lda sound_lo,x			; auto pitch rise/decrease
	clc
	adc pitch_add_lo,y
	sta sound_lo,x			; add to lo byte of pitch

	lda sound_hi,x
	adc pitch_add_hi,y
?4a
	sta sound_hi,x			; add to hi byte of pitch


	ldy sound_offset,x		; get pitch offset index

	clc
	adc pitch_offset,y		; add pitch offset

	sta AUDF1,x			; save actual pitch in audio frequency of channel			

	iny				; get next pitch offset index
	tya
	cmp sound_offset+1,x
	bcc ?5				; branch if not end index of pitch offset loop plus 1 (y >=)

	sbc #12				; wrap back to start of pitch loop (12 long loops)
?5
	sta sound_offset,x

	dex
	bpl ?1				; do until all sounds

	jmp play_sounds_rtn
?6
	lda #0

	sta sound_stop,x
	sta sound_hi,x
	sta AUDC1-1,x			; turn off sound
?7
	dex
	dex
	bpl ?1				; do until all sounds	

	jmp play_sounds_rtn

init_volume

 	iny				; index to next volume segment	 

	tya
	cmp sound_stop-1,x		; compare index to stop volume segment
	beq ?6				; exit if reached stop segment
	sta sound_index,x		; save index to next volume segment

	lda volume_timer,y
	sta sound_timer,x		; save timer for next volume segment

	lda volume_initial,y
	beq ?2				; branch to carry on volume processing
	bne ?3				; branch if theres an initial volume/distortion for segment	

init_pitch

	iny				; index to next pitch segment

	tya
	sta sound_index,x		; save index to next pitch segment

	lda pitch_timer,y
	sta sound_timer,x		; save timer for next pitch segment

	lda pitch_initial,y
	beq ?4				; branch to carry on pitch processing
	bne ?4a				; branch if theres an initial pitch for segment


;
; copy background
;

copy_background

; copy 4 colour background source data and convert to screen a and b format

	.local

	ldy #3				; switch to bank 3 where background graphics are
	jsr switch_bank

	lda #<object_buffer
	sta source_a
	lda #>object_buffer
	sta source_a+1

	lda #3				; 3 pages worth of data for background to copy
	sta temp
?1
	ldy #0
?2
	lda (source_a),y		; get 4 colour source
	tax

	lda colour_table_a,x
	sta (dest_a),y			; convert to screen a palette

	lda colour_table_b,x
	sta (dest_b),y			; convert to screen b palette

	dey				; do until 256 bytes
	bne ?2

	inc source_a+1			; next hi bytes
	inc dest_a+1
	inc dest_b+1

	dec temp			; do until 3*256 data
	bne ?2

	ldy #4				; switch back to main bank
	jsr switch_bank

	rts


;
; man die sequencer
;

sequence_man_die

; VBI man die routines. setup man frames to draw and position in player missile graphics.
; parcel out draw of individual players to play_sample routine as there is not much spare proc
; time when playing sample

	.local

	ldx player_index		; get next code slice index
	bmi ?1a				; exit if not enabled
	dex
	bpl ?1

	ldx #9				; wrap for 10 code slice indexes (0-9)
?1
	stx player_index		; save code slice index

	lda player_code,x
	bmi ?2				; branch for setup code section	

; draw player missile code section

	lda player_dest_lo,x		; setup draw position for this player missile
	clc
	adc player_offset
	sta player_dest

	lda player_dest_hi,x
	sta player_dest+1

	lda player_source_lo,x		; setup draw source for this player missile
	clc
	adc player_frame
	sta player_source

	lda player_source_hi,x
	adc #0
	sta player_source+1

	lda #0				; reset draw position for play sample routine
	sta player_pos
?1a				
	jmp sequence_man_die_rtn	; return
?2

; player setup code section

	lda player_pmbase_addr,x	; display completely drawn pmg frame
	sta PMBASE

	dec player_timer
	bmi ?5				; infinite wait if -ve
	bne ?3				; branch if not time for new setup

	ldx man_die_index		; get man die index
	inc man_die_index		; setup for next die commands

	lda man_die_timer,x		; get timer for next man die event
	sta player_timer

	lda man_die_add,x		; setup constant add (or subtract) to man position
	sta player_add

	lda man_die_frame,x		; setup frame to display
	sta player_frame

	jmp sequence_man_die_rtn	; return
?3
	lda player_offset		; add to man position
	clc
	adc player_add
	cmp #64+43-24-3			; check man position to ground
	bcc ?4
	lda #1				; if just under ground set timer for next setup
	sta player_timer
	lda #64+42-24-3			; set to ground
?4
	sta player_offset

	jmp sequence_man_die_rtn	; return
?5
	inc player_timer

	jmp sequence_man_die_rtn	; return


;
; man collision
;

 check_man_collision

; get positions of objects on dlist screen and check for collision with man
; NB has to now be done in VBI so as not to miss any frame checks.
; y reg has current dlist.

	.local

	ldx old_pos_index,y		; get object positions for current dlist screen

	lda old_pos_offset,y
	sta vbi_temp			; save last index position to check
	cpx vbi_temp
	beq ?3				; branch if no objects to check

	lda man_x_pos

	sec
	sbc #4
	sta man_left			; calc left edge of man (4 pix in). NB carry set

	adc #7
	sta man_right			; calc right edge of man (4 pix in)

	lda man_y_pos			; NB man middle of screen=76. object middle of screen=90

	adc #90-76-8
	sta man_top			; calc top edge of man (4 pix in)

	adc #16
	sta man_bottom			; calc bottom edge of man (4 pix in)
?1
	lda old_left-1,x		; left edge of object
	cmp man_right			; compare to man right edge
	bcs ?2				; branch if object left edge greater. ie miss

	lda old_right-1,x		; right edge of object
	cmp man_left			; compare to man left edge
	bcc ?2				; branch if object right edge less. ie miss

	lda old_top-1,x			; top edge of object
	cmp man_bottom			; compare to man bottom edge
	bcs ?2				; branch if object top edge greater. ie miss

	lda old_bottom-1,x		; bottom edge of object
	cmp man_top			; compare to man top edge
	bcc ?2				; branch if object bottom edge less. ie miss


	lda #0
	sta bullet_count		; stop any outstanding bullets being done after man has died

	lda #man_die_bit		; flag man hit
	sta man_status
	rts
?2
	dex				; do until all close objects
	cpx vbi_temp	
	bne ?1
?3
	rts


;
; copy frame
;

copy_frame

; copy frame from source a & b to dest a & b in memory. amount to copy in mem_reqd

	.local

	ldy #0

	ldx mem_reqd+1			; hi bytes required to copy					
	beq ?2				; if there are less than 256 bytes to copy, branch here.
?1
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny
	bne ?1

	inc source_a+1
	inc dest_a+1
	inc source_b+1
	inc dest_b+1
	
	dex				; hi byte counter.
	bne ?1				; branch to copy next 256 bytes
?2	
	ldx mem_reqd			; copy lo bytes remaining
	beq ?4				; branch if none to copy
?3	
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny
	dex
	bne ?3
?4
	rts


;
; mem remaining help. put all 8000-afff data before this
;

mem_position_8000-afff


;
; mostly program code
;


;
; sprite draw 
;
	
	*=$3001

	.local

draw_sprite_addr

; not the entry address of the sprite routine, just a useful address
; calculation for the assembler

; entry points for sprite lines of different widths 

	iny				; width 12
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 11
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 10
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 9
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 8
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 7
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 6
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 5
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 4
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 3
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; end of line mask (and width 2)
	lax (source_a),y
	ora (dest_a),y
	and mask_table,x
	sta (dest_a),y
	lax (source_b),y
	ora (dest_b),y
	and mask_table,x
	sta (dest_b),y
	
	iny				; get add required to start of next line
	lda (source_a),y
	adc dest_a
	sta dest_a
	sta dest_b			; NB screen a & b must be given the same lo byte address
	bcc ?1
	inc dest_a+1
	inc dest_b+1
	clc
?1

draw_sprite_noclip

; entry address if not using clipping. remember the drawing will start one byte 
; further to the right because of jump fetch. 
; (allowed for in draw_screen routine's convert of x position to screen addr)
	
	lda (source_b),y		; get jump addr for code to draw width of next line
	sta jump			; high byte of this address must be set before calling sprite

draw_sprite				; entry point after clipping

	iny
	lax (source_a),y		; start of line mask (and width 1)
	ora (dest_a),y
	and mask_table,x
	sta (dest_a),y
	lax (source_b),y
	ora (dest_b),y
	and mask_table,x
	sta (dest_b),y
		
	jmp (jump)			; jump to draw correct width of data


draw_sprite_offset

; 2nd part of sprite. being sent here, after drawing the next line, the following are checked for:
; negative add, y register refresh and end of object

; entry points for sprite lines of different widths 

	.local

	jmp sprite_12_wide		; width 12

sprite_11_wide

	iny				; width 11
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 10
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 9
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 8
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 7
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 6
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 5
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 4
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny				; width 3
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	iny
	lax (source_a),y		; end of line mask (and width 2)
	ora (dest_a),y
	and mask_table,x
	sta (dest_a),y
	lax (source_b),y
	ora (dest_b),y
	and mask_table,x
	sta (dest_b),y

	iny
	lda (source_b),y		; get jump addr for code to draw width of next line
	beq ?exit			; check for end of object
	sta jump			; high byte of this address must be set before calling sprite

; Extra checks to detect -ve add and y register refresh
; NB can't do -ve add and a y register refresh on the same line - optimize program does check for this
		
	lda (source_a),y		; get add required to start of next line
	bmi ?neg			; branch if the reason for this code call is for a -ve add.

; optimize program says y register will overflow next line.

	adc dest_a			; first store the add
	sta dest_a			; doesn't need to store b dest lo byte yet. done on y reset
	bcc ?1
	inc dest_a+1
	inc dest_b+1
	clc
?1
	tya				; reset the y register and indirect addresses
	adc dest_a
	sta dest_a
	sta dest_b
	bcc ?2
	inc dest_a+1
	inc dest_b+1
	clc
?2
	tya
	adc source_a
	sta source_a
	sta source_b			; screen a & b must be given same lo byte address
	bcc ?3
	inc source_a+1
	inc source_b+1
	clc
?3
	ldy #1				; reset y for next data.	

	lax (source_a),y		; start of line mask
	ora (dest_a),y
	and mask_table,x
	sta (dest_a),y
	lax (source_b),y
	ora (dest_b),y
	and mask_table,x
	sta (dest_b),y
	
	jmp (jump)			; jump to draw correct width of data


?exit
	jmp draw_sprite_rtn


?neg
	adc dest_a			; -ve add to start of next line
	sta dest_a
	sta dest_b			; NB screen a & b must be given the same lo byte address
	bcs ?4

	dec dest_a+1
	dec dest_b+1
?4
	clc

	iny
	lax (source_a),y		; start of line mask (and width 1)
	ora (dest_a),y
	and mask_table,x
	sta (dest_a),y
	lax (source_b),y
	ora (dest_b),y
	and mask_table,x
	sta (dest_b),y
	
	jmp (jump)			; jump to draw correct width of data


sprite_12_wide

; since 12 wide refresh or -ve add won't fit, a jump to here is done if a 12 wide
; refresh or add has to be done. the 3 byte jmp required just fits in the page boundary
; The event is so rare the extra penalty is negligable, and the benefits to the encoder
; are well worth it. ie no refresh problem on 12 wide lines, no problem with 13 wide -ve lines

	iny
	lda (source_a),y
	sta (dest_a),y
	lda (source_b),y
	sta (dest_b),y

	jmp sprite_11_wide


;
; draw screen frame
;

draw_screen

; code that calls the draw routines in the correct order of objects' z positions

	.local

	ldy sprite_count
	beq ?4				; skip if no objects to draw


	clc				; ensure c clear for before 1st sprite call
?1
	ldx sort_table+z_clip-1,y	; objects in sorted order excluding z_clipped objects

	sty temp


	ldy slot,x

	lda slot_addr_lo,y		; get slot base address
	sta table1
	lda slot_addr_hi,y
	sta table1+1

	lda x_pos_2d,x
	and #~00000011			; get pixel positioning
	ldy z_pos,x
	adc z_to_framex4,y		; add frame * 4 to include pixel pos. c is clear
	tay
	lda frame_mapx2,y		; convert to closest available frames
	tay

; frame data stored as follows:
; frame_lo is actual frame lo byte.
; frame_hi bit 0-4 hi address. (have to add $4000 to get mem bank offset)
; frame_hi bit 5-7 mem bank
	
	lda (table1),y			; lo frame address
	sta source_a
	sta source_b
	
	iny
	lda (table1),y
	and #~00011111			; get just hi address portion
	ora #~01000000			; add offset to $4000
	sta source_a+1			; hi frame address (of screen a)

	adc #>bankB->bankA		; add bank offset to b screen (offsets must always be same)
	sta source_b+1			; last sprite call always leaves c clear. c clear after this

; value of portb to select the correct bank for each type of object. (130xe only)
; bit 0 off disables os rom.
; bit 1 on disables basic.
; bit 7 on disables self test rom
; bit 5 on makes antic access main bank. off makes antic access extra banks
; bit 4 on makes cpu access main bank. off makes cpu access extra banks
; bit 2 & 3 - extra banks of 16K select. 0-3 available (64k) 
	
	lda (table1),y
	and #~11100000			; get just bank number
	lsr a				
	lsr a				; move bank number into bits 2-4
	lsr a
	ora #~10100010			; turn off os,basic,self test,antic main bank (default)
	
	sta PORTB			; switch to correct memory bank for object

; x is now in pixels and ranges from about -96 to +96. To get mem address +95 and /4
; pixels to be handled in frames of object.
; 92 centre used rather than 96 to compensate for sprite routine starting object one byte
; further right than specified position

	ldy x_pos_2d,x

	lda y_top,x
	tax

	tya

	lsr a
	lsr a

	ldy #0
	clc				; add x centre-up data for object. can be plus or minus
	adc (source_a),y

	clc
	adc x48min9_table_lo,x		; add 48 times table minus 9 (36/4) to allow for 36 offscreen
	sta dest_a
	sta dest_b
	lda x48min9_table_hi,x
	adc clip_screen_a

; NB clip_screen is 9 pages lower than draw screen, to allow objects to go off top of screen
; - 9 (48 * 48) / 256 from screen hi address to be able to give 48 y pixel offscreen.

	sta dest_a+1
	adc #>screen1b->screen1a
	sta dest_b+1
	
	lda #>sprite_addr		; setup sprite jump command hi byte
	sta jump+1

	jmp sprite

draw_sprite_rtn

	ldy temp
	dey
	bne ?1

	lda #~10110010			; main bank.turn off os,basic,self test,antic main bank
	sta PORTB			; switch back to main bank
?4
	rts



test_vars

	lda #12
	sta dest_a
	lda #>screen1a
	sta dest_a+1
	lda #12
	sta dest_b
	lda #>screen1b
	sta dest_b+1

	rts



draw_screen_end


;
; sprite clip routine
;

	*=$3201

	.local

sprite_addr

; not the entry address of the sprite clip routine just a useful address for the assembler

	tya
	adc #13
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #12
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #11
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #10
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #9
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #8
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #7
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #6
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #5
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #4
	tay
	bne ?0
	nop
	nop
	nop

; end of line mask.(19 bytes)(60 bytes to fill until 2nd part for -ve add etc.)

	iny
	iny
	iny
	bne ?0

	nop
	nop
	nop
	nop
	nop
	nop
?draw
	lda #>draw_sprite_addr		; set hi byte of jump to the draw routine
	sta jump+1
	clc
	jmp draw_sprite

; get add required to start of next line

	iny
	iny
?0
	lda (source_a),y
	adc dest_a
	sta dest_a
	sta dest_b			; NB the dither screens must be given the same lo byte address
	bcc ?1
	inc dest_a+1
	inc dest_b+1
	clc
?1


; sprite entry address. remember the drawing will start one byte 
; further to the right because of jump fetch. 
; (Fixed in Priority routines convert x position to screen addr)

sprite

	lda (source_b),y
	sta jump			; high byte of this must be set before calling sprite

; check if on-screen

	tya				; add y to get "real" dest address
	adc dest_a			; with dest lo. Carry set if on next page of mem
	lda #0		
	adc dest_a+1			; include this carry in dest hi address
	cmp draw_screen_a
	bcs ?draw			; branch if on-screen

?2
; where start of line mask was (19 bytes)

; goto part of routine to draw correct width of data
	
	jmp (jump)
	nop
	nop
	nop
	nop
	nop
	nop
	nop


; 2nd part of sprite. being sent here, after drawing the next line, the following are checked for:
; negative add, y register refresh and end of object.

sprite_offset

	.local

; special case of 12 wide refresh/-ve add. only have 3 bytes room

	iny
	nop
	nop

	tya
	adc #12
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #11
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #10
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #9
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #8
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #7
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #6
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #5
	tay
	bne ?0
	nop
	nop
	nop

	tya
	adc #4
	tay
	bne ?0
	nop
	nop
	nop

; end of line mask.

	iny
	iny
	iny
	bne ?0

	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop

; get jump addr for code to draw width of next line or detect end of object

	iny
	iny
?0
	lda (source_b),y
	beq ?exit			; check for end of object
	sta jump			; high byte of this must be set before calling sprite
	
; get add required to start of next line

	lda (source_a),y
	bmi ?neg			; branch if the reason for this code call is a -ve add.

; NB the code can't do a -ve add and a y register refresh on the same line
	
; if we are here the y register will overflow next line.

; first store the add code

	adc dest_a
	sta dest_a
;	sta dest_b			; doesn't need to store b dest lo byte. done on y reset
	bcc ?1
	inc dest_a+1
	inc dest_b+1
	clc
?1
; reset the y register and referrenced addresses

	tya
	adc dest_a
	sta dest_a
	sta dest_b
	bcc ?2
	inc dest_a+1
	inc dest_b+1
	clc
?2

	tya
	adc source_a
	sta source_a
	sta source_b			; each dither screen must be on same lo byte address
	bcc ?3
	inc source_a+1
	inc source_b+1
	clc
?3


; start of line mask

	ldy #0				; reset y reg.

; check if on-screen

	lda dest_a+1
	cmp draw_screen_a
	bcs ?draw			; branch if on-screen

?4	
; goto correct part of routine to draw correct width of data
	
	jmp (jump)

?exit
	jmp draw_sprite_rtn

?draw
	lda #>draw_sprite_addr		; set hi byte of jump to the draw routine
	sta jump+1
	clc
	jmp draw_sprite			; object is on-screen jump to start drawing

; -ve add code

?neg	adc dest_a
	sta dest_a
	sta dest_b			; NB the dither screens must be given the same lo byte address
	bcs ?5
	dec dest_a+1
	dec dest_b+1
?5	clc

; start of line mask

; check if on-screen

	tya				; add y to get "real" dest address
	adc dest_a			; with dest lo. Carry set if on next page of mem
	lda #0		
	adc dest_a+1			; include this carry in dest hi address
	cmp draw_screen_a
	bcs ?draw			; branch if on-screen

?6	
; goto correct part of routine to draw correct width of data
	
	jmp (jump)


;
; NMI handler
;

nmi_handler

	.local

	bit NMIST			; check for cause of interrupt			
	bpl vbi				; branch if VBI


;
; display list interrupt
;

dli

; bit 7 NMIST means dli


	pha				; save A

	sty dli_temp			; save Y
	
	ldy colour_buffer_read
	
	lda colour_buffer,y		; get next colour
	tay

	inc colour_buffer_read		; index for next colour

	pla				; restore A

	sty WSYNC			; wsync must be done soon enough or it waits for next scan line
	sty COLBK			; bak colour

	ldy dli_temp			; restore Y

	rti				; exit dli asap or sample playback messed up.


;
; vertical blank interrupt
;

vbi

; bit 6 NMIST means vbi

	.local

	pha				; save A

	lda sample_status
	beq ?2				; branch if no sample playing

	lda next_vcount
?1
	cmp VCOUNT
	bne ?1				; wait for sample playback

	stx AUDC3			; play sample. now we know we have time till next sample due
?2	

	sta NMIRES			; reset interrupt flag

	txa				; save other registers
	pha
	tya
	pha


	lda man_status			; branch past if man is not dying
	bpl ?3

	jmp sequence_man_die		; man dying animation and falling
?3

	dec frame_count			; check timer for page flip
	bpl ?7				; branch if not time yet

; testing code only
; default to put warning in missiles colour
; to show missed frame_count timing

 	lda #58
	sta COLPF3


	lda flip_count			; check if there are draw screens ready for flipping
	beq ?7				; branch if not. ie missed desired frame rate

; testing code only
; if frame_count in time then reset missiles colour

	lda frame_count
	cmp #255
	bne ?temp
	lda #0
	sta COLPF3

?temp
	dec flip_count			; flag as flipped	


	ldy sound_queue			; check for any sound fx to start
	beq ?4				; branch if none
	
	jsr init_sound			; start sound fx
?4
	lda game_speed			; set frames to wait till next flip
	sta frame_count

	ldy dlist_index			; get dlist 1,2 or 3 to display
	dey				; get next screen to flip. current in y
	bpl ?5

	ldy #2
?5
	sty dlist_index
 
	lda dlist_addr_lo,y		; change dlist to new screen
	sta DLISTL
	lda dlist_addr_hi,y
	sta DLISTH

	dec man_run_frame		; man run animation
	bpl ?6
	
	lda #3				; wrap man run animation. 4 frames 3-0
	sta man_run_frame
?6	
	lda man_flash_timer
	bne ?7				; skip man collision check if flashing

	jsr check_man_collision		; check for man collisions. NB requires y as screen
?7
sequence_man_die_rtn

	ldy dlist_index			; get dlist 1,2 or 3 to display

	ldx dlist_addr_offset,y		; get dlist addr offset into x

	lda vbi_count			; keep count of vbis - toggle between odd/even
	eor #1
	sta vbi_count
	bne ?8

	lda screen_addr_B,y		; change screen address of dlist to the B screen
	sta dlist+9,x


	lda #176+gamma_adjust		; green
	sta COLPF0

	lda #112+gamma_adjust		; blue
	sta COLPF1

	bne ?9				; always branch
?8	
	lda screen_addr_A,y		; change screen address of dlist to the A screen
	sta dlist+9,x

	lda #48+gamma_adjust		; red
	sta COLPF0

	lda #gamma_adjust+2		; white
	sta COLPF1

	lda man_flash_timer		; check man indestructible/flash timer active
	beq ?9				; branch if not

	and #~00000010			; timer will alternate between on/off. bit 2 turns on.
	sta GRACTL			; but make sure when timer done players are on.
	inc man_flash_timer		; next timer
?9
	lda buffer_addr,y		; get buffer addr of dlist being displayed this frame
	sta colour_buffer_read		; save for dli routine

	lda man_status
	bmi ?11

	jsr move_man			; move and draw man

	lda vbi_count
	bne ?10

	lda music_pat_index
	beq ?10				; music pat 0 means no music to play

	jmp play_music			; music player

play_music_rtn
?10
	jmp play_sounds			; play sounds

play_sounds_rtn
?11
	pla				; restore all registers and exit interrupt
	tay
	pla
	tax
	pla

	rti

;
; decode frame
;

decode_frame

; decode frame to frame buffer, then copy to free memory

	.local

	lda frame_no
	sta temp2			; save status of raw (-ve) or 4 colour data

	lda #0
	sta frame_no
?1
	lda temp2
	bmi ?2				; branch if raw 16 colour format data

	jmp decode_4_colour		; decode 4 colour psuedo ASH code into 16 colour ASH
?2
	jmp decode_raw_colour		; decode raw 16 colour format ASH code into 16 colour ASH

decode_4_colour_rtn
decode_raw_colour_rtn

; check if extra frame for large smooth object (very rarely used)
; the address of the extra frame must be set to the previous frame's address when it is not
; being used

	lda mem_reqd+1
	bne ?3	
	lda mem_reqd
	beq ?4
	cmp #1
	beq ?4				; if only 0 or 1 byte mem required then same as last frame 
?3
	jsr find_memory			; find somewhere to put object of mem_reqd size

	lda free_mem			; make dest address for frame a the bank's next memory
	sta dest_a
	sta dest_b			; b lo bytes are same as a
	lda free_mem+1
	sta dest_a+1

	ora #~00100000			; gives address + $2000 to give b frame offset
	sta dest_b+1

	jsr copy_frame			; copy the frame data to bank memory

; update frame address of the object

?4
	ldy frame_no
	
	lda free_mem
	sta (table1),y

	iny

	lda PORTB
	and #~00011100
	asl a
	asl a
	asl a
	sta temp			; memory bank into bits 5-7

	lda free_mem+1
	eor #~01000000			; turn off $4000 bank offset - need bits 5-7 for bank number
	adc temp			; c is clear due to asl a
	sta (table1),y

	iny

	sty frame_no

	cpy #48				; do until last frame
	beq ?5
	jmp ?1
?5
	ldy #4
	jsr switch_bank			; re-enable main bank

	rts

;
; decode 4 colour frame
;

decode_4_colour

	.local
?1
	lda #>frame_buffer_a		; dest hi addr for a frame
	sta dest_a+1
	sta source_a+1			; and source for later memory copy of frame a

	lda #>frame_buffer_b		; dest hi addr for b frame
	sta dest_b+1
	sta source_b+1			; and source for later memory copy of frame b

	ldy #1
	sty mem_reqd			; init count of mem required for frame

	dey

	sty mem_reqd+1			; set hi byte of mem required to zero
	sty dest_a			; frame a and b lo address are zero too
	sty dest_b
	sty source_a			; as are a and b lo source address for later copy
	sty source_b	
?2
	lda (table2),y			; get add byte
	sta (dest_a),y			; put in a frame add byte

	inc table2			; next source
	bne ?3				; NB source has in effect 1 more command byte than
	inc table2+1			; the final a & b frames per line, so can't use iny
?3
	lda (table2),y			; get byte counter of graphic data in line
	bpl ?4				; -ve denotes whether normal or special jump byte
	and #~01111111			; remove -ve
	tax
	lda jump_byte2-1,x		; lookup special jump byte value for the line -
	bne ?5				; NB no need for zero lookup like byte 1. always branch
?4
	tax
	lda jump_byte1,x		; lookup normal jump byte value for the line
?5
	sta (dest_b),y			; put into frame b jump byte
	beq ?10				; branch if end of frame

	stx temp			; count of graphic bytes in line
	bne ?7				; always branch for next data
?6
	lda (table2),y			; get 4 colour source
	tax

	lda colour_table_a,x		; convert to screen a colour space
	sta (dest_a),y

	lda colour_table_b,x		; convert to screen b colour space
	sta (dest_b),y
?7
	inc mem_reqd			; update memory required for a frame
	bne ?8
	inc mem_reqd+1
?8
	iny				; next data
	bne ?9
	inc table2+1
	inc dest_a+1
	inc dest_b+1
?9
	dec temp
	bpl ?6				; do until all graphic data in line finished
	bmi ?2				; go get next add byte

?10					; end of frame
	tya
	sec
	adc table2			; update source address to start of next frame
	sta table2			; carry is set so points to next rather than last
	bcc ?11				; of previous frame
	inc table2+1
?11
	jmp decode_4_colour_rtn

;
; decode raw colour frame
;

decode_raw_colour

	.local
	
?1
	ldy #0

	lda (table2),y
	sta mem_reqd
	
	inc table2
	bne ?2

	inc table2+1	
?2
	lda (table2),y
	sta mem_reqd+1			; get memory required for frame
	
	inc table2
	bne ?3
	
	inc table2+1
?3	
	lda table2
	sta source_a			; a data for copy

	clc
	adc mem_reqd
	sta source_b			; b data is mem_reqd further on than a
	
	lda table2+1
	sta source_a+1			; a hi bytes
	
	adc mem_reqd+1
	sta source_b+1			; b hi bytes

	lda source_b			; next frame is mem_reqd further on than b
	adc mem_reqd
	sta table2
	
	lda source_b+1
	adc mem_reqd+1
	sta table2+1

	jmp decode_raw_colour_rtn

;
; find memory
;

find_memory

; finds free memory of at least length mem_reqd
; looks in all banks. returns free_mem, bank_index and switches to free_bank
; free addresses returned are for object a data. object b data must be calculated

	.local

	ldy #3				; used as bank index. NB not using main bank (4)

?1	lda #<bankB-1
	sec
	sbc next_mem_lo,y
	sta mem_remain			; calculate memory remaining in bank
	lda #>bankB-1
	sbc next_mem_hi,y
	sta mem_remain+1
	
	cmp mem_reqd+1
	bcc ?3				; branch if not enough hi byte memory
	bne ?2				; if easily enough then branch to get memory
	lda mem_remain
	cmp mem_reqd
	bcc ?3				; branch if not enough lo byte memory

?2	
	jsr switch_bank			; switch to bank where spare mem is. passed in y reg.

; return available memory in free_mem
; add memory taken from bank to next available memory


	lda next_mem_lo,y
	sta free_mem
	clc
	adc mem_reqd			; add memory just taken to give next address
	sta next_mem_lo,y

	lda next_mem_hi,y
	sta free_mem+1
	adc mem_reqd+1
	sta next_mem_hi,y

	
	rts				; done

?3
	dey
	bpl ?1

; if we get here then no memory has been found. hang up as an error

?4
	bmi ?4


;
; switch bank
;

switch_bank

; y reg is bank number required

	.local

	lda PORTB				
	and #~11100011
	ora bank_table,y		; switch to bank y
	sta PORTB

	rts


;
; reorg bank
;

reorg_bank

; copy block of memory to dest from source, up to top of available memory in bank given in y
; since it is being moved down, have to copy from 0 sequentially upward to avoid overlap and data loss

	.local

	stx temp

	jsr switch_bank			; switch to bank where copy required

; setup addresses for bank B copy. B addresses are exactly $2000 more than A

	lda source_a
	sta source_b	
	lda source_a+1
	ora #~00100000			; gives address + $2000
	sta source_b+1

	lda dest_a
	sta dest_b
	lda dest_a+1
	ora #~00100000
	sta dest_b+1

; how much to copy?
; top of bank mem less source address

	lda next_mem_lo,y
	sec
	sbc source_a
	sta mem_reqd			; lo bytes required to copy
	lda next_mem_hi,y
	sbc source_a+1
	sta mem_reqd+1			; hi bytes required to copy

; subtrack objects size in bank from banks next memory available

	lda next_mem_lo,y
	sec
	sbc object_size_lo,y
	sta next_mem_lo,y
	lda next_mem_hi,y
	sbc object_size_hi,y
	sta next_mem_hi,y

	jsr copy_frame			; perform copy

	ldy #4
	jsr switch_bank			; switch back to main bank

	ldx temp

	rts


;
; optimize objects
;

optimize_objects

; optimize objects page faults (depends on start address of object)

	.local

	lda #0				; reset object comparison
	sta dest_a+1

	ldx #max_stage_objects-1	; all possible object slots
?1
	lda slot_usage,x
	bmi ?6				; branch if slot not in use

	stx slot_no

	lda slot_addr_lo,x		; get slot base address into table1
	sta table1
	lda slot_addr_hi,x
	sta table1+1

	ldy #46
?2
	sty frame_no

; frame data stored as follows:
; frame_lo is actual frame lo byte.
; frame_hi bit 0-4 hi address. (have to add $4000 to get mem bank offset)
; frame_hi bit 5-7 mem bank

	lda (table1),y			; lo frame address
	cmp dest_a			; check if same as previous frame address lo
	sta source_a
	sta source_b
	sta dest_a			; save lo address for checking against next frame
	clc				; c clear to indicate different frame address lo
	bne ?3				; branch if it is different frame address lo
	sec				; c set to indicate same lo address	
?3
	iny
	lda (table1),y
	and #~00011111			; get just hi address portion
	ora #~01000000			; add offset to $4000
	bcc ?4				; branch if no need to check hi address
	cmp dest_a+1			; check if same as previous frame address hi
	beq ?5				; skip if yes. It's already been done
?4
	sta source_a+1
	sta dest_a+1
	ora #~00100000			; add $2000 offset to give B bank
	sta source_b+1

; value of portb to select the correct bank for each type of object. (130xe only)
; bit 0 off disables os rom.
; bit 1 on disables basic.
; bit 7 on disables self test rom
; bit 5 on makes antic access main bank. off makes antic access extra banks
; bit 4 on makes cpu access main bank. off makes cpu access extra banks
; bit 2 & 3 - extra banks of 16K select. 0-3 available (64k) 
	
	lda (table1),y
	and #~11100000			; get just bank number
	lsr a				
	lsr a				; move bank number into bits 2-4
	lsr a
	ora #~10100010			; turn off os,basic,self test,antic main bank (default)
	sta PORTB


	jsr optimize_frame

?5
	ldy frame_no			; next frame (*2)
	dey
	dey
	bpl ?2				; do until all frames

	ldx slot_no
?6
	dex
	bpl ?1

	ldy #4
	jsr switch_bank			; switch back to main bank


	rts


;
; optimize frame
;

optimize_frame

; do the y register refresh according to object start address to minimize page faults.

; first remove existing y register refresh

	.local

	ldx #0
?1
	ldy #0 
	sty temp			; set temp jump byte holder to show no -ve jump byte
	lda (source_b),y		; get branch byte

	ldy #12				; next jump address will be returned in y. check from widest

	cmp #151			; check against offset to refresh jump
	bcs ?1b				; branch to refresh jump check if definitely a refresh
?1a
	cmp jump_byte1,y		; does this match current jump byte
	beq ?1d				; branch if yes

	dey
	bne ?1a				; do until all possible normal jump bytes
?1b
	cmp jump_byte2-1,y		; does this match current jump byte
	beq ?1c				; branch if yes

	dey
	bne ?1b				; do until all possible refresh, negative add jump bytes
?1c
	lda jump_byte1,y		; return the normal jump byte for the equivalent refresh one
	sta temp			; save in temp jump byte holder
?1d
	iny				; another 1 more to get to next jump

	lda #255
	sta line_remain,x		; set object bytes remaining as maximum

	tya				; save width for each line
	sta line_width,x
	
	lda source_b
	sta line_lo,x			; save source addr for each line
	lda source_b+1
	sta line_hi,x

	inx

	lda temp			; NB num between 1-150 or 0
	beq ?2				; if this jump is normal then branch - no need to change

	lda (source_b),y
	beq ?3				; if next jump is end of object then exit

	lda (source_a),y
	bmi ?2				; if next add is negative add then skip

	ldy #0
	lda temp			; get normal jump address
	sta (source_b),y		; replace existing jump with normal jump address
?2
	lda source_a
	clc
	adc line_width-1,x
	sta source_a			; get to next jump address
	sta source_b
	bcc ?1
	inc source_a+1
	inc source_b+1

	jmp ?1				; always loop
?3
	stx temp			; save index of just after last line

; do bytes remaining

	.local

	clc
	lda #0
?1
	sta line_remain,x		; remaining bytes in object (last line remain + 1 set to zero)

	cmp #30				; once width >=30 no point in carrying on
	bcs ?2				; as this is the cutoff for remaining bytes check

	adc line_width-1,x		; add width of previous line to remaining bytes

	dex				; work backwards
	bpl ?1				; up to all lines
?2

; do refresh add

	.local

	lda #0
	sta y_refresh			; initialize y refresh
	sta page_refresh		; initialize flag to no page fault
	tax				; reset line history
?1
	lda line_width,x		; add width to give next address
	clc
	adc y_refresh
	bcc ?3				; branch if still less than 256 bytes since refresh
?1a
	lda line_lo-1,x
	sta source_b			; get start address of previous line into source_b
	lda line_hi-1,x
	sta source_b+1

	ldy line_width-1,x		; get width of previous line
	lda jump_byte2-2,y		; lookup refresh jump address for it

	ldy #0
	sty page_refresh		; ensure page flag is cleared
	cmp (source_b),y		; check if jump address already a negative or end object
	bne ?2				; branch if it is not

	dex				; try line before for refresh
	bne ?1a				; try until all previous lines
?1b
	beq ?1b				; hang if can't refresh
?2
	sta (source_b),y		; change the previous line to a refresh
	lda line_width,x		; reset y refresh to width of current line
?3
	sta y_refresh

	lda page_refresh		; check if page fault still flagged
	bmi ?3a				; branch if it is

	lda line_lo,x			; check this line
	cmp line_lo-1,x			; against the last line
	bcs ?7				; branch if this line >= last line - they're on same page
?3a
	lda y_refresh
	clc				; NB carry could be set in ?1a - so this clear is needed
	adc line_remain,x		; if more remaining than can fit into current y refresh
	bcs ?3b				; then branch to do refresh now

	lda line_remain+1,x		; if less than 30 bytes left to draw AFTER refresh (the +1)
	cmp #30
	bcc ?6				; page fault refresh not worthwhile - branch past
?3b
	lda line_lo,x			; get address of line into source_b
	sta source_b
	lda line_hi,x
	sta source_b+1

	ldy line_width,x		; get width of line
	lda jump_byte2-2,y		; return refresh jump byte for line. NB this is a -ve number

	ldy #0
	cmp (source_b),y		; check if jump address already a negative or end object
	beq ?6				; branch if yes, to set page refresh to on. (A has -ve number)

	sta (source_b),y		; change current line to a refresh

	tya				; y is 0
	sta y_refresh			; restart y_refresh count
?6
	sta page_refresh		; set page flag to positive for no refresh, -ve for refresh
?7
	inx				; next line
	cpx temp		
	bne ?1				; do until last line

	rts


;
; draw_background
;

draw_background

	.local

	lda #~10101110			; bank 3. turn off os,basic,self test,antic main bank (default)
	sta PORTB			; switch to bank 3 where background data is stored.

	lda #0
	sta temp2			; default top grid overlap to none. store in temp2 for later

	lda top_grid_status
	bne ?0

	jmp ?2				; jmp if there's no grid overlap
?0
	lda top_grid_pos		; top grid
	sec
;	sbc horizon			; less horizon
;	sec
;	sbc #9				; less adjustment
	sbc background_y		; less background y
	
	cmp #8
	bcs ?0a				; branch if >=8
	
	sta temp2			; save pixel overlap figure in temp2 
?0a
;	lda horizon			; get y position of back grid
;	clc
;	adc #9				; convert to screen position
;	tax
	ldx background_y		; get y position of background

	lda x48min9_table_lo,x		; x 48 (minus 9) to get to start y position on screen
;	clc
	adc #5+9			; starts 5 bytes over from left (plus 9) 
	sta BC0+1			; modify lo screen a address in clear line code
	sta BC0+4			; screen b lo address

	lda x48min9_table_hi,x
	adc draw_screen_a		; add hi of x 48 to screen a for address required
	sta BC0+2			; modify hi address in clear line code
	tax
	adc #>screen1b->screen1a	; add screen b offset
	sta BC0+5			; modify hi address in clear line code
	tay

	lda BC0+1
	adc #48				; add 48 to previous line to get start of next line
	sta BC1+1			; modify lo screen a address in line 1 draw code
	sta BC1+4			; screen b lo address

	bcc ?1a
	inx				; carry set so next screen hi byte
	iny
	clc
?1a
	stx BC1+2			; modify hi screen a address in line 1 draw code
	sty BC1+5			; screen b hi address

	adc #48				; line 2
	sta BC2+1
	sta BC2+4

	bcc ?1b
	inx
	iny
	clc
?1b
	stx BC2+2
	sty BC2+5

	adc #48				; line 3
	sta BC3+1
	sta BC3+4

	bcc ?1c
	inx
	iny
	clc
?1c
	stx BC3+2
	sty BC3+5

	adc #48				; line 4
	sta BC4+1
	sta BC4+4

	bcc ?1d
	inx
	iny
	clc
?1d
	stx BC4+2
	sty BC4+5

	adc #48				; line 5
	sta BC5+1
	sta BC5+4

	bcc ?1e
	inx
	iny
	clc
?1e
	stx BC5+2
	sty BC5+5

	adc #48				; line 6
	sta BC6+1
	sta BC6+4

	bcc ?1f
	inx
	iny
	clc
?1f
	stx BC6+2
	sty BC6+5

	adc #48				; line 7
	sta BC7+1
	sta BC7+4

	bcc ?1g
	inx
	iny
	clc
?1g
	stx BC7+2
	sty BC7+5


	ldy temp2			; get which line is top line to draw
	lda clear_branch,y
	sta BC_Branch+1			; modify branch to correct line

	ldy #36
	lda #0
BC7
	sta screen1a,y			; clear line 0
	sta screen1b,y
BC6
	sta screen1a,y			; line 1
	sta screen1b,y
BC5
	sta screen1a,y			; line 2
	sta screen1b,y
BC4
	sta screen1a,y			; line 3
	sta screen1b,y
BC3
	sta screen1a,y			; line 4
	sta screen1b,y
BC2
	sta screen1a,y			; line 5
	sta screen1b,y
BC1
	sta screen1a,y			; line 6
	sta screen1b,y
BC0
	sta screen1a,y			; line 7
	sta screen1b,y

	dey
BC_Branch
	bne BC0
	
?2
	lda x_scroll			; get x scroll
	eor #255
	clc
	adc #1				; reverse direction of x_scroll
	bmi ?3				; branch for subtract of position

	clc
	adc background_x		; add to 2 pixel position of background
	cmp #96*2			; check against maximum background position
	bcc ?4				; branch if less than maximum
	sbc #96*2			; wrap back to 0 (or 1)
	bcs ?4
?3
	clc
	adc background_x
	cmp #96*2
	bcc ?4				; branch if position <192
	adc #96*2-1			; add 192 to wrap back to 192
?4
	sta background_x		; save 2 pixel backgound position

	lsr a				; convert to start byte position save to temp
	sta temp			; NB carry clear if shifted version required
	bcc draw_background_noshift	; branch if the non-shifted version required

	jmp draw_background_shift	; do the shifted version


;
; draw background noshift
;

draw_background_noshift

; non-shifted background draw with check for top grid overlap
; contains self-modifying code

	.local

;	lda horizon			; get y  positions of back grid
;	adc #9				; convert to screen position. NB C is clear
;	tax
	ldx background_y		; get y position of background. NB C is clear

	lda x48min9_table_lo,x		; x 48 (minus 9) to get to start y position on screen
;	clc
	adc #5+9			; starts 5 bytes over from left (plus 9) 
	sta BD0+4			; modify lo screen a address in line 0 draw code
	sta BD0+10			; screen b lo address

	lda x48min9_table_hi,x
	adc draw_screen_a		; add hi of x 48 to screen a for address required
	sta BD0+5			; modify hi address in line 0 draw code
	tax				; save in x for quick reference
	adc #>screen1b->screen1a	; add screen b offset
	sta BD0+11			; modify hi address in line 0 draw code
	tay				; save in y for quick reference

	lda BD0+4
	adc #48				; add 48 to previous line to get start of next line
	sta BD1+4			; modify lo screen a address in line 1 draw code
	sta BD1+10			; screen b lo address

	bcc ?1a
	inx				; carry set so next screen hi byte
	iny
	clc
?1a
	stx BD1+5			; modify hi screen a address in line 1 draw code
	sty BD1+11			; screen b hi address

	adc #48				; line 2
	sta BD2+4
	sta BD2+10

	bcc ?1b
	inx
	iny
	clc
?1b
	stx BD2+5
	sty BD2+11

	adc #48				; line 3
	sta BD3+4
	sta BD3+10

	bcc ?1c
	inx
	iny
	clc
?1c
	stx BD3+5
	sty BD3+11

	adc #48				; line 4
	sta BD4+4
	sta BD4+10

	bcc ?1d
	inx
	iny
	clc
?1d
	stx BD4+5
	sty BD4+11

	adc #48				; line 5
	sta BD5+4
	sta BD5+10

	bcc ?1e
	inx
	iny
	clc
?1e
	stx BD5+5
	sty BD5+11

	adc #48				; line 6
	sta BD6+4
	sta BD6+10

	bcc ?1f
	inx
	iny
	clc
?1f
	stx BD6+5
	sty BD6+11

	adc #48				; line 7
	sta BD7+4
	sta BD7+10

	bcc ?1g
	inx
	iny
	clc
?1g
	stx BD7+5
	sty BD7+11


	ldy temp2			; get which line is top line to draw
	lda background_branch,y
	sta BD_Branch+1			; modify branch to correct line

	ldx temp			; retrieve position in data
	ldy #36				; screen width

	bne BD_Branch			; branch to correct line to draw
BD0
	lda back_0a,x			; background line 0 screen a data
	sta screen1a,y			; this gets modified to screen a position for line 0
	lda back_0b,x			; screen b data
	sta screen1b,y			; screen b (modified) position
BD1
	lda back_1a,x			; line 1
	sta screen1a,y
	lda back_1b,x
	sta screen1b,y
BD2
	lda back_2a,x			; line 2
	sta screen1a,y
	lda back_2b,x
	sta screen1b,y
BD3
	lda back_3a,x			; line 3
	sta screen1a,y
	lda back_3b,x
	sta screen1b,y
BD4
	lda back_4a,x			; line 4
	sta screen1a,y
	lda back_4b,x
	sta screen1b,y
BD5
	lda back_5a,x			; line 5
	sta screen1a,y
	lda back_5b,x
	sta screen1b,y
BD6
	lda back_6a,x			; line 6
	sta screen1a,y
	lda back_6b,x
	sta screen1b,y
BD7
	lda back_7a,x			; line 7
	sta screen1a,y
	lda back_7b,x
	sta screen1b,y

	dex				; next source data
	bmi ?3				; branch if not reached end of source data
?2
	dey
BD_Branch
	bne BD0				; this gets modified to point back to top line of background

	lda #~10110010			; main bank. turn off os,basic,self test,antic main bank
	sta PORTB			; switch back to main bank

	rts				; exit

?3
	ldx #95				; wrap data to join ends of background
	bne ?2				; branch back into routine


;
; draw background shifted
;

draw_background_shift

; shifted background draw with check for top grid overlap
; contains self-modifying code

	.local

;	lda horizon			; get y  positions of back grid
;	adc #8				; convert to screen position. NB c is set
;	tax
	ldx background_y		; get y position of background. NB C is set


	lda x48min9_table_lo,x		; x 48 (minus 9) to get to start y position on screen
;	clc
	adc #5+9-1			; starts 5 bytes over from left (plus 9) 
	sta BDS0+4			; modify lo screen a address in line 0 draw code
	sta BDS0+10			; screen b lo address

	lda x48min9_table_hi,x
	adc draw_screen_a		; add hi of x 48 to screen a for address required
	sta BDS0+5			; modify hi address in line 0 draw code
	tax				; save in x for quick reference
	adc #>screen1b->screen1a	; add screen b offset
	sta BDS0+11			; modify hi address in line 0 draw code
	tay				; save in y for quick reference

	lda BDS0+4
	adc #48				; add 48 to previous line to get start of next line
	sta BDS1+4			; modify lo screen a address in line 1 draw code
	sta BDS1+10			; screen b lo address

	bcc ?1a
	inx				; carry set so next screen hi byte
	iny
	clc
?1a
	stx BDS1+5			; modify hi screen a address in line 1 draw code
	sty BDS1+11			; screen b hi address

	adc #48				; line 2
	sta BDS2+4
	sta BDS2+10

	bcc ?1b
	inx
	iny
	clc
?1b
	stx BDS2+5
	sty BDS2+11

	adc #48				; line 3
	sta BDS3+4
	sta BDS3+10

	bcc ?1c
	inx
	iny
	clc
?1c
	stx BDS3+5
	sty BDS3+11

	adc #48				; line 4
	sta BDS4+4
	sta BDS4+10

	bcc ?1d
	inx
	iny
	clc
?1d
	stx BDS4+5
	sty BDS4+11

	adc #48				; line 5
	sta BDS5+4
	sta BDS5+10

	bcc ?1e
	inx
	iny
	clc
?1e
	stx BDS5+5
	sty BDS5+11

	adc #48				; line 6
	sta BDS6+4
	sta BDS6+10

	bcc ?1f
	inx
	iny
	clc
?1f
	stx BDS6+5
	sty BDS6+11

	adc #48				; line 7
	sta BDS7+4
	sta BDS7+10

	bcc ?1g
	inx
	iny
	clc
?1g
	stx BDS7+5
	sty BDS7+11


	ldy temp2			; get which line is top line to draw
	lda background_branch,y
	sta BDS_Branch+1		; modify branch to correct line

	ldx temp			; retrieve position in data
	ldy #36				; screen width

	bne BDS_Branch			; branch to correct line to draw
BDS0
	lda back_0as,x			; background line 0 screen a data
	sta screen1a,y			; this gets modified to screen a position for line 0
	lda back_0bs,x			; screen b data
	sta screen1b,y			; screen b (modified) position
BDS1
	lda back_1as,x			; line 1
	sta screen1a,y
	lda back_1bs,x
	sta screen1b,y
BDS2
	lda back_2as,x			; line 2
	sta screen1a,y
	lda back_2bs,x
	sta screen1b,y
BDS3
	lda back_3as,x			; line 3
	sta screen1a,y
	lda back_3bs,x
	sta screen1b,y
BDS4
	lda back_4as,x			; line 4
	sta screen1a,y
	lda back_4bs,x
	sta screen1b,y
BDS5
	lda back_5as,x			; line 5
	sta screen1a,y
	lda back_5bs,x
	sta screen1b,y
BDS6
	lda back_6as,x			; line 6
	sta screen1a,y
	lda back_6bs,x
	sta screen1b,y
BDS7
	lda back_7as,x			; line 7
	sta screen1a,y
	lda back_7bs,x
	sta screen1b,y

	dex				; next source data
	bmi ?3				; branch if not reached end of source data
?2
	dey
BDS_Branch
	bne BDS0			; this gets modified to point back to top line of background

	lda #~10110010			; main bank. turn off os,basic,self test,antic main bank 
	sta PORTB			; switch back to main bank

	rts				; exit

?3
	ldx #95				; wrap data to join ends of background
	bne ?2				; branch back into routine


;
; player/missile data area
;

;
; bank a & b
;

man_data_0b
	.byte 0,0,0,0
	.incbin "Converted graphics/man9a.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man10a.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man11a.bin"
	.byte 0,0,0,0
man_data_0a
	.incbin "Converted graphics/man6a.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man0a.bin"
	.incbin "Converted graphics/man1a.bin"
	.incbin "Converted graphics/man2a.bin"
	.incbin "Converted graphics/man3a.bin"
	.incbin "Converted graphics/man4a.bin"
	.incbin "Converted graphics/man5a.bin"
;	.incbin "Converted graphics/man6a.bin"
	.incbin "Converted graphics/man7a.bin"
	.incbin "Converted graphics/man8a.bin"

man_data_1b
	.byte 0,0,0,0
	.incbin "Converted graphics/man9b.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man10b.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man11b.bin"
	.byte 0,0,0,0
man_data_1a
	.incbin "Converted graphics/man6b.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man0b.bin"
	.incbin "Converted graphics/man1b.bin"
	.incbin "Converted graphics/man2b.bin"
	.incbin "Converted graphics/man3b.bin"
	.incbin "Converted graphics/man4b.bin"
	.incbin "Converted graphics/man5b.bin"
;	.incbin "Converted graphics/man6b.bin"
	.incbin "Converted graphics/man7b.bin"
	.incbin "Converted graphics/man8b.bin"

man_data_2b
	.byte 0,0,0,0
	.incbin "Converted graphics/man9c.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man10c.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man11c.bin"
	.byte 0,0,0,0
man_data_2a
	.incbin "Converted graphics/man6c.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man0c.bin"
	.incbin "Converted graphics/man1c.bin"
	.incbin "Converted graphics/man2c.bin"
	.incbin "Converted graphics/man3c.bin"
	.incbin "Converted graphics/man4c.bin"
	.incbin "Converted graphics/man5c.bin"
;	.incbin "Converted graphics/man6c.bin"
	.incbin "Converted graphics/man7c.bin"
	.incbin "Converted graphics/man8c.bin"

man_data_3b
	.byte 0,0,0,0
	.incbin "Converted graphics/man9d.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man10d.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man11d.bin"
	.byte 0,0,0,0
man_data_3a
	.incbin "Converted graphics/man6d.bin"
	.byte 0,0,0,0
	.incbin "Converted graphics/man0d.bin"
	.incbin "Converted graphics/man1d.bin"
	.incbin "Converted graphics/man2d.bin"
	.incbin "Converted graphics/man3d.bin"
	.incbin "Converted graphics/man4d.bin"
	.incbin "Converted graphics/man5d.bin"
;	.incbin "Converted graphics/man6d.bin"
	.incbin "Converted graphics/man7d.bin"
	.incbin "Converted graphics/man8d.bin"


;
; mem remaining help. put all 3000-3fff data before this
;

mem_position_3000-3fff


; 
;
; routines that are not bank switch sensitive.
;

	*=$4000

;
; sample data
;

argh
.incbin "argh.bin"
@argh

getready
.incbin "getready.bin"
@getready


;
; game commands
;

do_init_stage

; initialize stage game command

	.local

	jsr init_stage

	jmp game_command_rtn	


do_init_boss

; initialize a boss

	.local

	ldx pattern_command_reqd	; get pattern command number required (passed from stream)
	ldy game_command_argument	; get song required (passed from stream)
	
	lda pattern_command_lo-1,x	; lookup address of pattern command number
	sta pattern_command
	lda pattern_command_hi-1,x
	sta pattern_command+1
	
	lda #0
	sta music_pat_index		; stop current music
	sta last_alien			; reset last alien for dragon priorities
	sta last_handler		; reset last handler for correct rotate object assignments
	
	jsr init_music			; y has song required
	
	jmp game_command_rtn		; return


do_grid_down

; top grid down

	.local

	ldx #20				; 21 top grid positions
?0
	lda tilt_lo,x			; remove tilt from top grid so starts from position 2
	eor #255
	sec
	sbc #127			; turn -ve but add back 128 to line up back of grid when down
	sta top_grid_lo,x
	
	lda tilt_hi,x
	eor #255
	adc #3				; turn hi byte -ve and add an extra 2 so starts from position 2
	sta top_grid_hi,x
	
	dex
	bne ?0				; do for all grid positions except 0 which is fixed at 2

	lda #top_grid_down		; turn on top grid down with bit 6, grid on with bit 0
	bne ?1				; always branch


do_grid_up

; top grid up

	lda #top_grid_up		; turn on top grid up with bit 7, grid on with bit 0
?1
	sta top_grid_status		; top grid on with non zero
	
	jmp game_command_rtn		; return


do_init_pattern_command

; generic initialize pattern command
	
	ldx pattern_command_reqd	; get pattern command number required (passed from stream)
	ldy game_command_argument	; get variable passed from stream

	sty pattern_temp		; save passed variable for later
	
	lda pattern_command_lo-1,x	; lookup address of pattern command number
	sta pattern_command
	lda pattern_command_hi-1,x
	sta pattern_command+1

	jmp game_command_rtn		; return


do_reset_last_alien

; reset next alien to 0 for highest priority in uriah man jump off
; NB This is only required because I should have had uriah's biggest frame at z=1 instead of z=0
; - too much effort has gone into timing uriah to work at z=0 than to change him to be at z=1

	.local

	lda #0
	sta last_alien			; reset last alien for priorities

	jmp game_command_rtn


;
; pattern commands
;


do_swap_priority

; swap sprite priority pattern command 

	lda priority_status
	eor #1
	sta priority_status

do_fireball_speedup

; speedup boss fireballs

	.local

	lda missile_speed+1		; get current fireball speed
	cmp #3
	bcc ?1				; branch past if already maximum speed

	sbc #2				; speed up fireballs
	sta missile_speed+1
?1
	jmp pattern_command_rtn		; return


do_total_counters

; add counters score to score (and control object at same time)

	.local
	
	tya
	pha
	txa
	pha				; save x and y regs

	ldx counters_small		; small counter needs erasing
	bne ?2				; branch if no wrap

	txa
	ora counters_big
	beq ?4				; exit if no more counters to total

	inx
?1
	stx counters_small

	jsr draw_small_counter		; draw small counter		

	ldx counters_small
	inx
	cpx #9
	bne ?1				; do until all 8 small counters

	ldx counters_big
	
	jsr draw_big_counter		; erase current big counter

	dec counters_big

	bpl ?3				; always branch
?2
	jsr draw_small_counter		; erase small counter

	dec counters_small
?3
	ldx #3				; 10,000 digit
	lda #5				; and value 5 to give 50,000

	jsr add_score
	
	lda #counterfx
	sta sound_queue
?4
	pla
	tax				; restore x reg

	jmp ?5

	
do_control_object

; move object based on joystick position. override pattern x vector while this command is active
; assumes last_handler and last_alien have been reset beforehand


	tya
	pha				; save y reg
?5
	ldy pattern_temp		; object number being controlled

	lda PORTA

	lsr a				; ignore up/down bits
	lsr a

	lsr a
	bcs ?6				; branch if not left

	lda x_pos_2d+12,y
	cmp #128-72+20
	bcc ?8				; branch if off screen left

	lda #252			; set left
	bne ?7				; always branch 
?6
	lsr a
	bcs ?8				; branch if not right

	lda x_pos_2d+12,y
	cmp #128+72-17
	bcs ?8				; branch if off screen right

	lda #4				; set right
?7
	sta x_vector_buffer,x		; save in vector_buffer
?8
	pla
	tay				; restore y reg

	jmp pattern_command_rtn


do_chase_object

; make object chase next object along, to centre on its position (ie man ready to jump onto uriah) 
; assumes last_handler and last_alien have been reset beforehand

	.local

	lda pattern_temp		; pattern_temp has uriahs pattern handler
	cmp temp			; temp has pattern handler of current pattern processing
	bne ?3				; branch if this pattern is not uriah

	lda #<?2
	sta pattern_command		; set pattern command to only call priority swap once
	lda #>?2
	sta pattern_command+1

	txa				; save registers
	pha
	tya
	pha


	ldy pattern_temp		; get handler
	ldx vector_buffer_limits,y	; get start of vector buffer
	
	ldy #vector_buffer_size		; counter
	
	lda #0
?1	
	sta z_vector_lo_buffer,x
	sta z_vector_buffer,x		; freeze z movement (of uriah)

	inx				; next vector buffer

	dey
	bne ?1				; do for all vector buffers of this handler
	
	stx priority_status		; turn on reverse object priorities (non zero)

	pla
	tay
	pla
	tax				; restore registers
	
	jmp pattern_command_rtn		; exit
	
	
?2
	lda pattern_temp		; check handler against uriah handler
	cmp temp
	beq ?6				; skip this if uriah handler
?3
	lda x_pos_2d+12+2		; x position of next object along (uriah)
	and #~11111100			; to nearest 4 pixels only (avoid centring jitter)
	cmp x_pos_2d+12+1		; check against object (man) - which moves at 4 pixel jumps
	
	beq ?6				; branch if same position
	bcc ?4				; branch if object further right
	
	lda #4
	bne ?5				; object to left of next - set to move right
?4
	lda #252			; object to right of next - set to move left		
?5	
	sta x_vector_buffer,x
?6		
	jmp pattern_command_rtn		; exit


do_centre_object

; head towards centre of screen and stop. (uriah movement before man gets off)

	.local

	tya
	pha

	ldy pattern_temp

	lda x_pos_2d+12,y
	and #~11111000
	cmp #128
	
	beq ?3
 	bcc ?1
 	
 	lda #252
 	bne ?2
?1
 	lda #4
?2
 	sta x_vector_buffer,x
?3
 	pla
 	tay
 	
	jmp pattern_command_rtn


;
; stop sound
;

stop_sound

; stop sound for all 4 channels

	.local

	lda #0
	ldx #7				; 4 channels *2 because interleaved registers
?1
	sta AUDC1-1,x			; switch off volume

	dex
	dex
	bpl ?1				; do for all channels

	sta sound_queue			; stop any queued sounds

	rts


;
; move objects
;

move_objects

; move scenery, aliens, missiles and bullets


;
; scenery
;

; get next scenery to display. wait for each timer before fetching next object
; there are 2 streams
; call move scenery for active scenery objects

	.local
?0
	ldx #7				; for all 8 scenery objects
	lda #0
	sta temp			; object setup counter
?1
	lda properties+24,x
	and #hide_bit+dead_bit
	beq ?5				; branch if not explosion and not hidden
	bpl ?6				; branch if explosion

	lda control+12,x
	beq ?7				; branch if no new object required

	lsr a				; get 3-in-a-row bit 0 into carry
	tay				; get just object number into y
	bcc ?2				; branch if not a 3-in-a-row object

	lda #0				; stop object being re-selected. 3-in-a-row is a one off setup
	sta control+12,x


	lda RANDOM
	and #~00000011			; random number between 0-3
	beq ?7				; to give gap, 1 in 4 chance of this object not 3-in-a-row

	lda scenery_x,x
	sta x_pos+24,x			; give object pre-defined x spacing

	bcs ?4				; always branch to common object setup
?2

	lda temp			; don't setup if already setup another object
	bmi ?7				; ie only 1 object per z position.

	dec temp			; flag objects as setup

	lda RANDOM			; random number

	lsr a

	bcc ?7				; 1 in 2 chance of setup 

	lsr a				; random number 0-63

	bcc ?3				; 1 in 2 chance of being positive

	eor #255			; convert to negative
	adc #0	
?3
	sta x_pos+24,x

?4
	lda object_sound,y
	beq ?4a

	sta sound_queue			; queue sound fx for object (if any)
?4a
	lda object_to_slot,y		; convert to current slot number
	sta slot+24,x

	lda object_properties,y		; get properties of the object
	sta properties+24,x

	and #sine_bit+low_bit		; get whether sine wave reqd, and y position of scenery object 
	tay				; into y for y position lookup

	and #sine_bit			; get just whether sine wave required
	beq ?4b				; branch if not

	lda RANDOM
	and #15				; get random start in sine wave 0-15
	clc
	adc #1				; add 1 to ensure non zero
?4b
	sta y_sine,x			; save sine position

	lda scenery_y,y			; get scenery y position required

	sta y_pos+24,x

	lda #21
	sta z_pos+24,x			; set it up in the background (1 more than max z pos)
?5
	jmp move_scenery		; move active object
?6
	jmp move_scenery_explosion	; move scenery explosion
?7
move_scenery_rtn
move_scenery_explosion_rtn

	dex
	bpl ?1				; do until all scenery


;
; pattern handler
;

	.local

	ldx #max_handlers-1		; number of pattern handlers
?1
	lda pat_kill,x
	beq ?4				; branch if pattern not in use

	dec pat_timer,x			; timer till next pattern segment
	bne ?3				; branch if not time yet

	lda pat_hi,x
	sta source_a+1			; save pattern address in zp
	lda pat_lo,x
	sta source_a

	lda pat_vector,x		; get current vector pointer

	cmp vector_buffer_limits,x	; check buffer limits (offsets + 1)
	bcs ?2				; branch if vector pointer >= buffer limit

	adc #vector_buffer_size+1	; reset to buffer start. allow for borrow on subtract 
?2
	sbc #1				; move pointer down
	sta pat_vector,x		; save next pointer.

	ldy pat_index,x			; get current position in pattern

	jmp decode_pattern		; decode the compressed vectors and functions
?3
decode_pattern_rtn
	lda pat_qty,x			; get qty of objects left to start
	beq ?4				; branch if none

	dec pat_start_timer,x		; timer till an object needs starting
	bne ?4				; branch if not time yet

	lda pat_hi,x			; save pattern address in zp
	sta source_a+1
	lda pat_lo,x
	sta source_a
?3a
	dec pat_qty,x			; reduce count of objects to start

	jsr init_alien			; initialize an alien

	lda pat_spacing,x		; setup timer for next object start
	sta pat_start_timer,x
?4
	dex
	bpl ?1				; do until all pattern handlers


;
; sum pattern
;

	.local

	ldx sum_status			; get the sum pattern
	bmi ?7				; branch past if not in use

	lda active_pat_count		; get number of active patterns
	cmp #1
	bne ?4				; branch if other patterns still going

	lda #128
	sta sum_status			; nothing left to control - finish sum pattern

	lda pat_kill,x
	bpl ?1				; branch if sum pattern has its own object (-ve set if none)

	eor #128			; turn off force no object bit, to return a zero object count	

	sta pat_kill,x			; stop the pattern handler. As there are no objects in this
	sta active_pat_count		; sum pattern to do this, we must do it ourselves. 

	jmp ?7				; exit

?1
	txa				; restore sum pattern
	ldy #11				; 12 aliens			
?2
	cmp handler,y
	bne ?3				; branch if alien not sum object
	
	lda #0
	sta vector,y			; stop sum pattern's object moving

	lda #3				; flag its timer so it dies after a short pause
	sta timer,y

	txa				; restore sum pattern
?3
	dey
	bpl ?2				; do for all aliens
	
	jmp ?7				; exit

?4
	ldy pat_vector,x		; get sum pattern vector

	ldx #11				; do for all aliens
?5
	lda properties+12,x
	and #dead_bit
	bne ?6				; branch past if object dead or an explosion

	lda control,x
	and #sum_bit*2			; NB sum_bit is shifted in init_alien
	bne ?6				; branch past if object is the sum pattern object

	lda x_vector_buffer,y		; add current x vector to object
	clc
	adc x_pos+12,x
	sta x_pos+12,x

	lda y_vector_buffer,y		; add current y vector to object
	clc
	adc y_pos+12,x
	sta y_pos+12,x

	lda z_vector_lo_buffer,y	; add current z lo and hi vector to object
	clc
	adc z_pos_lo+6,x
	sta z_pos_lo+6,x

	lda z_vector_buffer,y
	adc z_pos+12,x
	sta z_pos+12,x
?6
	dex				; do until all aliens
	bpl ?5
?7


;
; aliens
;

	.local

	ldx #11				; up to 12 aliens
?1
	lda properties+12,x
	and #hide_bit+dead_bit		; branch if object dead or an explosion
	bne ?5

	lda z_pos+12,x
	bmi ?7				; -ve z kill alien

	dec timer,x			; count down till next pattern segment
	bne ?4				; branch past till ready

	ldy handler,x			; get pattern handler being used by object

	lda vector,x			; get current vector pointer
	beq ?6				; if vector zero then object must die

	cmp vector_buffer_limits,y	; check buffer limits (offsets plus 1)
	bcs ?2				; branch if vector pointer >= buffer limit

	adc #vector_buffer_size+1	; reset to buffer start. allow for borrow on subtract 
?2
	sbc #1				; move pointer down
	sta vector,x			; save next pointer.

	tay				; use pointer
	
	lda ftimer_buffer,y		; get timer until next vector change
	beq ?7				; if timer zero then end of pattern
	bpl ?3				; branch if not required to fire

	jmp init_missile		; fire a missile from alien

init_missile_rtn

;	lda ftimer_buffer,y		; restore accumulator in init_missile to make room
;	and #255-fire_bit		; remove fire bit from timer
?3
	sta timer,x			; save timer till next vector


	lda frame_change_buffer,y	; check whether change of animation frame required 
	beq ?4				; branch if no change

;	cmp #254
;	bne ?3a
;
;	lda object,x
;	eor #~00000001
;	bne ?3b
;?3a
;	clc
;	adc object,x			; add +1 or -1 to object number - so objects for animating
;?3b
;	sta object,x			; must be next to each other. This cannot be done with slot

	lsr a				; mirror frame bit into C
	bcc ?3a				; branch if frame animation instead
	eor object,x			; mirror object. if 0 then 1, if 1 then 0 
	
	bne ?3b
?3a	
	adc object,x
	sbc #15				; c is clear convert to -16 to +15 frame skip value
?3b
	sta object,x			; save new object frame

	tay				; convert to slot number
	lda object_to_slot,y
	sta slot+12,x

	lda object_properties,y		; apply new objects properties to alien
	and #~01111111			; remove mirror bit
	sta properties+12,x

	ldy vector,x			; restore vector pointer
?4
	jmp move_alien			; move alien by vectors. Here coz of slot change above
?5
	bmi ?8				; branch if object not in use

	jmp move_alien_explosion	; move alien explosion
?6
	jmp kill_alien			; kill controlled linked alien
?7

	txa
	tay				; save x in a. make y same as x
	
	ldx handler,y			; get handler

	dec pat_kill,x			; reduce qty aliens left in pattern
	bne ?7a
	
	dec active_pat_count		; if all dead decrease active pattern count
?7a
	tax				; restore x

	lda #dead_bit+hide_bit		; flag object dead and stop an explosion by hiding it
	sta properties+12,x
	sta z_pos+12,x			; set z to -ve for sort routine
?8
move_alien_rtn
move_alien_explosion_rtn
kill_alien_rtn

	dex
	bpl ?1				; do until all aliens


	lda uriah_status
	beq rotate_dragon		; if not uriah stage check rotate and do missiles and bullets
	
	rts				; if uriah stage skip rotate, missiles and bullets


;
; rotate dragon
;

rotate_dragon

; position 3 dragon segments evenly between each rotating head and the tail section
; there are 2 heads and one tail, 3 segments for each head.
; assume we know object numbers (init_boss must reset last_alien for this to work)
; assume centre sum segment is really at centre for now (x=0)
; tail as the sum object has to be setup last or its check for patterns going will stop it instantly
; there should only be 4 objects in each pattern for the simple divide by 4 idea to work
; objects will get assigned in following order by handler:
; head1,segment1,head2,segment2,tail,segment1,segment2,segment1,segment2

	.local
	
	lda rotate_status
	bne ?0				; exit if status at zero

	jmp move_missiles
?0
	lda properties+12+1
	and #hide_bit+dead_bit
	bne ?5				; skip if head dead or explosion

	lda properties+12+8
	and #hide_bit+dead_bit
	bne ?5				; skip if last segment not setup yet - dead or explosion


	lda x_pos+12+1			; head x
	bmi ?1				; branch for -ve code

	lsr a
	sta x_pos+12+6			; save x divide by 2 in middle segment

	lsr a
	sta x_pos+12+8			; save x divide by 4 in last segment

	adc x_pos+12+6

	bpl ?2				; always branch
?1
	sec
	ror a
	sta x_pos+12+6

	sec
	ror a
 	sta x_pos+12+8

	adc x_pos+12+6
?2
	sta x_pos+12+2			; save x divide by 4 * 3 in first segment

	
	lda z_pos+12+5			; tail segment z
	sec
	sbc z_pos+12+1			; less head z

	bmi ?3				; branch for -ve code

	lsr a
	sta z_pos+12+6			; z divide by 2 in middle segment

	lsr a
	sta z_pos+12+2			; z divide by 4 in first segment

	adc z_pos+12+6			; z divide by 4 * 3 for last segment

	bpl ?4				; always branch
?3
	sec
	ror a
	sta z_pos+12+6

	sec
	ror a
	sta z_pos+12+2

	adc z_pos+12+6
?4
	clc
	adc z_pos+12+1			; add back head  z position
	sta z_pos+12+8			; to last segment

	lda z_pos+12+1			; add back head z
	clc				; 
	adc z_pos+12+6			; to middle segment
	sta z_pos+12+6

	lda z_pos+12+1			; add back head z
	clc
	adc z_pos+12+2			; to first segment
	sta z_pos+12+2

?5
	lda properties+12+3		; 2nd head
	and #hide_bit+dead_bit
	bne ?10				; skip if head dead or explosion

	lda properties+12+9
	and #hide_bit+dead_bit
	bne ?10				; skip if last segment not setup yet - dead or explosion


	lda x_pos+12+3			; head x
	bmi ?6				; branch for -ve code

	lsr a
	sta x_pos+12+7			; save x divide by 2 in middle segment

	lsr a
	sta x_pos+12+9			; save x divide by 4 in last segment

	adc x_pos+12+7

	bpl ?7				; always branch
?6
	sec
	ror a
	sta x_pos+12+7

	sec
	ror a
 	sta x_pos+12+9

	adc x_pos+12+7
?7
	sta x_pos+12+4			; save x divide by 4 * 3 in first segment

	
	lda z_pos+12+5			; tail segment z
	sec
	sbc z_pos+12+3			; less head z

	bmi ?8				; branch for -ve code

	lsr a
	sta z_pos+12+7			; z divide by 2 in middle segment

	lsr a
	sta z_pos+12+4			; z divide by 4 in first segment

	adc z_pos+12+7			; z divide by 4 * 3 for last segment

	bpl ?9				; always branch
?8
	sec
	ror a
	sta z_pos+12+7

	sec
	ror a
	sta z_pos+12+4

	adc z_pos+12+7
?9
	clc
	adc z_pos+12+3			; add back head  z position
	sta z_pos+12+9			; to last segment

	lda z_pos+12+3			; add back head z
	clc				; 
	adc z_pos+12+7			; to middle segment
	sta z_pos+12+7

	lda z_pos+12+3			; add back head z
	clc
	adc z_pos+12+4			; to first segment
	sta z_pos+12+4
?10


;
; move missiles
;

move_missiles

; move alien missiles (using Bresenham line draw method which avoids division to calculate slope)

	.local

	ldx #5				; 6 missiles
?1
	lda properties+6,x
	bmi ?5				; branch past if hidden

	lda z_pos_lo,x			; reduce amount till next z step of missile line
	sec
	sbc z_length,x
	sta z_pos_lo,x
	bcs ?2				; branch if not yet

	adc max_length,x		; add back maximum step amount
	sta z_pos_lo,x

	dec z_pos+6,x			; add to hi byte z vector
	bmi ?6				; branch if just off front of screen
	sec
?2
	lda x_pos_lo,x			; reduce amount till next x step of missile line
	sbc x_length,x
	sta x_pos_lo,x
	bcs ?3				; branch if not yet

	adc max_length,x		; add back maximum step amount
	sta x_pos_lo,x

	lda x_pos_2d+6,x		; add hi byte x vector
	clc
	adc x_vector+6,x
	sta x_pos_2d+6,x
	sec
?3
	lda y_pos_lo,x			; reduce amount till next y step of missile line
	sbc y_length,x
	sta y_pos_lo,x
	bcs ?4				; branch if not yet

	adc max_length,x		; add back maximum step amount
	sta y_pos_lo,x

	lda y_pos_2d+6,x		; add hi byte y vector
	clc
	adc y_vector+6,x
	sta y_pos_2d+6,x
?4
	ldy z_pos+6,x
	lda size_map,y			; convert z pos to a frame number 0-10
	ldy slot+6,x			; get slot number in y
	clc
	adc x11_table,y			; add frame no to slot*11
	tay				; put in y
	lda width_table,y		; lookup width
	sta x_size+6,x
	lda height_table,y		; full height of object (less 1)
	lsr a				; 1/2 height of object
	adc y_pos_2d+6,x		; add (rounded) to y position 
	sta y_bottom+6,x		; to give y bottom

	cmp #90+43			; check bottom of object to screen bottom
	bcs ?6				; branch to hide if too close

;	sec
	sbc height_table,y		; subtract full height of object
	sta y_top+6,x			; to give y top
?5
	dex				; do until all missiles
	bpl ?1
	bmi ?7				; exit move missiles
?6
	lda #hide_bit			; missile done. hide it.
	sta properties+6,x
	sta z_pos+6,x

	dex				; do until all missiles
	bmi ?7				; exit move missiles

	jmp ?1
?7


;
; move bullets
;

; move and setup new bullets from man

	.local

	ldy #255
	sty temp			; use to indicate free bullet. -ve means none free

	ldy #5				; number of bullets
?1
	lda properties,y
	and #hide_bit+bounce_bit	; get just whether bullet off or a bounce bullet

	bmi ?3				; branch if switched off
	beq ?1a				; branch if normal bullet

	lda z_pos,y
	sec				; move bounce bullet back towards player
	sbc #2
	bcc ?2				; branch if z out of range (less than 0)

	sta z_pos,y			; z for frame referrence
	tax

	lda x_pos_2d,y
	clc
	adc x_vector,y			; move bounce bullet by x vector

	cmp #128+72+20			; check middle to right side of screen plus overlap
	bcs ?2				; branch if off screen

	cmp #128-72-20			; check middle to left side of screen less overlap
	bcc ?2				; branch if off screen

	sta x_pos_2d,y			; save x position

	lda y_pos_2d,y
	clc
	adc y_vector,y			; move bounce bullet by y vector

	cmp #90-42-10			; check middle to top of screen less overlap
	bcc ?2				; branch if off screen

	cmp #90+42-10			; check middle to bottom of screen less safety margin
	bcs ?2				; - crashes if overlap bottom. branch if off screen

	sta y_pos_2d,y			; save y position

	bcc ?1b				; always branch (NB a clear carry is important)
?1a	
	lda z_pos,y
	cmp #18
	bcs ?2				; branch if z out of range (z=20 is maximum)

	adc #2
	sta z_pos,y			; move bullet forward 2

	tax
?1b
	lda size_map,x			; convert z pos to a frame number 0-10
	ldx slot,y			; get slot number in x
	adc x11_table,x			; add frame no to slot*11. NB Carry clear
	tax				; put in x
	lda width_table,x		; lookup width
	sta x_size,y
	lda height_table,x		; full height of object (less 1)
	lsr a				; 1/2 height of object
	adc y_pos_2d,y			; add (rounded) to y position 
	sta y_bottom,y			; to give y bottom
	sec
	sbc height_table,x		; subtract full height of object
	sta y_top,y			; to give y top

	dey				; do until all bullets
	bpl ?1
	bmi ?4				; branch to bullet setup check
?2
	lda #hide_bit

	sta properties,y		; switch object off
	sta z_pos,y			; set z pos -ve for sort routine
?3
	sty temp			; flag as free bullet

	dey				; do until all bullets
	bpl ?1

; check for setup of new bullet

?4
	lda bullet_count		; see if any bullets are required
	beq ?6 

	ldy temp			; check if available bullet object
	bmi ?6				; branch if no

	lda #0
	sta bullet_count		; flag bullet as having been setup

	sta z_pos,y			; z to 0

	lda man_x_pos
	sta x_pos_2d,y			; convert man x position to bullet position

	lda man_y_pos

	clc
	adc #90-74

	sta y_pos_2d,y			; convert man y position to bullet position

	ldx slot,y			; get slot number in y
	lda x11_table,x			; get slot*11
	tax				; put in x
	lda width_table,x		; lookup width of z=0 entry
	sta x_size,y
	lda height_table,x		; full height of object (less 1)
	lsr a				; 1/2 height of object
	adc y_pos_2d,y			; add (rounded) to y position 
	sta y_bottom,y			; to give y bottom
	sec
	sbc height_table,x		; subtract full height of object
	sta y_top,y			; to give y top

	lda #harmless_bit+passthru_bit

	sta properties,y		; switch bullet on

	lda sound_queue
	bne ?6				; don't make bullet noise if other sound queued
	
	lda #bulletfx
	sta sound_queue			; queue bullet sound fx

?6
move_objects_rtn

	rts


;
; initialize missile
;

init_missile

; setup missile type and vector. firing alien's number passed in x reg

	.local

	ldy last_missile		; get last missile used into y
	dey				; get next missile along
	bpl ?1

	ldy #5				; wrap back if needed
?1
	sty last_missile		; save for next check
	lda properties+6,y
	bmi ?2				; branch if missile is free

	ldy vector,x			; restore vector buffer

	lda ftimer_buffer,y		; restore accumulator
	and #255-fire_bit		; remove fire bit from timer
	
	jmp init_missile_rtn		; exit
?2
	stx temp			; save alien number

;	lda y_pos_2d+12,x
	lda y_bottom+12,x
	clc
	adc y_top+12,x
	lsr a				; get y centre of object


	sta y_pos_2d+6,y		; set missile y to alien y

	lda z_pos+12,x
	sta z_pos+6,y			; set missile z to alien z

 	lda x_pos_2d+12,x
	sta x_pos_2d+6,y		; set missile x to alien x


	lda properties+12,x
	and #missile_bits		; get missile type alien fires
	lsr a
	lsr a

	tax				; missile type in x

	lda missile_speed,x
	sta temp3			; save missile type speed

	lda object_to_slot,x		; get slot number

	sta slot+6,y			; set missile slot num

	lda #low_bit+high_bit+passthru_bit
	sta properties+6,y		; set missile properties

	lda object_sound,x		; get sound fx for missile type
	sta sound_queue			; queue sound fx


	lda x_pos_2d+6,y
	sec				; subtract man x from missile x
	sbc man_x_pos

	php				; save x vector direction (in c)
	bcs ?3				; branch if missile to right of man

	eor #255			; convert difference to +ve number
	adc #1				; NB this works even though the difference can be >128
?3
	lsr a				; scale x down by 8 for z comparison
	lsr a
	lsr a

	adc #0				; round up to nearest

	sta x_length,y			; x length missile has to travel

	lda y_pos_2d+6,y		; NB man middle of screen=76. object middle of screen=90

	sbc #13				; No need for sec now since NEW round up ensures c is clear

	sbc man_y_pos			; difference between missile y and man y

	php				; save y vector direction (in c)
	bcs ?4				; branch if missile under man

	eor #255			; convert difference to +ve number
	adc #1
?4
	lsr a				; scale y down by 4 for z comparison
	lsr a

	sta y_length,y			; y length missile has to travel

	lda z_pos+6,y
	sta z_length,y			; z length missile has to travel

	cmp x_length,y			; find longest length - x,y or z
	bcs ?5				; branch if x length less than z length

	lda x_length,y			; set longest length to x length
?5
	cmp y_length,y
	bcs ?6				; branch if y length less than longest length

	lda y_length,y			; set longest length to y length
?6
	cmp temp3			; if missile closer than missile speed dictates then 
	bcs ?7				; set missile to take missile speed steps as a minimum

	lda temp3
?7
	sta max_length,y		; save longest length

	lsr a

	sta x_pos_lo,y			; round up - initial steps to 1/2 longest length
	sta y_pos_lo,y
	sta z_pos_lo,y


	plp				; retrieve y vector direction
	bcs ?9				; branch if missile under man

	ldx #4				; maximum vector speed is 4
	lda y_length,y
	asl a
	cmp max_length,y		; if 2*y length is less than maximum length, then y can move
	bcs ?10				; in 2 pixel jumps rather than 4 pixel jumps of maximum length

	ldx #2				; 2 pixel vector speed
	sta y_length,y			; save 2*y vector length
	asl a				; try 4*y length now to see if 1 pixel jump is possible
	cmp max_length,y
	bcs ?10

	ldx #1				; 1 pixel vector speed
	sta y_length,y			; save 4*y vector length

	bne ?10				; always branch
?9
	ldx #256-4			; as above but for missile above man vector direction
	lda y_length,y
	asl a
	cmp max_length,y
	bcs ?10

	ldx #256-2
	sta y_length,y
	asl a
	cmp max_length,y
	bcs ?10

	ldx #256-1
	sta y_length,y
?10
	txa
	sta y_vector+6,y

?11
	plp				; retrieve x vector direction
	bcs ?13				; branch if missile to right of man

	ldx #8				; maximum vector speed is 8
	lda x_length,y
	asl a
	cmp max_length,y		; if 2*x length is less than maximum length, then x can move
	bcs ?14				; in 4 pixel jumps rather than 8 pixel jumps of maximum length

	ldx #4				; 4 pixel vector speed
	sta x_length,y			; save 2*x vector length
	asl a				; try 4*x length now to see if 2 pixel jump is possible
	cmp max_length,y
	bcs ?14

	ldx #2				; 2 pixel vector speed
	sta x_length,y			; save 4*y vector length

	bne ?14				; always branch
?13
	ldx #256-8			; as above but for missile to left of man vector direction
	lda x_length,y
	asl a
	cmp max_length,y
	bcs ?14

	ldx #256-4
	sta x_length,y
	asl a
	cmp max_length,y
	bcs ?14

	ldx #256-2
	sta x_length,y
?14
	txa
	sta x_vector+6,y

?15
	ldx temp			; restore alien number
	ldy vector,x			; restore vector buffer


	lda ftimer_buffer,y		; restore accumulator
	and #255-fire_bit		; remove fire bit from timer

	jmp init_missile_rtn		; exit


;
; sequencer
;

sequence_events

	.local

	dec sequencer_timer		; timer till next pattern
	bmi ?0				; branch if wait is active
	beq ?2				; branch if time for next event
	
	rts				; not time yet, exit
?0
	inc sequencer_timer		; ensure timer remains -ve
	lda active_pat_count		; wait for active patterns. (0 when no active patterns)

	bne ?5				; branch if there are active patterns

	ldx #11				; 12 aliens
?1
	lda properties+12,x
	bpl ?5				; exit if any alien is still exploding

	dex
	bne ?1				; do for all aliens
?2
	ldy sequencer_pointer		; get current sequencer index
	inc sequencer_pointer		; next index

	lda stream_object,y		; get sequencer object
	bne ?3				; branch if an alien pattern required

	lda stream_game_command,y
	sta game_status			; set game status to the game command required
	lda stream_game_argument,y
	sta game_command_argument	; argument for use at game command time
	lda stream_pat_command,y
	sta pattern_command_reqd	; pattern command required

	bpl ?4				; always branch
?3
	jsr init_pattern		; initialize pattern
?4
	lda stream_timer,y		; get timer till next sequencer event
	beq ?2				; branch if next event required straight away
	sta sequencer_timer		; save in timer
?5
	rts


;
; initialize pattern
;

init_pattern

; NB preserve y reg

; byte 1 - timer. timer is in seconds/2 (129-255 means wait until active patterns dead)
; byte 2 - properties (when no object then pattern command required)
; byte 3 - pattern offset lo (when no object then game command required)
; byte 4 - bit 0-3 - pattern offset hi
; byte 4 - bit 4-7 - object qty (NB sum patterns are allowed 0 objects) 
; byte 4 - (when no object then game command music required)
; byte 5 - object type (0 means no objects - a non-pattern game command)
; byte 6 - object spacing 
; byte 6 - also bit 2-7 - frame die hit count (when a hit count pattern)
; byte 6 - also bit 0-1 - qty of die animation frames (when a hit count pattern)

	.local

	ldx last_handler		; get last used pattern handler
?1
	dex				; move on to next handler
	bpl ?2

	ldx #max_handlers-1		; wrap around if necessary
?2
	lda pat_kill,x
	bne ?1				; branch back if handler still has aliens

	lda stream_properties,y
	and #link_bit
	beq ?2a				; branch if link bit is off
	
	txa				; this handler to A
	ldx last_handler		; index to last handler
	
	sta pat_link,x			; save this handler as linked into the last handler
	tax				; restore current handler index
?2a
	stx last_handler		; x has free handler. save for next check

	lda stream_qty,y		; get encoded qty of aliens

	lsr a				; move to correct bit position
	lsr a
	lsr a
	lsr a

	sta pat_qty,x			; save qty of aliens required in pattern
	bne ?2b				; branch if pattern has objects, (ie it is not a sum pattern)

	ora #128			; "force no object" bit on, for sum pattern with no object
?2b
	sta pat_kill,x			; default number of hits before pattern is killed

	lda stream_properties,y		; get properties
	sta pat_properties,x		; save as pattern properties

	lsr a				; move sum bit into carry
	bcc ?3				; branch if no sum pattern required

	stx sum_status			; initialize sum pattern with handler number
	clc
?3
	and #spread_bit/2		; check properties for multiple start positions
	beq ?4				; branch if (now shifted) spread_bit not on

	lda pat_qty,x
	asl a
	adc pat_qty,x			; pattern qty * 3 is amount of z,y,x alien co-ords
	bne ?5				; always branch
?4
	lda #3				; only 1 set of z,y,x alien co-ords
?5
	sta pat_index,x			; pattern data starts immediately after z,y,x co-ords

; NB Carry is clear

	lda stream_pat_lo,y		; get pattern lo offset
	adc #<pattern_data		; add lo start address of pattern data 
	sta pat_lo,x			; save lo address of pattern

	lda stream_pat_hi,y
	and #pat_hi_bits		; get just pattern hi offset
	adc #>pattern_data		; add hi start address of pattern data
	sta pat_hi,x			; save hi address of pattern

	lda stream_spacing,y		; get encoded spacing
	sta pat_spacing,x		; save in pattern spacing

	lda vector_buffer_limits,x	; initialize the vector buffer for pattern decoder
	sta pat_vector,x

	lda #0
	sta pat_frame_hits,x		; default hits till next frame to off
	sta pat_loops,x			; initialize loop counter (to zero)
	sta priority_status		; reset any previous pattern priority changes

	lda #128
	sta pat_link,x			; default to no linked handler
	
	lda stream_properties,y		; check for multiple hit object pattern
	and #hit_count_bit
	beq ?7				; branch if no multiple hit


	lda stream_spacing,y		; get qty of hits till next frame and die frame qty
	and #~00000011			; bit 0 and 1 are die frame qty
	sta pat_die_frames,x

	lda stream_spacing,y		; shifted bits 2-7 are qty of hits till next frame
	lsr a
	lsr a
	sta pat_frame_hits,x
	sta pat_kill,x
	
	lda #2
	sta pat_spacing,x		; overwrite stream_spacing value so control starts next frame

?7
	lda stream_properties,y
	and #rotate_bit
	
	sta rotate_status		; if needed turn on rotate, by using a non-zero value

	lda stream_object,y		; get object encoded in stream
	sta pat_object,x

	lda #norm_fireball_speed
	sta missile_speed+1		; re-initialize fireball missile speed 

	lda #2
	sta pat_timer,x			; initialize the pattern decoder to start next spacing 
	sta pat_start_timer,x		; initialize the alien launcher


	inc active_pat_count		; update active pattern counter


	rts


;
; initialize alien
;

init_alien

	.local

	stx temp			; save pattern handler

	lda pat_properties,x
	php				; x mirror status

	and #spread_bit			; check for spread aliens
	beq ?1				; branch if not multiple starting positions

	lda pat_qty,x
	asl a
	adc pat_qty,x			; (qty aliens left -1) * 3
?1
	tay				; save pattern index to object start z,y,x

	ldx last_alien			; get last alien object used
?2
	inx
	cpx #12				; get next alien
	bne ?3

	ldx #0				; wrap back to start if needed
?3
	lda properties+12,x
	bpl ?2				; branch if alien not hidden
	and #dead_bit
	beq ?2				; branch if alien not dead

	stx last_alien			; x is a free alien. save for next alien requirement

	lda (source_a),y		; get initial z position
	sta z_pos+12,x

	lda #0
	sta z_pos_lo+6,x

	iny

	lda (source_a),y		; get initial y position
	sta y_pos+12,x

	iny

	lda (source_a),y		; get initial x position

	plp				; retrieve x mirror status
	bpl ?4				; branch if no x mirror

	eor #255
	clc
	adc #1
?4
	sta x_pos+12,x

	ldy temp			; restore pattern handler to y

	tya
	sta handler,x			; save pattern handler


	lda pat_properties,y
	asl a				; shift so 3d bit moves to bit 7 for easy check
	sta control,x			; set bit 7 if a 3d pattern (include other control bits)

	lda vector_buffer_limits,y	; set at starting point in vector buffer
	sta vector,x

	lda #1
	sta timer,x			; set alien timer to get 1st segment



	lda pat_object,y		; get object required
	bpl ?4a				; branch if other object bit not set
	
	and #~01111111			; remove other object bit (mainly for uriah head problem)

	clc
	adc #2				; next 2 objects along (avoid any animation frame)

	sta pat_object,y		; set for rest of aliens in pattern
	
	adc #254			; return to original object for this alien (c is clear)
?4a	
	sta object,x			; save object for animation patterns
	tay

	lda object_to_slot,y		; get slot number of object
	sta slot+12,x

	lda object_properties,y		; get properties of object
	and #~01111111			; ensure hide_bit is off as this bit also used to denote mirror 
	sta properties+12,x

	lda object_sound,y
	beq ?5				; branch if no sound for object
	sta sound_queue			; queue sound fx for object
?5
	ldx temp			; restore pattern handler to x


	rts


;
; decode pattern
;

decode_pattern

; pattern segment specifies an alien direction for how long (and other functions)
; byte 1 - bit 7 - fire. bit 6-0 - timer
; byte 2 - bit 7-6 - z vector lo. bit 5-2 - y vector. bit 1-0 - z vector hi
; byte 3 - bit 7 - pattern command. bit 6-2 - x vector. bit 1-0 - frame animation (using x vector) 
; when end of pattern:
; byte 1 - 0 - end of pattern/repeat
; byte 2 - bit 7 - infinite repeats. bit 6-0 - loops required before carrying on. if 0 loops then end
; byte 3 - bit 7-0 - position in pattern to loop back to

	.local

	stx temp			; save pattern handler
	sta temp2			; save vector buffer pointer

	lda pat_properties,x		; get mirror property. x mirror if -ve. save status
	php				; NB carry is still set here
?0
	lda (source_a),y		; get fire and timer segment
	bne ?2				; branch past if not end of pattern

	iny				; next segment

	lda (source_a),y		; get number of repeats required
	beq ?6				; branch if no repeats (ie finished pattern)
	bmi ?1				; branch if infinite repeats

	cmp pat_loops,x
	bne ?0b				; branch if pattern has not reached loop count

	lda #0
	sta pat_loops,x			; reset loop counter

	iny				; skip over loop back pointer
	iny				; and move on to next fire and timer segment
	bne ?0				; always branch back to get segment
?0b
	inc pat_loops,x			; increase loop count of pattern
?1
	iny				; next segment

	lda (source_a),y		; get position in pattern to loop back to
	tay				; go to loop back position

	lda (source_a),y
?2
	and #255-fire_bit		; remove fire bit from timer value
	sta pat_timer,x			; save in pattern timer

	ldx temp2			; restore pattern vector pointer

	lda (source_a),y		; get segment again
	sta ftimer_buffer,x		; save fire and timer in buffer

	iny				; next segment

	lda (source_a),y		; get z vector and y vector segment
	and #z_vector_lo_bits		; get z vector lo bits
	sta z_vector_lo_buffer,x

	lda (source_a),y		; get segment again
	and #z_vector_hi_bits		; get z vector hi bits
	sec
	sbc #2				; convert to -2 to +1 range on high

	sta z_vector_buffer,x		; save z vector hi bits

	lda (source_a),y		; get segment again
	and #y_vector_bits		; get y vector
	lsr a
	lsr a				; shift to correct position in byte
	sbc #7				; convert to -8 to +7 range. NB C is clear

	sta y_vector_buffer,x

	iny				; next segment

	lda (source_a),y		; pat command, x vector and frame animation
	and #x_vector_bits+frame_bits	; get frame animation and x vector
	
	lsr a
	bcc ?3				; branch if frame change bit not set

	sta frame_change_buffer,x	; x vector bits used to give the animation frames to skip

	plp				; remove unused mirror stack entry

	lda #0
	sta x_vector_buffer,x		; no x vector can be done if frame changes being used

	beq ?5				; always branch
?3
	lsr a				; mirror frame bit into C. Will be clear if no frame animation
	sbc #15				; convert x to -16 to +15 range. carry clear
	
	plp				; retrieve x mirror status. if -ve then mirror is on
	bpl ?4				; branch if no x mirror required
	eor #255			; convert to negative
	adc #0				; NB carry was set at time of saving status
?4
	sta x_vector_buffer,x

	lda #0				; no frame changes can be done if x vector being used
	sta frame_change_buffer,x
?5
	lda (source_a),y
	bmi ?8				; branch if pattern command set
?5a
pattern_command_rtn

	ldx temp			; restore pattern handler

	iny				; move to next segment
	tya
	sta pat_index,x			; save in pattern index

	jmp decode_pattern_rtn
?6
	plp				; free the unused mirror stack entry

	ldy temp2			; get vector buffer pointer

	sta ftimer_buffer,y		; end pattern command for aliens

	jmp decode_pattern_rtn
?8
	lda game_status			; check if pattern command could be awaiting configuration
	bne ?5a				; branch if it could - command would be an invalid address
	
	jmp (pattern_command)		; run pattern command (like dragon priority swap)


;
; kill linked alien
;

kill_alien

; kill linked objects (such as dragon segments impossible to kill otherwise)

	.local

	lda pat_kill,y			; reduce aliens in pattern
	sec
	sbc #1
	sta pat_kill,y

	bne ?0
	
	dec active_pat_count		; if no aliens left reduce active pattern count
?0
	ldy #xplosion
	lda object_to_slot,y		; convert object to an explosion
	sta slot+12,x

	lda #20				; setup time to live timer for explosion
	sta timer,x			; longer than normal explosion

	lda object_properties,y		; get properties of explosion
	sta properties+12,x

	ldy z_pos+12,x

	lda size_map,y			; convert z pos to a frame number 0-10
	ldy slot+12,x			; get slot number in y
	clc
	adc x11_table,y			; add frame no to slot*11
	tay				; put in y
	lda y_pos_2d+12,x
	sbc height_table,y		; subtract height (height less 1 precalc)
	sta y_top+12,x			; top y of object
					; as no conversion from 2d is done here for the explosion

	lda control,x
	bmi ?1				; branch past if 3d bit is set.

	lda z_pos+12,x
	clc
	adc #4
	sta multiplier			; change z to 1,1.25,1.5 etc.

	lda x_pos_2d+12,x		; get position

	sbc #127			; convert back to +/-. NB c is clear

	jsr x_multiply			; convert to 3d

	sta x_pos+12,x			; save back in position
?1
	stx temp

	ldx #2				; 100,000 digit
	lda #1				; value 1

	jsr add_score			; add 100,000 to score
	
	lda #explosionfx
	sta sound_queue			; queue explosion sound

	ldx temp

	jmp kill_alien_rtn		; return


;
; move alien explosion
;

move_alien_explosion

	.local

	dec timer,x			; decrement explosion time to live timer
	beq ?2				; branch if run out

	dec z_pos+12,x			; move object forward
	bmi ?2a				; branch if off-screen

	ldy z_pos+12,x

	tya
	clc
	adc #4
	sta divisor			; convert z position to divide by 4 for x division later

	lda y_pos_2d+12,x		; deal with 2d y position only. 3d not needed
	sec
	sbc tilt_hi,y			; remove grid tilt
	adc x_size+12,x			; drop explosion by the 1/2 width (don't ask, it looks ok)
	cmp div43z_2d_table,y		; compare with ground - predefined 43 divided by z table
	bcc ?1

	lda div43z_2d_table,y		; reset to ground
	clc
?1
	adc tilt_hi,y			; add grid tilt value back to y position

	sta y_pos_2d+12,x

	lda size_map,y			; convert z pos to a frame number 0-10
	ldy slot+12,x			; get slot number in y
	clc
	adc x11_table,y			; add frame no to slot*11
	tay				; put in y
	lda width_table,y		; lookup width
	sta x_size+12,x
	lda y_pos_2d+12,x
	sbc height_table,y		; subtract height (height less 1 precalc)
	sta y_top+12,x			; top y of object

	
	lda x_scroll		
	clc
	adc x_pos+12,x
	sta x_pos+12,x			; add scroll value to x position

	jsr x_divide			; convert x to 2d value

	sta x_pos_2d+12,x

	sec		
	sbc x_size+12,x
	cmp #128+75			; check left side of object to right side of screen
	bcs ?2				; branch if off screen (allow for some size overlap)

	lda x_pos_2d+12,x
	adc x_size+12,x
	cmp #128-75			; check right side of object to left side of screen
	bcs ?3				; branch if on screen (allow for some size overlap)
?2
	lda #255
	sta z_pos+12,x			; set z pos -ve for sort ignore
?2a
	lda #hide_bit+dead_bit		; switch off object. ensure aliens have dead bit set
	sta properties+12,x

?3
	jmp move_alien_explosion_rtn	; return


;
; move alien
;

move_alien

	.local

	ldy vector,x			; get index of current vector of object

	lda x_vector_buffer,y		; add current x vector to object
	clc
	adc x_pos+12,x
	sta x_pos+12,x
	sta x_pos_2d+12,x

	lda y_vector_buffer,y		; add current y vector to object
	clc
	adc y_pos+12,x
	sta y_pos+12,x
	sta y_pos_2d+12,x

	lda z_vector_lo_buffer,y	; add current z lo and hi vector to object
	clc
	adc z_pos_lo+6,x
	sta z_pos_lo+6,x

	lda z_vector_buffer,y
	adc z_pos+12,x
	sta z_pos+12,x

	ldy control,x
	bpl ?1				; branch past if not 3d

	clc
	adc #4
	sta divisor			; convert z position to divide by 4	

	lda x_pos+12,x

	jsr x_divide			; convert x to 2d value
	sta x_pos_2d+12,x

	lda y_pos+12,x
	beq ?0

	jsr y_divide			; convert y to 2d value
?0
	clc
	adc #90				; convert y to 2d screen co-ordinate


	ldy z_pos+12,x

	jmp ?1a				; always branch
?1
	tay				; fix optimize ldy control,x - must get z into y reg
	lda y_pos_2d+12,x
?1a
	clc
	adc tilt_hi,y			; add grid tilt value to y position

	sta y_pos_2d+12,x


	lda size_map,y			; convert z pos to a frame number 0-10
	ldy slot+12,x			; get slot number in y
	clc
	adc x11_table,y			; add frame no to slot*11
	tay				; put in y
	lda width_table,y		; lookup width
	sta x_size+12,x


	lda properties+12,x
	and #low_bit+high_bit		; get info for y is top, middle or bottom of object
	beq ?middle			; branch if middle (high and low bit not set)

	lsr a
	lda y_pos_2d+12,x
	bcc ?bottom			; branch if top bit not set
?top
	sta y_top+12,x			; y position is top of object
	adc height_table,y		; add (height less 1) plus carry set for bottom
	sta y_bottom+12,x

	jmp move_alien_rtn		; exit

?middle
	lda height_table,y		; y position is middle of object
	lsr a				; 1/2 of y height
	adc y_pos_2d+12,x		; add (height less 1) with round up for bottom
?bottom
	sta y_bottom+12,x
	sbc height_table,y		; subtract (height less 1) with carry clear for top
	sta y_top+12,x

	jmp move_alien_rtn		; exit


;
; load_background
;

load_background

	.local

	lda stage_no
	beq ?5				; no background for stage 0 (title screen)

	clc
	adc #[back_files_lo-files_lo]-1	; start of background files + stage number
	tax
	stx object_no			; store file number

	lda #<object_buffer
	sta 772				; destination of file
	sta source_b			; start address for shift routine
	lda #>object_buffer
	sta 773
	sta source_b+1

	jsr load_file			; load background 4 colour data to object buffer

	ldx object_no			; setup colour decode table for file in x
	ldy object_palette,x		; get palette required for background file. NB No mirror
	jsr make_colour_table		; make colour decode lookup table

	lda #<back_overlay		; background screen a overlay
	sta dest_a
	lda #>back_overlay
	sta dest_a+1
	lda #<[back_overlay+96*8]	; background screen b overlay
	sta dest_b
	lda #>[back_overlay+96*8]
	sta dest_b+1

	jsr copy_background		; copy background to non-shifted overlay area

	lda #8				; setup for 8 lines of source data
	sta temp			; to get 2 pixel shifted
?1
	ldx #4				; setup for 4 bit (2 pixel) shift
	ldy #0
	lda (source_b),y		; get shift from 1st byte in line
?2
	asl a

	ldy #95				; now get end byte of line
?3
	lda (source_b),y
	rol a				; put shift data in from previous data
	sta (source_b),y		; save it

	dey
	bpl ?3				; do until all data in line

	dex
	bne ?2				; do until 2 pixel shift completed for line

	dec temp			; countdown lines done
	beq ?4				; branch if none left

	lda source_b
	clc
	adc #96
	sta source_b			; point source to start of next line of data
	bcc ?1				; branch back for next line
	inc source_b+1			; deal with hi byte of source
	bcs ?1				; branch back for next line
?4

	lda #<[back_overlay+96*8*2]	; shifted background screen a overlay
	sta dest_a
	lda #>[back_overlay+96*8*2]
	sta dest_a+1
	lda #<[back_overlay+96*8*3]	; shifted background screen b overlay
	sta dest_b
	lda #>[back_overlay+96*8*3]
	sta dest_b+1

	jsr copy_background		; copy background data to shifted overlay area

	rts
?5
	jsr clear_background

	rts


;
; wait press
;

wait_press

	.local

	lda #8
	sta CONSOL			; reset console buttons
?1
	lda #15				; vbi's to count before flash on/off
	sta temp
?2
	lda vbi_count
?3
	cmp vbi_count			; wait 1 vbi
	beq ?3

	lda TRIG0			; check stick trigger
	sta last_button			; ensure last_button is updated so firing is not commenced
	beq ?4				; branch if pressed

	lda CONSOL			; get console buttons pressed
	and #1				; just leave start button
	beq ?4				; branch if start pressed

	dec temp
	bne ?2

	lda vbi_count
	sta CHACTL			; store 0 or 1 in char ctrl alternately to flash

	jmp ?1
?4
	lda #0
	sta CHACTL			; turn flashing characters off

	lda #8
	sta CONSOL			; reset console buttons

	rts


;
; wait vbi
;

wait_vbi

; wait until os vbi routine has run by checking real-time clock

	.local

	lda 20
?1
	cmp 20
	beq ?1

	rts


;
; update score
;

update_score

; do score housekeeping and stay alive score

	.local

	ldx #6				; add 1 to 10's digit(6th from right)
	lda #1
	jsr add_score

	lda lives_status		; check extra life status 
	bne ?0				; branch if it has already been had

	ldy #1				; select millions digit
	lda score,y
	cmp #5				; check if 5 million score has been reached
	bne ?0				; branch if not

	sta lives_status		; set status to life had

	ldy man_lives			; draw an extra man icon
	lda #20
	sta lives,y

	inc man_lives			; increase the amount of lives
?0
	lda score_status		; check if score flagged as being a high score
	bne ?4				; branch if it already has

	ldy #0				; start from highest power of 10 check if score is high
?1
	lda top_score,y			; get top score digit
	and #~11101111			; remove any blank char bit from number
	sta temp			; save for compare

	lda score,y			; get score digit
	and #~11101111			; remove any blank

	cmp temp			; compare score to top score
	beq ?2				; branch if digits same - need to keep checking
	bcs ?3				; branch if score digit > top score digit 

	rts				; exit - score is less than top score
?2
	iny
	cpy #7
	bne ?1				; do until all digits

	rts				; NB falls through here when all digits the same
?3
	sta score_status		; store non-zero to indicate score more than top score
?4
	ldy #6				; for all digits of score
?5
	lda score,y
	sta top_score,y			; copy score digit into high score digit

	dey
	bpl ?5				; do until all digits

	rts


;
; add score
;

add_score

; add value in A to score digit indexed by x

	.local

	clc
	adc score,x			; add value to score digit
	cmp #16				; check if was blank (ie digit never used)
	bcs ?3				; branch past if digit blank (>=16)
?1
	cmp #10
	bcc ?2				; branch if digit <10 - no carry to next digit

	sbc #10				; remove extra 10 to give correct score digit
	sta score,x

	lda score-1,x			; get digit to left	
	and #~11101111			; set to a zero if blank (char 16 to char 0) 

	adc #0				; add carry from prev value to this digit

	dex
	bpl ?1				; do until all digits checked for carry

	rts				; exit
?2
	sta score,x			; save digit

	rts				; exit
?3
	and #~11101111			; remove char 16 bit to leave 0 plus value to add
	sta score,x			; (store value over blank)
?4
	lda score+1,x			; get next digit to right
	eor #~00010000			; check if blank (will be 0 only when char 16)
	bne ?5				; branch if digit not blank

	sta score+1,x			; set blank trailing digit to a zero

	inx				; get next digit
	cpx #6
	bne ?4				; do until all possible blank trailing digits
?5
	rts				; exit


;
; check object collision
;

check_object_collisions

	.local

	lda uriah_status
	beq ?6

; check for scenery collisions with uriah. uriah consists of alien objects

	ldy #7				; 8 scenery objects
?1
	lda properties+24,y
	and #hide_bit+dead_bit
	bne ?5				; branch if scenery not in use or an explosion

	ldx z_pos+24,y			; get z position of scenery
	lda sort_index+1,x		; get last object index at z+0 to check against uriah	
	sta last_object			; well, actually the first at z+1

	lda sort_index,x		; get first object index at z+0
	tax

	clc
	adc #1
	cmp last_object			; check for more objects than just scenery in z range
	bcs ?5				; branch if none (or fewer)

	lda x_pos_2d+24,y
	adc x_size+24,y
	sta scenery_right		; calc right edge of scenery
	
	lda x_pos_2d+24,y
	sec
	sbc x_size+24,y
	sta scenery_left		; calc left edge of scenery

	lda y_top+24,y			; top edge of scenery
	sta scenery_top

	lda y_bottom+24,y		; bottom edge of scenery
	sta scenery_bottom

	sty temp			; save current scenery index

	clc				; ensure carry clear for first add 
?3
	ldy sort_table,x		; get objects with same z positions as bullet
	
	lda properties,y		; get object properties
	eor #harmless_bit+bounce_bit	; only uriah object has harmless_bit+bounce_bit set
	bne ?4				; branch if object not a uriah segment

	lda x_pos_2d,y
	adc x_size,y			; right edge of object
	cmp scenery_left		; compare to left edge of scenery
	bcc ?4				; branch if right edge less than scenery left. ie miss

	lda x_pos_2d,y
	sbc x_size,y			; left edge of object
	cmp scenery_right		; compare to right edge of scenery
	bcs ?4				; branch if left edge greater than scenery right. ie miss

	lda y_top,y			; top edge of object
	cmp scenery_bottom		; compare to bottom edge of scenery
	bcs ?4				; branch if greater than bottom edge of scenery. ie miss

	lda y_bottom,y			; bottom edge of object
	cmp scenery_top			; compare to top edge of scenery
	bcc ?4				; branch if less than top edge of scenery. ie miss

	jmp collide_uriah		; scenery has hit object
?4
	inx				; next object
	cpx last_object
	bcc ?3				; branch until last object (but not including)

collide_uriah_rtn

	ldy temp			; restore scenery index
?5

	dey				; do until all scenery
	bpl ?1

	rts
?6

	.local

; check for bullet collisions. need to check z positions from bullets z to bullet z-2 or objects
; can jump through the bullets which move in 2 z jumps. that means fastest moving forward object
; is only allowed 1 z jump.

	ldy #5				; 6 bullets to check
?1
	lda properties,y
	and #hide_bit+bounce_bit
	bne ?5				; branch if bullet off or a bounce bullet

	ldx z_pos,y			; get z position of bullet
	lda sort_index+2,x		; get last object index at z+1 to check against bullet	
	sta last_object			; well, actually the first at z+2

	lda sort_index-1,x		; get first object index at z-1
	tax

	clc
	adc #1
	cmp last_object			; check if there are more objects than bullet in z range
	bcs ?5				; branch if not

	lda x_pos_2d,y
	adc x_size,y
	sta bullet_right		; calc right edge of bullet
	
	lda x_pos_2d,y
	sec
	sbc x_size,y
	sta bullet_left			; calc left edge of bullet

	lda y_top,y			; top edge of bullet
	sta bullet_top

	lda y_bottom,y			; bottom edge of bullet
	sta bullet_bottom

	sty temp			; save current bullet index

	clc				; ensure carry clear for first add 
?3
	ldy sort_table,x		; get objects with same z positions as bullet
	
	lda properties,y		; get object properties. branch if object hidden or
	and #hide_bit+passthru_bit	; branch past check if bullets are allowed to pass
	bne ?4				; through the object. ie it is a bullet or explosion

	lda x_pos_2d,y
	adc x_size,y			; right edge of object
	cmp bullet_left			; compare to left edge of bullet
	bcc ?4				; branch if right edge less than bullet left. ie miss

	lda x_pos_2d,y
	sbc x_size,y			; left edge of object
	cmp bullet_right		; compare to right edge of bullet
	bcs ?4				; branch if left edge greater than bullet right. ie miss

	lda y_top,y			; top edge of object
	cmp bullet_bottom		; compare to bottom edge of bullet
	bcs ?4				; branch if greater than bottom edge of bullet. ie miss

	lda y_bottom,y			; bottom edge of object
	cmp bullet_top			; compare to top edge of bullet
	bcc ?4				; branch if less than top edge of bullet. ie miss

	jmp collide_objects		; bullet has hit object
?4
	inx				; next object
	cpx last_object
	bcc ?3				; branch until last object (but not including)

	ldy temp			; restore bullet index
?5
collide_objects_rtn

	dey				; do until all bullets
	bpl ?1


; copy closest objects for man collision frame sync

	ldx draw_screen_index
	lda old_pos_offset,x
	sta temp			; start index of z=0 objects for this screen no 

	lda sort_index+1		; get start of z=1. last z=0 object is index just before
	sec
	sbc #z_clip			; subtract z clipped objects
	beq ?8				; branch if no objects

	tay
?6
	ldx sort_table+z_clip-1,y	; get next object with z=0 (exclude z clipped objects)
	lda properties,x		; get object properties
	and #harmless_bit+hide_bit	
	bne ?7				; ignore object if it's harmless to man or hidden

	sty temp2			; save y
	
	ldy temp			; get index of z=0 objects for this screen no

	lda x_pos_2d,x
	sec
	sbc x_size,x
	sta old_left,y			; save left edge of object

	lda x_pos_2d,x
	clc
	adc x_size,x
	sta old_right,y			; save right edge of object

	lda y_top,x
	sta old_top,y			; save top edge of object

	lda y_bottom,x
	sta old_bottom,y		; save bottom edge of object

	inc temp			; index for next object

	ldy temp2			; restore y
?7
	dey
	bne ?6				; do until all z=0 objects 
?8
	ldx draw_screen_index		
	lda temp
	sta old_pos_index,x		; save index of z zero objects in this screen

	rts


;
; collide uriah
;

collide_uriah

; collide uriah segment and scenery object

	.local

	ldy temp			; restore scenery index

	lda properties+24,y
	and #high_bit+low_bit	
	tax				; high/low scenery bits for scoring into x lookup

	lda object_score_digit,x	; get which digit to add to
	sta temp2

	lda object_score_value,x	; get value to add to score
	ldx temp2

	jsr add_score			; add to score. digit in x. value in A

	ldx #xplosion			; get explosion object number

	lda object_to_slot,x		; get slot number of explosion
	sta slot+24,y

	lda #low_bit+harmless_bit+dead_bit	
	sta properties+24,y		; properties for the explosion object

	lda z_pos+24,y			; move explosion back so kill can be seen
	clc
	adc #1
	sta z_pos+24,y

	lda #8
	sta timer+12,y			; initialize explosion time to live timer

	lda #2
	sta sound_queue			; queue explosion sound

	ldx counters_small		; a small counter needs displaying
	inx
	cpx #9
	bne ?3				; branch if no wrap
?1	
	ldx counters_small		; next small counter 

	jsr draw_small_counter		; erase small counter

	dec counters_small		
	bne ?1				; do until all 8 small counters

	ldx counters_big		; a large counter needs displaying
	inx
	cpx #9
	beq ?2				; branch if wrap... shouldn't be possible

	stx counters_big
	
	jsr draw_big_counter

	beq ?4				; always branch
?2	
	dec counters_big		; this should never happen...but...

	bne ?4				; always branch
?3
	stx counters_small

	jsr draw_small_counter 
?4
	jmp collide_uriah_rtn		; exit



;
; draw_small_counter
;

draw_small_counter

	.local

	lda counter_draw_addr_lo-1,x	; get address of player counter draw routine required
	sta jump			
	lda counter_draw_addr_hi-1,x
	sta jump+1
	
	txa				; get whether left or right counter shape required
	and #1				; odd counter is to left of Player. Even to the right
	asl a
	asl a				; small counter shapes 4 bytes high
	adc #3				; add to give end of frame address
	
	ldx uriah_status
	adc counter_frames_offset-1,x	; add which set of shapes to use (trees or pylons)
	tax
	
	ldy #64				; small counter bottom position
	
	lda #64-4			; small counter top position
	sta counter_y_stop

	jmp (jump)			; draw counter


;
; draw_big_counter
;

draw_big_counter

	.local

	lda counter_draw_addr_lo-1,x	; get address of player counter draw routine required
	sta jump			
	lda counter_draw_addr_hi-1,x
	sta jump+1
	
	txa				; get whether left or right counter shape required
	and #1				; odd counter is to left of Player. Even to the right
	asl a
	asl a				; big counter shapes 8 bytes high
	asl a
	adc #8+7			; add to skip small counters, add for end of frame address
	
	ldx uriah_status
	adc counter_frames_offset-1,x	; add which set of shapes to use (trees or pylons)
	tax
	
	ldy #64-6			; big counter bottom position
	
	lda #64-6-8			; big counter top position
	sta counter_y_stop

	jmp (jump)			; draw counter


;
; draw counter overlay
;

draw_P0_counter

	.local
?1	
	lda counter_frames,x
	eor player0,y
	sta player0,y

	dex
	dey
	cpy counter_y_stop
	bne ?1
	
	rts

draw_P1_counter

	.local
?1	
	lda counter_frames,x
	eor player1,y
	sta player1,y

	dex
	dey
	cpy counter_y_stop
	bne ?1
	
	rts
	
draw_P2_counter

	.local
?1	
	lda counter_frames,x
	eor player2,y
	sta player2,y

	dex
	dey
	cpy counter_y_stop
	bne ?1
	
	rts
	
draw_P3_counter

	.local
?1	
	lda counter_frames,x
	eor player3,y
	sta player3,y

	dex
	dey
	cpy counter_y_stop
	bne ?1
	
	rts


;
; collide bullet and objects
;

collide_objects

	.local

	lda properties,y
	and #bounce_bit+high_bit+low_bit
	cmp #bounce_bit			; set carry if bounce bit is on
	bcc ?2				; branch if not indestructible

	ldy temp			; get bullet object index back
	lda #bounce_bit+harmless_bit+passthru_bit
	sta properties,y		; setup a bounce bullet

	lda RANDOM
	and #31
	lsr a				; random number 0-15. random carry
	bcc ?1
	eor #255			; randomly convert to negative number
	adc #1
?1
	sta x_vector,y			; save as x vector

	lda RANDOM
	and #7				; random number 0-7
	eor #255			; always convert to negative number
	clc
	adc #1

	sta y_vector,y			; save as y vector

	
	lda #4
	sta sound_queue			; queue bounce sound


	jmp collide_objects_rtn		; exit
?2
	cpy #24				; last 8 of 32 objects are scenery
	bcs ?7				; branch if a scenery object

	ldx handler-12,y		; get pattern handler

	lda pat_frame_hits,x
	beq ?4				; branch if normal pattern (no multiple hits)
	
	dec pat_kill,x
	bne ?8				; branch if more hits required on control alien
	
	dec pat_die_frames,x		; check how many dying frame animations to go
	beq ?3				; branch if last frame done - time to die

	sta pat_kill,x			; reset dying frame animation hit counter
	
	jmp apply_die_frames		; apply the dying animation to the aliens
?3	
	jmp init_kill_aliens		; setup the controlled aliens to die

?4
	dec pat_kill,x			; count down normal objects left in pattern
	bne ?5				; branch if still objects left

	dec active_pat_count		; reduce active pattern count
?5
	lda control-12,y
	bmi ?6				; branch past if 3d bit is set.

	lda z_pos,y
	clc
	adc #4
	sta multiplier			; change z to 1,1.25,1.5 etc.

	lda x_pos_2d,y			; get position

	sbc #127			; convert back to +/-. NB c is clear

	jsr x_multiply			; convert to 3d

	sta x_pos,y			; save back in position
?6
	lda #0				; set score digit to 10,000

	ldx sum_status
	bmi ?7				; branch if sum pattern not active

	lda #4				; set 100,000 score digit during sum pattern
?7
	tax				; high/low scenery bits or a above for scoring into x lookup

	lda object_score_digit,x	; get which digit to add to
	sta temp2

	lda object_score_value,x	; get value to add to score
	ldx temp2

	jsr add_score			; add to score. digit in x. value in A

	ldx #xplosion			; get explosion object number

	lda object_to_slot,x		; get slot number of explosion
	sta slot,y

	lda #low_bit+harmless_bit+dead_bit	
	sta properties,y		; properties for the explosion object

	lda z_pos,y			; move explosion back so kill can be seen
	clc
	adc #1
	sta z_pos,y

	lda #8
	sta timer-12,y			; initialize explosion time to live timer

	
	lda #2
	sta sound_queue			; queue explosion sound
?8
apply_die_frames_rtn
init_kill_aliens_rtn

	ldy temp			; get bullet object index back
	lda #hide_bit

	sta z_pos,y			; set z -ve for sort routine
	sta properties,y		; hide bullet

	jmp collide_objects_rtn		; exit


;
; apply dying frames
;

apply_die_frames

; move linked aliens onto the next dying frame if their pattern object allows animation
; the next object number along must be the next dying frame
; x is control pattern. y is control object alien

	.local
	
	lda pat_link,x
	sta temp2			; save linked pattern handler number for comparison
	
	lda pat_properties,x		; check if control object allowed to be animated
	and #animate_bit
	beq ?1				; branch if not allowed		
	
	lda pat_object,x
	tax
	inx				; next frame is next object along from control object

	lda object_to_slot,x
	sta slot,y			; replace control object frame with the next animation object

?1
	ldx temp2			; linked pattern handler number
	
	lda pat_properties,x		; check if linked pattern object allowed to be animated
	and #animate_bit
	beq ?4				; branch if not allowed
	 
	lda pat_object,x
	tax
	inx				; next animate frame is next object number along

	lda temp2			; A has handler number to check 

	ldy #11				; 12 aliens
?2
	cmp handler,y			; check if alien is in the linked pattern handler
	bne ?3				; branch if not
	
	lda object_to_slot,x
	sta slot+12,y			; replace the aliens frame with the next animation object

	lda temp2			; restore handler number to A
?3
	dey
	bpl ?2				; do until all aliens

	tax
?4
	lda pat_link,x			; check for any further linked patterns
	sta temp2
	
	bpl ?1				; branch if more linked patterns
	
	
	jmp apply_die_frames_rtn	; return


;
; initialize kill aliens
;

init_kill_aliens

; setup chain reaction to kill all aliens in linked patterns
; x is control pattern. y is control object alien - must be restored
; NB this routine assumes last_alien was initialized to 0 for control patterns which it must be
; for priority reasons.

	.local

	lda #0
	sta vector-12,y			; stop control alien

	lda properties,y
	ora #bounce_bit
	sta properties,y		; turn on bounce bit so can't be shot more than once
	
	lda #1
	sta timer-12,y			; setup control to die now

	ldx handler-12,y	
	sta pat_kill,x			; as pat_kill gets counted down by kill_alien, must be 1 here.

	lda pat_link,x
	bmi ?4				; branch if no linked patterns
?1
	sta temp2			; save linked pattern handler number

	ldx #0				; start from first alien
	stx temp3			; initialize time till die timer
?2
	cmp handler,x
	bne ?3				; branch if alien not in linked pattern handler
	
	lda temp3
	clc
	adc #3
	sta temp3			; keep adding 3 to time till die timer for the next alien

	sta timer,x			; save in alien timer
	
	lda #0
	sta vector,x			; stop alien moving by setting special no move vector
	
	lda temp2			; restore handler number to check against
?3
	inx
	cpx #12
	bne ?2				; do until all objects checked

	tax
	lda pat_link,x
	bpl ?1				; branch if linked pattern is also linked itself

?4
	jmp init_kill_aliens_rtn	; return


;
; kill man
;

kill_man

	.local

	dec man_lives			; reduce lives by 1
	bpl ?0				; branch if still lives left

	lda #56				; make last frame still dead rather than standing up
	sta man_die_frame+4
?0
	jsr stop_sound			; stop sound channels playing. NB A=0 on return

	sta man_die_index		; initialize setup sequence

	sta source_A			; save lo address for clear_b_players
	sta source_B

	lda #1				; set timer of setup to next vbi
	sta player_timer

	lda man_y_pos			; initial y position
	sec
	sbc #24+3
	sta player_offset

	lda #32				; initialize as no draw till vbi setup ready
	sta player_pos

	ldy draw_screen_index		; get current cleared draw screen addr
	lda screen_addr_A,y

	sta player_pmbase_addr+4	; use it as pmbase for screen b player missiles

	clc
	adc #2
	sta player_dest_hi+8		; hi address of player 0b
	sta player_dest_hi+7		; hi address of player 1b
	sta source_A+1			; save address for clear

	adc #1					
	sta player_dest_hi+6		; hi address of player 2b
	sta player_dest_hi+5		; hi address of player 3b
	sta source_B+1			; save address for clear

	lda #0				; initialize player segment program
	sta player_index

	jsr clear_b_players		; clear screen area for b players

	lda #<argh			; initialize start address of argh sample
	sta sample
	lda #>argh
	sta sample+1

	lda #>[@argh-argh]+>argh	; initialize end address of argh sample
	sta dest_A+1
	lda #<[@argh-argh]
	sta dest_A

	jsr play_sample			; play "argh" sample

	sta player_index		; turn off man die sequencer

	lda #84				; ensure last frame put back to standing up
	sta man_die_frame+4

	ldy man_lives
	bmi ?5				; exit if no lives left
	lda #16				; replace a man icon with blank
	sta lives,y

	lda #256-90
	sta man_flash_timer		; set up invulnerable flash timer 90*2 vbi's.

	lda #<getready			; initialize start address of getready sample
	sta sample
	lda #>getready
	sta sample+1

	lda #>[@getready-getready]+>getready
	sta dest_A+1
	lda #<[@getready-getready]
	sta dest_A			; initialize end address of getready sample


	jsr play_sample			; play "getready" sample


	lda #64+42
	sta man_y_pos			; reset man to on ground position


	ldy #5				; 6 bullets
?3
	lda z_pos,y
	bne ?4				; branch past any bullets that are not z=0

	lda properties,y
	ora #hide_bit
	sta properties,y		; Stop any bullets about to launch in next frame
	sta z_pos,y			; set z -ve for sort routine ignore
?4
	dey
	bpl ?3				; do until all bullets

	lda dlist_index			; get current dlist 1,2 or 3
	sta draw_screen_index		; resync with draw screen to lose any pre-drawn screens

	lda #0
	sta flip_count			; reset any outstanding screen flips

	jsr new_screen			; flip to get a new screen
	


	ldx #7				; start sound for all 4 channels
?4a 
	lda sound_hi,x
	sta AUDC1-1,x			; switch on previous volume value

	dex
	dex
	bpl ?4a

	lda #0
	sta man_run_frame
	sta man_status			; allow man control again. restart main vbi routines
?5
	lda #>pmbase_addr
	sta PMBASE			; reset pmbase to the normal one

	lda #norm_fireball_speed
	sta missile_speed+1		; re-initialize fireball missile speed 

	rts


;
; sequence scenery streams
;

sequence_scenery

; check the 2 streams for new scenery objects

	.local

; scenery stream 1

	dec scenery1_timer		; get timer
	bmi ?2				; if timer -ve then infinite wait (for end of stage)
	bne ?3				; else wait until timer done

	ldy scenery1_pointer
	inc scenery1_pointer		; index to  next scenery data

	lda stream1_timer,y		; get next timer
	sta scenery1_timer		; save timer

	lda stream1_control,y		; get control byte

	lsr a				; get multi-object bit into carry - object no in A

	tax				; object no to x

	stx control+12+3		; object number into control scenery 3
	stx control+12+2		; same for scenery 2
	bcc ?1
	inx				; next object number for scenery 1 if multi-object
	inx
?1
	stx control+12+1
	bcc ?1a
	inx				; next object number for scenery 0 if multi-object
	inx
?1a
	stx control+12


	bpl ?3				; always branch
?2
	inc scenery1_timer		; infinite wait for end of stage
?3

; scenery stream 2

	dec scenery2_timer		; get timer
	bmi ?6				; if timer -ve then infinite wait (for end of stage)
	bne ?5				; else wait until timer done

	ldy scenery2_pointer
	inc scenery2_pointer		; index to  next scenery data

	lda stream2_timer,y		; get next timer
	sta scenery2_timer		; save timer

	lda stream2_control,y		; get control byte

	lsr a				; get multi-object bit into carry - object no in A

	tax				; object no to x

	stx control+16+3		; object number into control scenery 3
	stx control+16+2		; same for scenery 2
	bcc ?4
	inx				; next object number for scenery 1 if multi-object
	inx
?4
	stx control+16+1
	bcc ?4a
	inx				; next object number for scenery 0 if multi-object
	inx
?4a
	stx control+16

?5
	rts
?6
	inc scenery2_timer		; infinite wait for end of stage

	rts


;
; move scenery
;

move_scenery

	.local

	dec z_pos+24,x			; move object forward
	bmi ?2a				; branch if off-screen

	lda z_pos+24,x

	clc
	adc #4
	sta divisor			; convert z position to divide by 4	


	ldy y_sine,x			; get position in sine wave table
	beq ?0				; skip if sine wave not in use for object

	dec y_sine,x			; next position in sine wave
	bne ?0

	lda #16				; wrap wave if needed
	sta y_sine,x
?0
	lda y_pos+24,x			; 3d y position
	adc sine_table,y		; add any sine wave offset

	beq ?1

	jsr y_divide			; convert y to 2d value
?1
	clc
	adc #90				; convert y to 2d screen co-ordinate

	ldy z_pos+24,x			
;	clc
	adc tilt_hi,y			; add grid tilt value to y position

	sta y_pos_2d+24,x

	lda size_map,y			; convert z pos to a frame number 0-10
	ldy slot+24,x			; get slot number in y
	clc
	adc x11_table,y			; add frame no to slot*11
	tay				; put in y

	lda width_table,y		; lookup width
	sta x_size+24,x

	lda y_pos_2d+24,x
	sta y_bottom+24,x		; bottom y of object
;	sec				; nb carry is clear. so (height plus 1) is subtracted
	sbc height_table,y		; subtract height (height less 1 precalc)
	sta y_top+24,x			; top y of object

	lda x_scroll
	clc
	adc x_pos+24,x
	sta x_pos+24,x			; add scroll value to x position

	jsr x_divide			; convert x to 2d value

	sta x_pos_2d+24,x

	sec		
	sbc x_size+24,x
	cmp #128+75			; check left side of object to right side of screen
	bcs ?2				; branch if off screen (allow for some size overlap)

	lda x_pos_2d+24,x
	adc x_size+24,x
	cmp #128-75			; check right side of object to left side of screen
	bcs ?3				; branch if on screen (allow for some size overlap)
?2
	lda #255
	sta z_pos+24,x			; set z -ve for sort routine ignore
?2a
	lda #hide_bit			; switch off object
	sta properties+24,x
?3
	jmp move_scenery_rtn		; return


;
; scenery explosion
;

move_scenery_explosion

	.local

	dec timer+12,x			; decrement explosion time to live timer
	beq ?3				; branch if run out

	dec z_pos+24,x			; move object forward
	bmi ?3a				; branch if off-screen

	ldy z_pos+24,x

	tya
	clc
	adc #4
	sta divisor			; convert z position to divide by 4 for x division later

	lda y_pos_2d+24,x		; deal with 2d y position only. 3d not needed
	sec
	sbc tilt_hi,y			; remove grid tilt
	adc x_size+24,x			; drop explosion by the 1/2 width (don't ask, it looks ok)
	cmp div43z_2d_table,y		; compare with ground - predefined 43 divided by z table
	bcc ?1

	lda div43z_2d_table,y		; reset to ground
	clc
?1
	adc tilt_hi,y			; add grid tilt value back to y position

	sta y_pos_2d+24,x

	lda size_map,y			; convert z pos to a frame number 0-10
	ldy slot+24,x			; get slot number in y
	clc
	adc x11_table,y			; add frame no to slot*11
	tay				; put in y
	lda width_table,y		; lookup width
	sta x_size+24,x
	lda y_pos_2d+24,x
;	sta y_bottom+24,x		; bottom y of object - not needed for explosions
;	sec				; nb carry is clear. so (height plus 1) is subtracted
	sbc height_table,y		; subtract height (height less 1 precalc)
	sta y_top+24,x			; top y of object

	
	lda x_scroll		
	clc
	adc x_pos+24,x
	sta x_pos+24,x			; add scroll value to x position

	jsr x_divide			; convert x to 2d value

	sta x_pos_2d+24,x

	sec		
	sbc x_size+24,x
	cmp #128+75			; check left side of object to right side of screen
	bcs ?3				; branch if off screen (allow for some size overlap)

	lda x_pos_2d+24,x
	adc x_size+24,x
	cmp #128-75			; check right side of object to left side of screen
	bcs ?4				; branch if on screen (allow for some size overlap)
?3
	lda #255
	sta z_pos+24,x			; set z -ve for sort routine ignore
?3a
	lda #hide_bit+dead_bit		; switch off object. ensure aliens have dead bit set
	sta properties+24,x
?4
	jmp move_scenery_explosion_rtn	; return


;
; play sample
;

play_sample

; slave processor to play sample

	.local

	lda #0
	tay				; 1st byte of sample data

	sta sample_nibble		; initialize counters
	sta sample_index
?0a
	lda #80
	cmp VCOUNT
	bne ?0a				; wait till line 80

	sta next_vcount
?0
	lda next_vcount
	clc
	adc #2				; vcount+2 is next value to wait for
	cmp max_vcount			; max value of vcount is 130 or 155 on PAL
	bcc ?1
	sbc max_vcount			; wrap to 0 at 131 (or 155). 1 at 132 or 156
?1
	sta next_vcount

	lda sample_nibble
	eor #1				; toggle nibble count between 0 and 1
	sta sample_nibble
	beq ?2

; deal with a hi nibble of data
 
	sty temp
	ldy player_pos

	cpy #32
	beq ?1a

	lda (player_source),y		; draw a player missile for die animation
	sta (player_dest),y
	inc player_pos			; next scan line of player missile
?1a
	ldy temp


	lda (sample),y
	lsr a
	lsr a
	lsr a
	lsr a				; get hi nibble into lo byte
	ora #16				; turn on volume only bit
	tax

	bne ?4				; always branch

; deal with a lo nibble of data

?2
	lda (sample),y
	and #15
	ora #16
	tax

	iny				; increment byte pointer for next hi nibble
	bne ?3
	inc sample+1
?3
	lda sample+1
	cmp dest_A+1
	bne ?4
	cpy dest_A
	beq ?6

?4
	sta sample_status		; set finished variable to non-zero - ready to play sample
	
	lda next_vcount			; wait for the specified vcount. 3.9khz is every 2
?5
	cmp VCOUNT
	bne ?5

	stx AUDC3			; play sample asap


	jmp ?0				; always loop

?6
	lda #0

	sta sample_status		; reset sample done flag for vbi
	sta AUDC3			; switch off volume only


?7
	ldy player_pos			; carry on player draw until man die sequencer finished
	cpy #32
	beq ?8				; branch if this player's frame draw finished

	lda (player_source),y		; draw a player missile for die animation
	sta (player_dest),y
	inc player_pos			; next scan line of player missile
?8
	lda player_timer		; do until man die sequencer finished
	bpl ?7


	rts


;
; initialize objects
;

init_objects

; initilize the slot number and properties of all the bullets.
; and make sure the other objects are turned off initially

	.local

	ldy #bullet			; want bullet

	ldx #5				; 6 bullets to initialize
?1
	lda object_to_slot,y		; get slot number of object
	sta slot,x			; save slot number to object x

	lda object_properties,y		; get object properties
	sta properties,x		; save properties for object x

	lda #255
	sta z_pos,x			; set z -ve for sort routine ignore

	dex				; do until all bullets
	bpl ?1

	ldx #25				; 6 missiles, 8 scenery and 12 aliens to initialize
	lda #hide_bit+dead_bit
?2
	sta properties+6,x		; hide the object and make sure it's dead

	sta z_pos+6,x			; set z pos -ve for sort routine ignore

	dex
	bpl ?2				; do until all missiles, scenery and aliens

	ldx #12+8
	lda #0
?3	
	sta control-1,x			; stop scenery
	
	dex
	bne ?3

	
	rts


;
; load object
;

load_object

; load object number passed in x into memory

	.local

	tya
	pha				; save y reg

	stx object_no			; save object number

	lda file_sector_lo,x 		; check whether file is same as last objects
	cmp last_sector_lo		; lo byte check
	bne ?0				; branch if different
	lda file_sector_hi,x
	cmp last_sector_hi		; hi byte check
	bne ?0				; branch if different

	lda object_properties,x		; NB if we're here, it is the same object shape
	and #~10000000			; get whether mirror version of object required
	cmp last_properties	
	bne ?1				; branch if mirror status is different, ie mirror required 
	
	lda object_palette,x
	cmp last_palette
	bne ?1a				; branch if object the same, but a different colour


	lda slot_no			; exact same object - still the last object slot number
	sta object_to_slot,x		; make this object share the slot, only properties differ

; NB It is important that objects sharing a slot have an object number next to each other
; in object_to_slot - the remove_object won't work and will crash unless this is the case
; eg look at how heads have been done.

	bpl ?5				; always branch to exit

?0
	lda #<file_buffer		; setup file buffer as destination
	sta 772
	lda #>file_buffer
	sta 773

	jsr load_file			; object required not in buffer, so load from disk


	ldx object_no			; restore object number to x

	lda object_properties,x
	and #~10000000			; reset mirror for newly loaded object
?1
	sta last_properties		; update last mirror properties for next object check

	jsr create_object		; create object data in object buffer

	ldx object_no			; restore object number to x
?1a
	jsr find_slot			; find free slot. slot num returned in y reg and slot_no


	tya
	sta object_to_slot,x		; object number to slot reference. x holds object
?2

	ldx object_no

	ldy object_palette,x
	sty last_palette		; save palette for next object comparison

	jsr make_colour_table


	ldy slot_no
	jsr slot_addr			; get start address of slot, returned in table1 zp


	ldx x11_table,y			; save slot*11 for start addr in width & height tables			
	stx temp


	ldy #0
?3
	lda object_buffer,y
	sta height_table,x		; 11 bytes for height table
	inx
	iny
	cpy #11
	bne ?3


	ldx temp			; return slot*11 for start addr
?4
	lda object_buffer,y
	sta width_table,x		; next 11 bytes for width table
	inx
	iny
	cpy #11+11
	bne ?4

	lda #>object_buffer		; sprite format data will be in object buffer
	
	ldx frame_no
	bpl ?4a				; branch if sprite format data

	lda #>file_buffer		; raw ASH format data will be in file buffer
	iny				; skip frame no in file buffer
?4a
	sta table2+1			; hi byte of object data

	sty table2			; save lo byte of start of object data for decode


	jsr decode_frame		; decode frames

	jsr insert_stack_addr		; update bank stack addresses for memory management
?5
	pla
	tay				; restore y reg

	rts


;
; load file
;

load_file

; pass file number in x. load file into object buffer

	.local

	lda file_sector_lo,x		; lo sector
	sta DAUX1
	sta last_sector_lo
	lda file_sector_hi,x
	sta DAUX2			; hi sector
	sta last_sector_hi


	lda file_length,x
	sta sect_count


	lda #drive_number		; disk drive to use for data
	sta DUNIT			; either 1 for final game or 3 for development

	lda #82
	sta DCOMND			; command read

?2
	jsr DSKINV			; disk i/o get sector

	inc DAUX1			; get next sector along
	bne ?3
	inc DAUX2
?3
	lda #128			; point to next 128 bytes in ram
	clc
	adc DBUFLO
	sta DBUFLO
	bcc ?4
	inc DBUFHI
?4
	dec sect_count			; do until all sectors
	bne ?2
?5
	rts


;
; make mirror table
;

make_mirror_table

	.local

	ldx #0
?1
	txa
	and #~11000000
	asl a
	rol a
	rol a
	sta temp
	
	txa
	and #~00110000
	lsr a
	lsr a
	ora temp
	sta temp
	
	txa
	and #~00001100
	asl a
	asl a
	ora temp
	sta temp
	
	txa
	and #~00000011
	lsr a
	ror a
	ror a
	ora temp
	
	sta mirror_table,x
	
	dex
	bne ?1
	
	rts


;
; make colour decode table
;

make_colour_table

; make colour decode table. palette required passed in y register

	.local

	lda palette_colours_a,y		; get palette colour mapping for 4 possible pixels
	sta pix_a			; a screen palette
	lda palette_colours_b,y
	sta pix_b			; b screen palette

	ldy #4
?0
	lda pix_a
	and #~00000011
	sta pix_swap_a-1,y		; 2 bit pixel colour for a into byte value
	lda pix_b
	and #~00000011
	sta pix_swap_b-1,y		; 2 bit pixel colour for b into byte value

	lsr pix_a			; get next pixel into bits 0 and 1
	lsr pix_a			; for a
	lsr pix_b
	lsr pix_b			; and b

	dey				; do until all 4 pixels done
	bne ?0


	tya				; y and a must equal zero
?1
	sta colour_table_a,y		; clear temporary area for holding palette
	sta colour_table_b,y		; conversion tables

	dey
	bne ?1				; do until all 256

 
	lda #1				; NB y=0 after last loop is essential
	sta pix_change_count		; count of when to change pixel

	bne ?entry			; jump to entry setup values

?2
	lda pix_a			; put pixel data into table
	ora colour_table_a,y		; combine with other pixels already there
	sta colour_table_a,y
	lda pix_b
	ora colour_table_b,y
	sta colour_table_b,y

	dex
	beq ?3				; branch if pixel change

	iny				; next palette table element
	bne ?2				; always branch.(y=0 only on pixel change)
?3
	iny				; next palette table element
	bne ?5				; branch if not at end of pixel run

	asl pix_change_count		; next 2 bit pixel in palette table. next pixel run
	asl pix_change_count
	bcs ?7				; branch when all pixels done

	ldx #3
?4
	asl pix_swap_a,x		; next 2 bit pixel in swap bytes
	asl pix_swap_a,x
	asl pix_swap_b,x
	asl pix_swap_b,x

	dex				; do until all swap bytes
	bpl ?4
	
?5
	dec pix_index			; next pixel index
	bpl ?6
?entry
	lda #3				; wrap back to 3
	sta pix_index
?6
	ldx pix_index

	lda pix_swap_a,x		; get next pixel
	sta pix_a
	lda pix_swap_b,x
	sta pix_b

	ldx pix_change_count		; reset count
	bne ?2				; always branch
?7
	rts


;
; remove object
;

remove_object

; object number to be removed passed in x reg
; nb can't remove 1st object in stack, or it will crash. make this a permanent object

	.local

	txa				; save x register
	pha


	ldy object_to_slot,x		; find slot number of object to remove

	lda #128			; negative denotes no object
	sta object_to_slot,x		; flag object as no longer in slot

	cpy slot_no			; check for shared slot object
	bne ?0

	jmp ?11				; exit if same slot as last deleted object - its done
?0
	sty slot_no			; save current slot number for next shared slot check

	ldx slot_usage,y		; find stack index of slot

	sta slot_usage,y		; free slot for other use

; get total size of objects frames by each stack.

	lda stack0_lo,x			; stack 0
	sta source_a			; save lo address for copy source
	sec
	sbc stack0_lo+1,x
	sta object_size_lo
	lda stack0_hi,x
	sta source_a+1			; hi address copy source
	sbc stack0_hi+1,x
	sta object_size_hi
	
	bne ?1				; stack contains data belonging to object. copy
	lda object_size_lo
	beq ?2				; no data belonging to object in this stack. skip copy
?1
	lda stack0_lo+1,x
	sta dest_a			; address of copy destination
	lda stack0_hi+1,x
	sta dest_a+1

	ldy #0
	jsr reorg_bank			; copy from source to dest. NB y reg passes bank number

?2
	lda stack1_lo,x			; stack 1
	sta source_a
	sec
	sbc stack1_lo+1,x
	sta object_size_lo+1
	lda stack1_hi,x
	sta source_a+1
	sbc stack1_hi+1,x
	sta object_size_hi+1

	bne ?3				; stack contains data belonging to object. copy
	lda object_size_lo+1
	beq ?4				; no data belonging to object in this stack. skip copy
?3
	lda stack1_lo+1,x
	sta dest_a			; address of copy destination
	lda stack1_hi+1,x
	sta dest_a+1

	ldy #1
	jsr reorg_bank			; copy from source to dest. NB y reg passes bank number

?4
	lda stack2_lo,x
	sta source_a			; stack 2
	sec
	sbc stack2_lo+1,x
	sta object_size_lo+2
	lda stack2_hi,x
	sta source_a+1
	sbc stack2_hi+1,x
	sta object_size_hi+2

	bne ?5				; stack contains data belonging to object. copy
	lda object_size_lo+2
	beq ?6				; no data belonging to object in this stack. skip copy
?5
	lda stack2_lo+1,x
	sta dest_a			; address of copy destination
	lda stack2_hi+1,x
	sta dest_a+1

	ldy #2
	jsr reorg_bank			; copy from source to dest. NB y reg passes bank number

?6	
	lda stack3_lo,x
	sta source_a			; stack 3
	sec
	sbc stack3_lo+1,x
	sta object_size_lo+3
	lda stack3_hi,x
	sta source_a+1
	sbc stack3_hi+1,x
	sta object_size_hi+3

	bne ?7				; stack contains data belonging to object. copy
	lda object_size_lo+3
	beq ?8				; no data belonging to object in this stack. skip copy
?7
	lda stack3_lo+1,x
	sta dest_a			; address of copy destination
	lda stack3_hi+1,x
	sta dest_a+1

	ldy #3
	jsr reorg_bank			; copy from source to dest. NB y reg passes bank number
?8	
	

; get all objects after delete object and update their frame addresses
; re-organize the stack itself with updated addresses, and move entries down

	dex
	cpx stack_pointer
	beq ?10				; branch if last object in stack
?9
	ldy stack_slot,x		; get slot number of object in y 
	inx
	txa
	dex
	sta slot_usage,y		; update stack pointer reference in slot usage by 1
	

	jsr reorg_slot			; reorganize slot. slot no passed in y reg				


	lda stack_slot,x		; copy slot number entry down stack
	sta stack_slot+1,x

	lda stack0_lo,x			; correct stack address entries now bank copied
	sec				; move stack entries down to fill entry
	sbc object_size_lo		; of deleted object
	sta stack0_lo+1,x	
	lda stack0_hi,x			; stack 0
	sbc object_size_hi
	sta stack0_hi+1,x

	lda stack1_lo,x			; correct stack address entries now bank copied
	sec				; move stack entries down to fill entry
	sbc object_size_lo+1		; of deleted object
	sta stack1_lo+1,x	
	lda stack1_hi,x			; stack 1
	sbc object_size_hi+1
	sta stack1_hi+1,x

	lda stack2_lo,x			; correct stack address entries now bank copied
	sec				; move stack entries down to fill entry
	sbc object_size_lo+2		; of deleted object
	sta stack2_lo+1,x	
	lda stack2_hi,x			; stack 2
	sbc object_size_hi+2
	sta stack2_hi+1,x

	lda stack3_lo,x			; correct stack address entries now bank copied
	sec				; move stack entries down to fill entry
	sbc object_size_lo+3		; of deleted object
	sta stack3_lo+1,x	
	lda stack3_hi,x			; stack 3
	sbc object_size_hi+3
	sta stack3_hi+1,x

	dex				; get next slot number in stack
	cpx stack_pointer
	bne ?9				; branch until all left in stack done

?10	
	inx				; update pointer to one less object
	stx stack_pointer
?11
	pla
	tax				; restore x reg

	rts


;
; find slot
;

find_slot

; find available slot for object

	.local

	ldy #max_stage_objects-1	; set to highest object number
?1
	lda slot_usage,y
	bmi ?2

	dey
	bpl ?1
	bmi ?1				; hang as error if no free slots
?2
	sty slot_no
	
	lda stack_pointer
	sta slot_usage,y		; mark slot as being used and what position in stack

	rts


;
; stack insert
;

insert_stack_addr

; new object to insert into stack memory

	.local

	ldx stack_pointer		; stack index

	lda slot_no			; which slot number this stack entry refers to
	sta stack_slot,x

	lda next_mem_lo			; bank 0
	sta stack0_lo,x
	lda next_mem_hi
	sta stack0_hi,x

	lda next_mem_lo+1		; bank 1
	sta stack1_lo,x
	lda next_mem_hi+1
	sta stack1_hi,x

	lda next_mem_lo+2		; bank 2
	sta stack2_lo,x
	lda next_mem_hi+2
	sta stack2_hi,x

	lda next_mem_lo+3		; bank 3
	sta stack3_lo,x
	lda next_mem_hi+3
	sta stack3_hi,x

	dex
	stx stack_pointer		; update pointer

	rts


;
; setup banks
;

init_banks

; reset free memory address of each bank back to start of bank.

	.local

	ldx #2
?1
	lda #0
	sta next_mem_lo,x
	
	lda #$40
	sta next_mem_hi,x

	dex
	bpl ?1

	lda #<bank3_start		; set bank 3 free mem to start after background graphics
	sta next_mem_lo+3
	lda #>bank3_start
	sta next_mem_hi+3

	lda #max_stage_objects-1
	sta stack_pointer		; initialize memory bank stack


	ldx #max_stage_objects-1

	lda #128			; mark slots as unused
?2
	sta slot_usage,x
	
	dex
	bpl ?2

	rts

;
; reorg slot
;

reorg_slot

; re-address frames in slot by size of deleted object in each bank
; slot number passed in y register

	.local

	stx temp

	jsr slot_addr			; get start of slot addresses into table

	ldy #47				; start at last frames hi address
?1
	lda (table1),y			; get hi address
	and #~11100000			; get bank number portion of address
	sta bank_no			; save bank number for later
	asl a
	rol a
	rol a
	rol a
	tax				; get bank number into y as range 0-4

	lda (table1),y
	and #~00011111
	sta mem_reqd+1			; get hi memory part of address

	dey
	lda (table1),y			; get lo address
	sec
	sbc object_size_lo,x
	sta (table1),y			; subtract objects lo size in this bank

	iny				; get to hi address again
	lda mem_reqd+1
	sbc object_size_hi,x		; subtract objects hi size
	ora bank_no			; re-attach bank number to address
	sta (table1),y

	dey				; get next frame
	dey
	bpl ?1

	ldx temp
	rts


;
; slot addr
;

slot_addr

; calculate slot address of slot number given in y register. result returned in zp table

	.local

	lda slot_addr_lo,y		; table of y*48 lo
	sta table1
	lda slot_addr_hi,y		; table of y*48 hi
	sta table1+1

	rts


;
; new screen
;

new_screen

; swap screens to draw to and clear the screens

	.local

	lda #13				; gr 7  i.e. clear dli

	ldx draw_screen_index
	dex				; get next screen to draw to in x
	bpl ?1
	ldx #2
	stx draw_screen_index		; save index
?0
	cpx dlist_index
	beq ?0				; if chosen draw screen same as dlist then wait for flip
	jsr draw_dlist3			; clear dlis on dlist
	jsr tilt
	jsr clear_screen3
	beq ?4				; always branch
?1
	stx draw_screen_index		; save index
	beq ?3
?2
	cpx dlist_index
	beq ?2				; if chosen draw screen same as dlist then wait for flip
	jsr draw_dlist2			; clear dlis on dlist
	jsr tilt
	jsr clear_screen2			
	beq ?4				; always branch
?3
	cpx dlist_index
	beq ?3				; if chosen draw screen same as dlist then wait for flip
	jsr draw_dlist1			; clear dlis on dlist
	jsr tilt
	jsr clear_screen1
?4
	ldx draw_screen_index

	lda screen_addr_a,x		; get address of the new screens to draw to
	sta draw_screen_a
	clc
	adc #247	
	sta clip_screen_a		; start address of minimum y value offscreen

	rts	


;
; clear screen 1
;

clear_screen1

	.local

	ldx #41-5
	lda #0
clear1a
?1	sta screen1a+5,x
	sta screen1a+48*1+5,x
	sta screen1a+48*2+5,x
	sta screen1a+48*3+5,x
	sta screen1a+48*4+5,x
	sta screen1a+48*5+5,x
	sta screen1a+48*6+5,x
	sta screen1a+48*7+5,x
	sta screen1a+48*8+5,x
	sta screen1a+48*9+5,x
	
	sta screen1a+48*10+5,x
	sta screen1a+48*11+5,x
	sta screen1a+48*12+5,x
	sta screen1a+48*13+5,x
	sta screen1a+48*14+5,x
	sta screen1a+48*15+5,x
	sta screen1a+48*16+5,x
	sta screen1a+48*17+5,x
	sta screen1a+48*18+5,x
	sta screen1a+48*19+5,x

	sta screen1a+48*20+5,x
	sta screen1a+48*21+5,x
	sta screen1a+48*22+5,x
	sta screen1a+48*23+5,x
	sta screen1a+48*24+5,x
	sta screen1a+48*25+5,x
	sta screen1a+48*26+5,x
	sta screen1a+48*27+5,x
	sta screen1a+48*28+5,x
	sta screen1a+48*29+5,x

	sta screen1a+48*30+5,x
	sta screen1a+48*31+5,x
	sta screen1a+48*32+5,x
	sta screen1a+48*33+5,x
	sta screen1a+48*34+5,x
	sta screen1a+48*35+5,x
	sta screen1a+48*36+5,x
	sta screen1a+48*37+5,x
	sta screen1a+48*38+5,x
	sta screen1a+48*39+5,x

	sta screen1a+48*40+5,x
;	sta screen1a+48*41+5,x
;	sta screen1a+48*42+5,x
;	sta screen1a+48*43+5,x
;	sta screen1a+48*44+5,x
;	sta screen1a+48*45+5,x
;	sta screen1a+48*46+5,x
;	sta screen1a+48*47+5,x
;	sta screen1a+48*48+5,x
	sta screen1a+48*49+5,x

	sta screen1a+48*50+5,x
	sta screen1a+48*51+5,x
	sta screen1a+48*52+5,x
	sta screen1a+48*53+5,x
	sta screen1a+48*54+5,x
	sta screen1a+48*55+5,x
	sta screen1a+48*56+5,x
	sta screen1a+48*57+5,x
	sta screen1a+48*58+5,x
	sta screen1a+48*59+5,x

	sta screen1a+48*60+5,x
	sta screen1a+48*61+5,x
	sta screen1a+48*62+5,x
	sta screen1a+48*63+5,x
	sta screen1a+48*64+5,x
	sta screen1a+48*65+5,x
	sta screen1a+48*66+5,x
	sta screen1a+48*67+5,x
	sta screen1a+48*68+5,x
	sta screen1a+48*69+5,x

	sta screen1a+48*70+5,x
	sta screen1a+48*71+5,x
	sta screen1a+48*72+5,x
	sta screen1a+48*73+5,x
	sta screen1a+48*74+5,x
	sta screen1a+48*75+5,x
	sta screen1a+48*76+5,x
	sta screen1a+48*77+5,x
	sta screen1a+48*78+5,x
	sta screen1a+48*79+5,x

	sta screen1a+48*80+5,x
	sta screen1a+48*81+5,x
	sta screen1a+48*82+5,x
	sta screen1a+48*83+5,x
	sta screen1a+48*84+5,x
clear1b
	sta screen1b+5,x
	sta screen1b+48*1+5,x
	sta screen1b+48*2+5,x
	sta screen1b+48*3+5,x
	sta screen1b+48*4+5,x
	sta screen1b+48*5+5,x
	sta screen1b+48*6+5,x
	sta screen1b+48*7+5,x
	sta screen1b+48*8+5,x
	sta screen1b+48*9+5,x
	
	sta screen1b+48*10+5,x
	sta screen1b+48*11+5,x
	sta screen1b+48*12+5,x
	sta screen1b+48*13+5,x
	sta screen1b+48*14+5,x
	sta screen1b+48*15+5,x
	sta screen1b+48*16+5,x
	sta screen1b+48*17+5,x
	sta screen1b+48*18+5,x
	sta screen1b+48*19+5,x

	sta screen1b+48*20+5,x
	sta screen1b+48*21+5,x
	sta screen1b+48*22+5,x
	sta screen1b+48*23+5,x
	sta screen1b+48*24+5,x
	sta screen1b+48*25+5,x
	sta screen1b+48*26+5,x
	sta screen1b+48*27+5,x
	sta screen1b+48*28+5,x
	sta screen1b+48*29+5,x

	sta screen1b+48*30+5,x
	sta screen1b+48*31+5,x
	sta screen1b+48*32+5,x
	sta screen1b+48*33+5,x
	sta screen1b+48*34+5,x
	sta screen1b+48*35+5,x
	sta screen1b+48*36+5,x
	sta screen1b+48*37+5,x
	sta screen1b+48*38+5,x
	sta screen1b+48*39+5,x

	sta screen1b+48*40+5,x
;	sta screen1b+48*41+5,x
;	sta screen1b+48*42+5,x
;	sta screen1b+48*43+5,x
;	sta screen1b+48*44+5,x
;	sta screen1b+48*45+5,x
;	sta screen1b+48*46+5,x
;	sta screen1b+48*47+5,x
;	sta screen1b+48*48+5,x
	sta screen1b+48*49+5,x

	sta screen1b+48*50+5,x
	sta screen1b+48*51+5,x
	sta screen1b+48*52+5,x
	sta screen1b+48*53+5,x
	sta screen1b+48*54+5,x
	sta screen1b+48*55+5,x
	sta screen1b+48*56+5,x
	sta screen1b+48*57+5,x
	sta screen1b+48*58+5,x
	sta screen1b+48*59+5,x

	sta screen1b+48*60+5,x
	sta screen1b+48*61+5,x
	sta screen1b+48*62+5,x
	sta screen1b+48*63+5,x
	sta screen1b+48*64+5,x
	sta screen1b+48*65+5,x
	sta screen1b+48*66+5,x
	sta screen1b+48*67+5,x
	sta screen1b+48*68+5,x
	sta screen1b+48*69+5,x

	sta screen1b+48*70+5,x
	sta screen1b+48*71+5,x
	sta screen1b+48*72+5,x
	sta screen1b+48*73+5,x
	sta screen1b+48*74+5,x
	sta screen1b+48*75+5,x
	sta screen1b+48*76+5,x
	sta screen1b+48*77+5,x
	sta screen1b+48*78+5,x
	sta screen1b+48*79+5,x

	sta screen1b+48*80+5,x
	sta screen1b+48*81+5,x
	sta screen1b+48*82+5,x
	sta screen1b+48*83+5,x
	sta screen1b+48*84+5,x


	dex
	beq ?2
	jmp ?1
?2	rts


;
; clear screen 2
;

clear_screen2

	.local

	ldx #41-5
	lda #0
clear2a
?1	sta screen2a+5,x
	sta screen2a+48*1+5,x
	sta screen2a+48*2+5,x
	sta screen2a+48*3+5,x
	sta screen2a+48*4+5,x
	sta screen2a+48*5+5,x
	sta screen2a+48*6+5,x
	sta screen2a+48*7+5,x
	sta screen2a+48*8+5,x
	sta screen2a+48*9+5,x
	
	sta screen2a+48*10+5,x
	sta screen2a+48*11+5,x
	sta screen2a+48*12+5,x
	sta screen2a+48*13+5,x
	sta screen2a+48*14+5,x
	sta screen2a+48*15+5,x
	sta screen2a+48*16+5,x
	sta screen2a+48*17+5,x
	sta screen2a+48*18+5,x
	sta screen2a+48*19+5,x

	sta screen2a+48*20+5,x
	sta screen2a+48*21+5,x
	sta screen2a+48*22+5,x
	sta screen2a+48*23+5,x
	sta screen2a+48*24+5,x
	sta screen2a+48*25+5,x
	sta screen2a+48*26+5,x
	sta screen2a+48*27+5,x
	sta screen2a+48*28+5,x
	sta screen2a+48*29+5,x

	sta screen2a+48*30+5,x
	sta screen2a+48*31+5,x
	sta screen2a+48*32+5,x
	sta screen2a+48*33+5,x
	sta screen2a+48*34+5,x
	sta screen2a+48*35+5,x
	sta screen2a+48*36+5,x
	sta screen2a+48*37+5,x
	sta screen2a+48*38+5,x
	sta screen2a+48*39+5,x

	sta screen2a+48*40+5,x
;	sta screen2a+48*41+5,x
;	sta screen2a+48*42+5,x
;	sta screen2a+48*43+5,x
;	sta screen2a+48*44+5,x
;	sta screen2a+48*45+5,x
;	sta screen2a+48*46+5,x
;	sta screen2a+48*47+5,x
;	sta screen2a+48*48+5,x
	sta screen2a+48*49+5,x

	sta screen2a+48*50+5,x
	sta screen2a+48*51+5,x
	sta screen2a+48*52+5,x

	sta screen2a+48*53+5,x
	sta screen2a+48*54+5,x
	sta screen2a+48*55+5,x
	sta screen2a+48*56+5,x
	sta screen2a+48*57+5,x
	sta screen2a+48*58+5,x
	sta screen2a+48*59+5,x

	sta screen2a+48*60+5,x
	sta screen2a+48*61+5,x
	sta screen2a+48*62+5,x
	sta screen2a+48*63+5,x
	sta screen2a+48*64+5,x
	sta screen2a+48*65+5,x
	sta screen2a+48*66+5,x
	sta screen2a+48*67+5,x
	sta screen2a+48*68+5,x
	sta screen2a+48*69+5,x

	sta screen2a+48*70+5,x
	sta screen2a+48*71+5,x
	sta screen2a+48*72+5,x
	sta screen2a+48*73+5,x
	sta screen2a+48*74+5,x
	sta screen2a+48*75+5,x
	sta screen2a+48*76+5,x
	sta screen2a+48*77+5,x
	sta screen2a+48*78+5,x
	sta screen2a+48*79+5,x

	sta screen2a+48*80+5,x
	sta screen2a+48*81+5,x
	sta screen2a+48*82+5,x
	sta screen2a+48*83+5,x
	sta screen2a+48*84+5,x
clear2b
	sta screen2b+5,x
	sta screen2b+48*1+5,x
	sta screen2b+48*2+5,x
	sta screen2b+48*3+5,x
	sta screen2b+48*4+5,x
	sta screen2b+48*5+5,x
	sta screen2b+48*6+5,x
	sta screen2b+48*7+5,x
	sta screen2b+48*8+5,x
	sta screen2b+48*9+5,x
	
	sta screen2b+48*10+5,x
	sta screen2b+48*11+5,x
	sta screen2b+48*12+5,x
	sta screen2b+48*13+5,x
	sta screen2b+48*14+5,x
	sta screen2b+48*15+5,x
	sta screen2b+48*16+5,x
	sta screen2b+48*17+5,x
	sta screen2b+48*18+5,x
	sta screen2b+48*19+5,x

	sta screen2b+48*20+5,x
	sta screen2b+48*21+5,x
	sta screen2b+48*22+5,x
	sta screen2b+48*23+5,x
	sta screen2b+48*24+5,x
	sta screen2b+48*25+5,x
	sta screen2b+48*26+5,x
	sta screen2b+48*27+5,x
	sta screen2b+48*28+5,x
	sta screen2b+48*29+5,x

	sta screen2b+48*30+5,x
	sta screen2b+48*31+5,x
	sta screen2b+48*32+5,x
	sta screen2b+48*33+5,x
	sta screen2b+48*34+5,x
	sta screen2b+48*35+5,x
	sta screen2b+48*36+5,x
	sta screen2b+48*37+5,x
	sta screen2b+48*38+5,x
	sta screen2b+48*39+5,x

	sta screen2b+48*40+5,x
;	sta screen2b+48*41+5,x
;	sta screen2b+48*42+5,x
;	sta screen2b+48*43+5,x
;	sta screen2b+48*44+5,x
;	sta screen2b+48*45+5,x
;	sta screen2b+48*46+5,x
;	sta screen2b+48*47+5,x
;	sta screen2b+48*48+5,x
	sta screen2b+48*49+5,x

	sta screen2b+48*50+5,x
	sta screen2b+48*51+5,x
	sta screen2b+48*52+5,x
	sta screen2b+48*53+5,x
	sta screen2b+48*54+5,x
	sta screen2b+48*55+5,x
	sta screen2b+48*56+5,x
	sta screen2b+48*57+5,x
	sta screen2b+48*58+5,x
	sta screen2b+48*59+5,x

	sta screen2b+48*60+5,x
	sta screen2b+48*61+5,x
	sta screen2b+48*62+5,x
	sta screen2b+48*63+5,x
	sta screen2b+48*64+5,x
	sta screen2b+48*65+5,x
	sta screen2b+48*66+5,x
	sta screen2b+48*67+5,x
	sta screen2b+48*68+5,x
	sta screen2b+48*69+5,x

	sta screen2b+48*70+5,x
	sta screen2b+48*71+5,x
	sta screen2b+48*72+5,x
	sta screen2b+48*73+5,x
	sta screen2b+48*74+5,x
	sta screen2b+48*75+5,x
	sta screen2b+48*76+5,x
	sta screen2b+48*77+5,x
	sta screen2b+48*78+5,x
	sta screen2b+48*79+5,x

	sta screen2b+48*80+5,x
	sta screen2b+48*81+5,x
	sta screen2b+48*82+5,x
	sta screen2b+48*83+5,x
	sta screen2b+48*84+5,x


	dex
	beq ?2

	jmp ?1
?2	rts


;
; clear screen 3
;

clear_screen3

	.local

	ldx #41-5
	lda #0
clear3a
?1	sta screen3a+5,x
	sta screen3a+48*1+5,x
	sta screen3a+48*2+5,x
	sta screen3a+48*3+5,x
	sta screen3a+48*4+5,x
	sta screen3a+48*5+5,x
	sta screen3a+48*6+5,x
	sta screen3a+48*7+5,x
	sta screen3a+48*8+5,x
	sta screen3a+48*9+5,x
	
	sta screen3a+48*10+5,x
	sta screen3a+48*11+5,x
	sta screen3a+48*12+5,x
	sta screen3a+48*13+5,x
	sta screen3a+48*14+5,x
	sta screen3a+48*15+5,x
	sta screen3a+48*16+5,x
	sta screen3a+48*17+5,x
	sta screen3a+48*18+5,x
	sta screen3a+48*19+5,x

	sta screen3a+48*20+5,x
	sta screen3a+48*21+5,x
	sta screen3a+48*22+5,x
	sta screen3a+48*23+5,x
	sta screen3a+48*24+5,x
	sta screen3a+48*25+5,x
	sta screen3a+48*26+5,x
	sta screen3a+48*27+5,x
	sta screen3a+48*28+5,x
	sta screen3a+48*29+5,x

	sta screen3a+48*30+5,x
	sta screen3a+48*31+5,x
	sta screen3a+48*32+5,x
	sta screen3a+48*33+5,x
	sta screen3a+48*34+5,x
	sta screen3a+48*35+5,x
	sta screen3a+48*36+5,x
	sta screen3a+48*37+5,x
	sta screen3a+48*38+5,x
	sta screen3a+48*39+5,x

	sta screen3a+48*40+5,x
;	sta screen3a+48*41+5,x
;	sta screen3a+48*42+5,x
;	sta screen3a+48*43+5,x
;	sta screen3a+48*44+5,x
;	sta screen3a+48*45+5,x
;	sta screen3a+48*46+5,x
;	sta screen3a+48*47+5,x
;	sta screen3a+48*48+5,x
	sta screen3a+48*49+5,x

	sta screen3a+48*50+5,x
	sta screen3a+48*51+5,x
	sta screen3a+48*52+5,x
	sta screen3a+48*53+5,x
	sta screen3a+48*54+5,x
	sta screen3a+48*55+5,x
	sta screen3a+48*56+5,x
	sta screen3a+48*57+5,x
	sta screen3a+48*58+5,x
	sta screen3a+48*59+5,x

	sta screen3a+48*60+5,x
	sta screen3a+48*61+5,x
	sta screen3a+48*62+5,x
	sta screen3a+48*63+5,x
	sta screen3a+48*64+5,x
	sta screen3a+48*65+5,x
	sta screen3a+48*66+5,x
	sta screen3a+48*67+5,x
	sta screen3a+48*68+5,x
	sta screen3a+48*69+5,x

	sta screen3a+48*70+5,x
	sta screen3a+48*71+5,x
	sta screen3a+48*72+5,x
	sta screen3a+48*73+5,x
	sta screen3a+48*74+5,x
	sta screen3a+48*75+5,x
	sta screen3a+48*76+5,x
	sta screen3a+48*77+5,x
	sta screen3a+48*78+5,x
	sta screen3a+48*79+5,x

	sta screen3a+48*80+5,x
	sta screen3a+48*81+5,x
	sta screen3a+48*82+5,x
	sta screen3a+48*83+5,x
	sta screen3a+48*84+5,x
clear3b
	sta screen3b+5,x
	sta screen3b+48*1+5,x
	sta screen3b+48*2+5,x
	sta screen3b+48*3+5,x
	sta screen3b+48*4+5,x
	sta screen3b+48*5+5,x
	sta screen3b+48*6+5,x
	sta screen3b+48*7+5,x
	sta screen3b+48*8+5,x
	sta screen3b+48*9+5,x
	
	sta screen3b+48*10+5,x
	sta screen3b+48*11+5,x
	sta screen3b+48*12+5,x
	sta screen3b+48*13+5,x
	sta screen3b+48*14+5,x
	sta screen3b+48*15+5,x
	sta screen3b+48*16+5,x
	sta screen3b+48*17+5,x
	sta screen3b+48*18+5,x
	sta screen3b+48*19+5,x

	sta screen3b+48*20+5,x
	sta screen3b+48*21+5,x
	sta screen3b+48*22+5,x
	sta screen3b+48*23+5,x
	sta screen3b+48*24+5,x
	sta screen3b+48*25+5,x
	sta screen3b+48*26+5,x
	sta screen3b+48*27+5,x
	sta screen3b+48*28+5,x
	sta screen3b+48*29+5,x

	sta screen3b+48*30+5,x
	sta screen3b+48*31+5,x
	sta screen3b+48*32+5,x
	sta screen3b+48*33+5,x
	sta screen3b+48*34+5,x
	sta screen3b+48*35+5,x
	sta screen3b+48*36+5,x
	sta screen3b+48*37+5,x
	sta screen3b+48*38+5,x
	sta screen3b+48*39+5,x

	sta screen3b+48*40+5,x
;	sta screen3b+48*41+5,x
;	sta screen3b+48*42+5,x
;	sta screen3b+48*43+5,x
;	sta screen3b+48*44+5,x
;	sta screen3b+48*45+5,x
;	sta screen3b+48*46+5,x
;	sta screen3b+48*47+5,x
;	sta screen3b+48*48+5,x
	sta screen3b+48*49+5,x

	sta screen3b+48*50+5,x
	sta screen3b+48*51+5,x
	sta screen3b+48*52+5,x
	sta screen3b+48*53+5,x
	sta screen3b+48*54+5,x
	sta screen3b+48*55+5,x
	sta screen3b+48*56+5,x
	sta screen3b+48*57+5,x
	sta screen3b+48*58+5,x
	sta screen3b+48*59+5,x

	sta screen3b+48*60+5,x
	sta screen3b+48*61+5,x
	sta screen3b+48*62+5,x
	sta screen3b+48*63+5,x
	sta screen3b+48*64+5,x
	sta screen3b+48*65+5,x
	sta screen3b+48*66+5,x
	sta screen3b+48*67+5,x
	sta screen3b+48*68+5,x
	sta screen3b+48*69+5,x

	sta screen3b+48*70+5,x
	sta screen3b+48*71+5,x
	sta screen3b+48*72+5,x
	sta screen3b+48*73+5,x
	sta screen3b+48*74+5,x
	sta screen3b+48*75+5,x
	sta screen3b+48*76+5,x
	sta screen3b+48*77+5,x
	sta screen3b+48*78+5,x
	sta screen3b+48*79+5,x

	sta screen3b+48*80+5,x
	sta screen3b+48*81+5,x
	sta screen3b+48*82+5,x
	sta screen3b+48*83+5,x
	sta screen3b+48*84+5,x


	dex
	beq ?2

	jmp ?1
?2	rts


;
; draw dlis
;

draw_dlis

	.local

	lda #13+INT			; gr 7 + dli

	ldx draw_screen_index
	beq ?2				; if index 0 then dlist1
	cpx #1
	beq ?1				; if index 1 then dlist 2 else drop through to dlist 3			

	jsr draw_dlist3			; into dlists from buffer_read
	rts
?1
	jsr draw_dlist2			; into dlists from buffer_read
	rts
?2
	jsr draw_dlist1			; into dlists from buffer_read
	rts


;
; draw dlist1
;

draw_dlist1

; put dlis onto dlist of screen 1 to display from dli_buffer

	.local

	ldy #0				; screen 1 buffer area

	ldx dli_buffer,y		; position into x
	beq ?2				; exit if no DLIS

	cpx #1
	bne ?1				; skip if not special case of 1st DLI
	
	ora #RLDMSC			; turn on load memory scan (LMS)
	sta dlist1+7			; save graphic mode with LMS
	eor #RLDMSC			; ensure LMS turned off for subsequent DLIS
	
	iny
	ldx dli_buffer,y
	beq ?2
?1
	sta dlist1+8,x			; into dlist
	
	iny				; next in buffer
	ldx dli_buffer,y		; position into x
	bne ?1
?2
	rts


;
; draw dlist2
;

draw_dlist2

; put dlis onto dlist of screen 2 to display from dli_buffer

	.local

	ldy #max_dlis			; screen 2 buffer area

	ldx dli_buffer,y		; position into x
	beq ?2				; exit if no DLIS

	cpx #1
	bne ?1				; skip if not special case of 1st DLI
	
	ora #RLDMSC			; turn on load memory scan (LMS)
	sta dlist2+7			; save graphic mode with LMS
	eor #RLDMSC			; ensure LMS turned off for subsequent DLIS
	
	iny
	ldx dli_buffer,y
	beq ?2				; 0 means done
?1
	sta dlist2+8,x			; into dlist

	iny				; next in buffer
	ldx dli_buffer,y		; position into x
	bne ?1
?2
	rts


;
; draw dlist3
;

draw_dlist3

; put dlis onto dlist of screen 3 to display from dli_buffer

	.local

	ldy #max_dlis*2			; screen 3 buffer area

	ldx dli_buffer,y		; position into x
	beq ?2				; exit if no DLIS

	cpx #1
	bne ?1				; skip if not special case of 1st DLI
	
	ora #RLDMSC			; turn on load memory scan (LMS)
	sta dlist3+7			; save graphic mode with LMS
	eor #RLDMSC			; ensure LMS turned off for subsequent DLIS
	
	iny
	ldx dli_buffer,y
	beq ?2				; 0 means done
?1
	sta dlist3+8,x			; into dlist

	iny				; next in buffer
	ldx dli_buffer,y		; position into x
	bne ?1
?2
	rts


;
; copy title screen dlis
;

copy_title_dlis

; sky dlis and colours into buffers
; NB there is one extra colour in the colour list than the position list.

	.local

	ldx draw_screen_index
	ldy buffer_addr,x		; get dli buffer index for this draw screen

	ldx #0

	lda sky_colour,x		; sky colour
?2
	sta colour_buffer,y

	lda sky_pos,x			; sky position
	sta dli_buffer,y

	iny
	inx

	lda sky_colour,x		; next sky colour
	bne ?2				; do until reached 0 in the colour list

	sta colour_buffer,y		; save extra colour black
	
	
	rts


;
; move dlis
;

move_dlis

; move dlis for top grid, sky and floor grid
; write positions into dli buffer and colours into colour buffer
; colour cycle colours in grid

	.local

	dec grid_index			; next grid animation
	bpl ?1

	lda #2
	sta grid_index			; reset if wrapped

	lda grid_colour_swap
	eor #1
	sta grid_colour_swap		; flag swap of grid colours for animation to occur
?1
	dec grid_colour_timer		; timer till next colour change
	bne ?3				; branch if not time yet
	
	dec grid_colour_index		; next colour change
	bpl ?2
	
	lda grid_colour_wrap
	sta grid_colour_index		; wrap index if needed
?2
	ldx grid_colour_index
	
	lda grid_colour1,x		; next grid colour 1
	sta grid_colour			; save in grid colour 1 and 3 
	sta grid_colour+2

	lda grid_colour2,x		; next grid colour 2
	sta grid_colour+1		; save in grid colour 2

	lda grid_colour_pause,x
	sta grid_colour_timer		; set timer till next colour change
?3
	lda #0
	sta top_grid_pos		; default top grid pos to top of screen


	ldx grid_colour_swap		; NB 1 if colour 1 and colour 2 should be swapped for animation

	lda grid_colour,x		; colour at the front of grid(s)
	sta colour1
	
	inx				; get other colour
	lda grid_colour,x		; other grid colour
	sta colour2


	ldx draw_screen_index
	ldy buffer_addr,x		; get dli buffer index for this draw screen


;
; top grid up/down
;

	.local

	bit top_grid_status		; NB colour2 must have one of bits 0-5 on for this to work..
	bne ?1
	
	jmp ?3h				; jump if top grid off
?1	
	bmi ?2a				; branch if top grid set to going up (bit 7)
	bvc ?3a				; branch if top grid already fully down (bit 6)

	ldx #20				; 21 grid positions
?1a
	lda top_grid_lo,x		; scroll grid positions down by z_scroll values
	clc
	adc z_scroll_lo,x
	sta top_grid_lo,x
	lda top_grid_hi,x
	adc #0
	sta top_grid_hi,x

	dex				; do for all grid positions except 0 which is fixed
	bne ?1a

	lda #49-2			; back of floor grid less 2 pixel gap
	cmp top_grid_hi+20		; compare to back of top grid
	bpl ?3a				; branch if top grid is 2 pixels above floor grid
	
	lda #top_grid_on
	sta top_grid_status		; turn off grid down bits, just leave on top grid bits
	
	bmi ?3a				; always branch
?2a
	ldx #20				; 21 grid positions
?2b
	lda top_grid_lo,x		; scroll grid positions up by z_scroll values
	sec
	sbc z_scroll_lo,x
	sta top_grid_lo,x
	lda top_grid_hi,x
	sbc #0
	sta top_grid_hi,x
	
	dex				; do for all grid positions except 0 which is fixed
	bne ?2b

	lda top_grid_hi+20		; bottom of top grid
	clc
	adc tilt_hi+20			; plus tilt

	cmp #2
	bpl ?3a				; branch if grid not fully up yet

	lda #0
	sta top_grid_status		; switch off top grid
	beq ?3h				; always branch


;
; top grid positions and colours
;

?3a
	ldx grid_index			; get animation frame of grid

	lda top_grid_hi+1,x
	clc
	adc tilt_hi+1,x
	
	bne ?3c				; always do first top grid position
?3b
	lda top_grid_hi+1,x
	adc tilt_hi+1,x			; add tilt to the top grid position values

	cmp dli_buffer-1,y		; check against last dli position
	beq ?3d
	bmi ?3d				; branch if <= to last position. ie already a dli there
?3c
	sta dli_buffer,y		; position into dli buffer

	lda colour1
	sta colour_buffer,y		; colour into colour buffer

	iny				; next buffer index

	bne ?3f				; always branch	to colour 2 loop
?3d
	inx				; grid colour strip is 3 positions deep
	inx
	inx

	cpx #20
	bcc ?3b				; do until all grid positions

	bcs ?3g				; always branch
?3e
	lda top_grid_hi+1,x
	adc tilt_hi+1,x			; add tilt to the top grid position values
	
	cmp dli_buffer-1,y		; check against last dli position
	beq ?3f
	bmi ?3f				; branch if <= to last position. ie already a dli there

	sta dli_buffer,y		; position into dli buffer

	lda colour2
	sta colour_buffer,y		; colour into colour buffer
	
	iny
	
	bne ?3d				; always branch to colour 1 loop
?3f
	inx				; grid colour strip is 3 positions deep
	inx
	inx
	
	cpx #20
	bcc ?3e				; do until all grid positions
?3g
	lda dli_buffer-1,y		; last top grid dli position
	sta top_grid_pos
?3h


;
; scroll sky up/down according to sky_scroll value. sky dlis and colours into buffers
;

	.local

	ldx sky_index			; get current position in sky list
	beq ?1b				; if not scrolled don't check for previous position on screen
?1a
	lda sky_pos-1,x			; previous sky position
	clc
	adc sky_scroll			; add scroll value to previous sky position
	
	cmp top_grid_pos
	bmi ?1b
	beq ?1b				; branch if previous position not past top grid
	
	dex				; bring previous position into view
	stx sky_index			; save sky index
	bne ?1a				; branch back to check if any more previous DLIS past top grid

	beq ?2b				; always branch	
?1b
	lda sky_pos,x
	beq ?2c				; branch if end of sky

	clc
	adc sky_scroll			; add scroll value to sky position
	
	cmp top_grid_pos
	beq ?1d
	bpl ?2b				; branch past if position on screen

?1d
	inx				; hide position from view
	stx sky_index			; save sky_index
	bne ?1b				; always branch back to check if next DLI is past top grid

?2a
	clc
	adc sky_scroll
?2b
	sta dli_buffer,y

	lda sky_colour,x
	sta colour_buffer,y

	iny
	inx

	lda sky_pos,x			; next sky position
	bne ?2a				; do until reached 0 in the list
?2c
	lda sky_colour,x
	sta colour_buffer,y		; extra sky colour because of fixed dli at top

	sty temp			; save buffer index


;
; floor grid positions
;

	.local

	ldx grid_index			; get animation frame of grid
	lda grid_add,x
	tax				; get furthest grid position index
?1
	lda grid_init_pos+1,x		; NB pos+0 is always drawn so must discount it		
	clc				; add tilt to the initial floor level values
	adc tilt_hi+1,x

	cmp dli_buffer-1,y		; check against last dli
	beq ?2
	bcc ?2				; branch if <= to last position. ie already a dli there

	sta dli_buffer,y		; position into dli buffer

	iny				; next buffer index
?2
	dex				; grid colour strip is 3 positions deep
	dex
	dex
	bpl ?1				; do until all grid positions
?3
	lda #0				; write zero as end of data in buffer
	sta dli_buffer,y


;
; floor colour buffer code
;

	ldx temp			; get back buffer addr before grid positions done

	tya
	sec
	sbc temp			; subtract buffer addr after grid positions done
	tay				; to give grid dli count

	lsr a
	bcs ?5				; branch if an odd number of dlis, last dli must be colour 1
?4 
	lda colour2
	sta colour_buffer+1,x		; grid colour 2 into buffer (+1 coz of extra sky colour)
	inx

	dey				; next dli
?5
	lda colour1
	sta colour_buffer+1,x		; grid colour 1 into buffer (+1 coz of extra sky colour)
	inx

	dey
	bne ?4				; do until last dli on grid

	tya				; last dli is always black
	sta colour_buffer+1,x		; last dli at lives (+1 coz of extra sky colour)


	rts


;
; tilt
;

tilt

; do y tilting of grid. provide tilt value so objects can be placed on grid
; NB horizon value to give a dli position is man_y/2 + 4
; move horizon and create self modifying code for fast background clear

	.local

	ldx #20				; last z pos
	
	lda man_y_pos			; back of grid allowed to tilt up to 1/2 man's
	lsr a				; y position, plus an adjustment
	adc #4				; no need to clear carry

	cmp horizon			; check back of grid to man's position
	beq ?1b				; branch past if same
	bcs ?2				; branch if move down required

	dec horizon			; move up 1 pix
	dec sky_scroll			; back of grid and sky
?1
	lda tilt_lo,x
	sec				; tilt grid
	sbc z_scroll_lo,x
	sta tilt_lo,x
	lda tilt_hi,x
	sbc #0
	sta tilt_hi,x

	dex				; do for all grid positions
	bne ?1	

	
	lda horizon			; horizon
	clc
	adc #9				; add offset till background start
	sta background_y		; (save as background y position)
	asl a
	adc background_y		; *3 into index x
	tax

	lda clear1a+3+1,x		; get lo byte of the sta at background+1 for clear1a
	sbc #47				; sub 48 to restore value for next line up. NB c clear
	sta clear1a+1,x			; save in next line up
	sta clear1b+1,x			; save lo value for all 6 other screens
	sta clear2a+1,x
	sta clear2b+1,x
	sta clear3a+1,x
	sta clear3b+1,x

	lda clear3b+3+2,x
	sbc #0				; sub 1 from hi address of background+1. note C clear
	sta clear3b+2,x			; save in background
	sbc #>screen3b->screen3a	; sub screen offset to get to screen a from b
	sta clear3a+2,x			; do for all screens
	sbc #>screen3a->screen2b
	sta clear2b+2,x
	sbc #>screen2b->screen2a
	sta clear2a+2,x
	sbc #>screen2a->screen1b
	sta clear1b+2,x
	sbc #>screen1b->screen1a
	sta clear1a+2,x
?1b
	rts
?2
	inc horizon			; move down 1 pix
	inc sky_scroll			; back of grid and sky
?3
	lda tilt_lo,x			; tilt grid
	clc
	adc z_scroll_lo,x
	sta tilt_lo,x
	lda tilt_hi,x
	adc #0
	sta tilt_hi,x

	dex
	bne ?3				; do for all grid positions

?4
	lda horizon			; horizon
	clc
	adc #9				; add offset till background start
	sta background_y		; (save as background y position)
	
	asl a
	adc background_y		; *3 into index x
	tax

	lda clear1a-6+1,x		; get lo byte of the sta at background-2 for clear1a
	adc #48				; add 48 to restore value for next line down
	sta clear1a-3+1,x		; save in next line down
	sta clear1b-3+1,x		; save lo value for all 6 other screens
	sta clear2a-3+1,x
	sta clear2b-3+1,x
	sta clear3a-3+1,x
	sta clear3b-3+1,x

	lda clear1a-6+2,x
	adc #0				; add 1 to hi address of background-2.
	sta clear1a-3+2,x		; save in background-1
	adc #>screen1b->screen1a
	sta clear1b-3+2,x		; add offsets for all screens
	adc #>screen2a->screen1b
	sta clear2a-3+2,x
	adc #>screen2b->screen2a
	sta clear2b-3+2,x
	adc #>screen3a->screen2b
	sta clear3a-3+2,x
	adc #>screen3b->screen3a
	sta clear3b-3+2,x

	rts


;
; clear players
;

clear_players

; clear the player missile graphics before first use

	.local

	ldx #127
	lda #0

?1	sta player0,x
	sta player1,x
	sta player2,x
	sta player3,x

	dex
	bpl ?1

	rts

;
; clear players b area
;

clear_b_players

; clear the b players area only, which is usually for screen data

	.local

	ldy #0
	tya
?1
	sta (source_A),y
	sta (source_B),y

	dey
	bne ?1

	rts


;
; x multiply
;

x_multiply

; multiply accumulator by multiplier. can be +ve or -ve. multiplicand is divided by 4 and the multiplier
; should have 4 added to give result of multiplication of 1,1.25,1.5,1.75,2 etc.
; multiplicand in pixels 4 pixels per byte (+ve or -ve)
; result in bytes (+ve or -ve), ie pixels/4.
; NB No lo byte result is needed until the last loop, because of the dividend divide by 4 and the pixels
; divided by 4, and the multiplier is only 5 bits long.

	.local

	php				; save sign of number
	bpl ?1

	eor #255			; convert to positive number if a negative
	clc
	adc #1
?1
	sta multiplicand		; multiplicand passed in A
	lda #0				; initialize high result. NB no lo result is needed
	
	lsr multiplier
	bcc ?2				; branch if no *1

	clc
	adc multiplicand
?2
	ror a				; shift result down to lo direction

	lsr multiplier
	bcc ?3				; branch if no *2

	clc
	adc multiplicand
?3
	ror a				; shift result down to lo direction

	lsr multiplier
	bcc ?4				; branch if no *4

	clc
	adc multiplicand
?4
	ror a				; shift result down to lo direction

	lsr multiplier
	bcc ?5				; branch if no *8

	clc
	adc multiplicand
?5
	ror a				; shift result down to lo direction

	lsr multiplier
	bcc ?6				; branch if no *16

	clc
	adc multiplicand
?6

; here is where we would re-scale number. amazingly there is no need, as long as we don't shift the
; result down as may be expected

	plp				; restore sign of number
	bpl ?7
	eor #255
	adc #1				; C is clear
?7
	rts


;
; x divide
;

x_divide

; divide accumulator. can be +ve or -ve. dividend is multiplied by 4, and the divisor
; should have 4 added to give result of fractional division of 1,1.25,1.5,1.75,2 etc.
; dividend should be in bytes (+ve or -ve)
; result is in pixels. (4 pixels per byte). ie multiplied by another 4

; Non-restoring divide routines. x/y (x & y to mem). ans=x 
; 	ldx #8, lda #0, asl x, ?1 rol a, cmp y, bcc ?2, sbc y,?2 rol x, dex ?1

	.local

	clc

	php				; save sign of dividend (NB c clear)
	bpl ?1

	eor #255			; convert dividend to +ve for divide to work
	adc #1
?1
	asl a				; sign bit will never be set so..
	asl a				; move next bit into carry instead
	sta dividend

	ldy #11				; normal division=8. +2 for *4 divide fract, +2 for *4 pixels
	lda #0				; one less needed because sign bit not used.
?2
	rol a

	cmp divisor
	bcc ?3

	sbc divisor
?3
	rol dividend

	dey
	bne ?2

	lda dividend			; answer in dividend
	plp				; restore original sign (NB c clear)
	bpl ?4

	eor #255
	adc #1
	clc
?4	
	adc #128			; convert x to 2d screen co-ordinate

	rts


;
; y divide
;

y_divide

; divide accumulator. can be +ve or -ve. dividend is multiplied by 4, and the divisor
; should have 4 added to give result of fractional division of 1,1.25,1.5,1.75,2 etc.
; dividend should be in pixels (+ve or -ve)
; result is in pixels. (4 pixels per byte)

	.local

	clc

	php				; save sign of dividend (NB c clear)
	bpl ?1

	eor #255			; convert dividend to +ve for divide to work
	adc #1
?1
	cmp #43
	bne ?1a				; branch if not special case of on ground

	ldy divisor
	lda div43z_table-4,y		; lookup predefined 43 divided by z table
	bpl ?3a				; always branch
?1a
	asl a				; sign bit will never be set so..
	asl a				; move next bit into carry instead
	sta dividend

	ldy #9				; normal division=8. +2 for *4 divide fract
	lda #0				; one less needed because sign bit not used.
?2
	rol a

	cmp divisor
	bcc ?3

	sbc divisor
?3
	rol dividend

	dey
	bne ?2

	lda dividend			; answer in dividend
?3a
	plp				; restore original sign (NB c clear)
	bpl ?4

	eor #255
	adc #1
;	clc				; comment out because always clearing on return anyway
?4	
	rts


;
; sort
;
	
sort	

; sort objects according to z postition
; z must be between 0-20+z_clip. z_clip will have minimal processing done. -ve z will be ignored.
; nb extra routine to be added which does iny by object number
; instead of dey to swap object priorities.

	.local

	lda #0				; clear sort indexes

	sta sort_index+1		; NB sort_index+0 will already be reset
	sta sort_index+2
	sta sort_index+3
	sta sort_index+4
	sta sort_index+5
	sta sort_index+6
	sta sort_index+7
	sta sort_index+8
	sta sort_index+9
	sta sort_index+10
	sta sort_index+11
	sta sort_index+12
	sta sort_index+13
	sta sort_index+14
	sta sort_index+15
	sta sort_index+16
	sta sort_index+17
	sta sort_index+18
	sta sort_index+19
	sta sort_index+20

; generate new sort index. do not include hidden objects (properties -ve).
; z pos must always be in range 0-20 or problems will occur. -ve now allowed so can be excluded quickly

	ldy #31
?2
	ldx z_pos,y
	bmi ?2a
	inc sort_index,x		; get a count of each occurrence of z in the sort index
?2a
	dey
	bpl ?2


	clc
	lda sort_index+0

	adc sort_index+1		; add count of z's in numerical order to get table position
	sta sort_index+1
	adc sort_index+2
	sta sort_index+2
	adc sort_index+3
	sta sort_index+3
	adc sort_index+4
	sta sort_index+4
	adc sort_index+5
	sta sort_index+5
	adc sort_index+6
	sta sort_index+6
	adc sort_index+7
	sta sort_index+7
	adc sort_index+8
	sta sort_index+8
	adc sort_index+9
	sta sort_index+9
	adc sort_index+10
	sta sort_index+10
	adc sort_index+11
	sta sort_index+11
	adc sort_index+12
	sta sort_index+12
	adc sort_index+13
	sta sort_index+13
	adc sort_index+14
	sta sort_index+14
	adc sort_index+15
	sta sort_index+15
	adc sort_index+16
	sta sort_index+16
	adc sort_index+17
	sta sort_index+17
	adc sort_index+18
	sta sort_index+18
	adc sort_index+19
	sta sort_index+19
	adc sort_index+20
	sta sort_index+20

	sbc #z_clip-1
	sta sprite_count		; count of objects. NB c is clear


	lda priority_status		; check for required sprite priorities
	bne ?6				; branch for sprite 31 highest priority when same z

	ldy #31				; 32 objects. object 0 ends up on top when same z 
?4
	ldx z_pos,y
	bmi ?5
	dec sort_index,x
	lda sort_index,x
	tax
	tya
	sta sort_table,x		; put object number in sort table based on sort index (z)
?5
	dey
	bpl ?4				; do until all objects

	rts				; exit

?6
	ldy #0				; 32 objects. object 31 ends up on top when same z
?7
	ldx z_pos,y
	bmi ?8
	dec sort_index,x
	lda sort_index,x
	tax
	tya
	sta sort_table,x
?8
	iny
	cpy #32				; do until all objects
	bne ?7

	rts				; exit


;
; initialize stage
;

init_stage

; turn os back on
; load stage overlay file into memory
; remove unrequired objects, load required objects
; turn os off again
; reset overlay pointers, and generally prepare for next stage

	.local

	lda #0


	sta music_pat_index		; music off

	ldx #8
?1
	sta sound_stop-1,x		; ensure sound player off
	sta sound_hi-1,x		; ensure volume off

	dex
	bne ?1

	jsr stop_sound			; stop sound channels playing


	.local

	sta NMIEN			; turn off nmi's. (irq already disabled)

	sta GRACTL			; disable players

	sta DMACTL			; disable screen dma


	ldx #3
?1
	sta GRAFP0,x			; set PM graphics direct draw registers to zero

	dex
	bpl ?1


	.local

	sta frame_count			; set screen ready to flip as soon as possible

	sta flip_count			; set to no flip screens outstanding

	sta dlist_index			; sync which screens to draw and display
	sta draw_screen_index

	sta bullet_count		; reset any outstanding bullet count

	sta man_flash_timer		; stop any outstanding man flashing

	sta last_properties		; reset mirror done flag

	sta sequencer_pointer		; initialize scenery and sequencer pointers and timers
	sta scenery1_pointer
	sta scenery2_pointer

	sta sky_index			; reset sky index

	sta wait_timer

	sta priority_status		; reset priority to normal. object 0 highest priority
	
	sta last_alien			; initialize last alien

	sta last_handler		; initialize last pattern handler

	sta uriah_status		; reset uriah status to normal

	sta counters_small		; ensure the counter overlays are reset
	sta counters_big


	.local

	lda #1
	sta sequencer_timer		; event sequencer must start straight away


	lda #3
	sta scenery1_timer
	sta scenery2_timer		; there's always a delay on scenery start

	jsr init_objects		; initialize/stop active objects - here to ensure right memory
	
	lda #~00000001
	ora PORTB
	sta PORTB			; re-enable os rom

	lda #3+32			; enable wide screen+screen dma only
	sta SDMCTL

	lda #192
	sta NMIEN			; turn on nmi's
	cli				; turn on irq's

	jsr wait_vbi			; loading stops shadow registers. wait to allow loading screen


	ldy #16				; default 10's digit of stage number to blank

	lda stage_no			; get stage number
	cmp #10
	bcc ?1				; branch if <10

	ldy #1				; 10's digit to 1

	sbc #10				; get 1's digit of stage number over 10
?1
	sta stage+1			; save 1's digit of stage number
	sta load_msg_stage+1
	sty stage			; save 10's digit of stage number
	sty load_msg_stage

	lda stage_no
	clc
	adc #stage_files_lo-files_lo	; start of stage files + stage number
	tax	
	
	lda #<stage_overlay
	sta DBUFLO
	lda #>stage_overlay
	sta DBUFHI

	jsr load_file			; load stage file to stage overlay area

	jsr load_background		; load background data into overlay area

	
	.local

	ldx #max_objects-1		; index for possible objects in memory
	
	stx slot_no			; slot_no to a non valid slot for first shared slot check
?1
	lda object_to_slot,x
	bmi ?3				; branch if object not in memory		

	ldy #max_stage_objects-1	; index for objects required
	txa				; memory object number to check into A reg
?2
	cmp objects_required,y
	beq ?3				; branch if memory object in required list

	dey
	bpl ?2				; do until all on required list

	jsr remove_object		; remove object not on required list
?3
	dex
	bpl ?1				; do until all possible objects


	jsr make_mirror_table		; create mirror table for any mirrored sprites required


	ldy #max_stage_objects-1	; index for objects required
?4
	ldx objects_required,y		; object number into x
	bmi ?5				; branch if no object to load

	lda object_to_slot,x		; get any slot number of object
	bpl ?5				; branch if object already in memory

	jsr load_object			; load object into memory
?5
	dey
	bpl ?4				; do until all objects required


	.local

	jsr optimize_objects		; optimize page faults on objects in memory

	lda #0
	
	sta AUDCTL			; enable normal sound

	sta NMIEN			; turn off nmi's
	sei				; turn off irq's

	lda #3
	sta SKCTL			; ensure 2-tone mode disabled

	lda #~11111110
	and PORTB
	sta PORTB			; disable os rom

	lda #128			; initialize man's position
	sta man_x_pos

	jsr clear_players		; clear player/missile graphics area

	lda #64
	sta man_y_pos

	ldx grid_colour_start		; NB start must be 1 more than table position required
	stx grid_colour_index		; initialize grid colour start index

	lda #1
	sta grid_colour_timer		; initialize grid colour timer to fetch next call



	lda PAL				; Read for PAL or NTSC
	lsr a		
	lsr a				; bit one into carry (C set is NTSC)

	lda #0
	adc stage_speed
	sta game_speed			; VBI ticks before screen flip. EG 4=15 fps, 2=30 fps NTSC

	ldy song_required		; get required song position from stage overlay
	beq ?1				; skip if no song

	jsr init_music			; start playing song
?1
	lda #3+8+32			; enable wide screen+player+screen dma
	sta DMACTL			

	lda #192
	sta NMIEN			; turn on nmi's DLIS and VBI

	ldx uriah_stage
	stx uriah_status		; turn on uriah bonus stage if uriah stage (non zero)
	bne ?2				; skip past man colours if uriah stage

	lda stage_no
	beq ?4				; skip past enable players if title screen

	lda #64+4			; red for man
	sta COLPM0			; player 0 colour
	sta PCOLR0
	sta COLPM2			; player 2 colour
	sta PCOLR2

	lda #144+6			; blue for man
	sta COLPM1			; player 1 colour
	sta PCOLR1
	sta COLPM3			; player 3 colour
	sta PCOLR3
	
	bne ?3				; always branch
?2
	lda counter_colour-1,x		; colour of counter overlays

	sta COLPM0			; player 0 colour
	sta PCOLR0
	sta COLPM2			; player 2 colour
	sta PCOLR2	
	sta COLPM1			; player 1 colour
	sta PCOLR1
	sta COLPM3			; player 3 colour
	sta PCOLR3

	lda #128-16			; line up players next to each other horizontally
	sta HPOSP0			; in the centre of the screen
	lda #128-8
	sta HPOSP1
	lda #128
	sta HPOSP2
	lda #128+8
	sta HPOSP3
?3
	lda #2
	sta GRACTL			; enable players
?4
	inc stage_no			; ready for next stage


	rts


;
;	Initial Setup routine.
;

setup

; to be run only once after the load of the program

	.local

	lda #0

	sta NMIEN			; turn off nmi's

	sei				; disable irq's

	sta DMACTL			; disable screen dma

; leave the irq mask alone for the sake of the os which is going to be re-enabled for loading 
;	sta 53774


	lda #0+2+16+32+128		; disable rom,basic,os & self test
	sta PORTB			; enable 130xe main mem bank only


	lda #<nmi_handler		; setup new handlers for nmi's and irq's
	sta 65530
	lda #>nmi_handler
	sta 65531

;	lda #<irq_handler
;	sta 65534
;	lda #>irq_handler
;	sta 65535

	lda #>charset			; space harrier limited character set
	sta CHBAS
	sta CHBASE

	lda #1				; charset control to allow blinking characters with bit 7
	sta CHACT
	sta CHACTL



	lda #>pmbase_addr		; set player/missile address
	sta PMBASE

	lda #1+16+32			; setup missiles to mask off screen
	sta PRIOR			; bit 0 - players in front of playfield.
	sta GPRIOR			; bit 4 - 5th player colour. bit 5 - player overlap colour

	lda #255			; all missiles to 4x size
	sta SIZEM

	sta GRAFM			; set graphics for missiles since we're not using dma

	lda #128-72-8			; arrange x positions of missiles to mask sides of screen
	sta HPOSM0			; visible screen is 144 pixels wide. (0-143)
	lda #128-72-16
	sta HPOSM1
	lda #128+72
	sta HPOSM2
	lda #128+72+8
	sta HPOSM3


	lda #0

	sta COLPF3			; set missiles as colour black
	sta COLOR3

	sta COLPF2			; black colour as background
	sta COLOR2

	sta vbi_count			; vbi count must be 0 or 1. initialize to 0

	sta man_run_frame		; initialize man run frame

	sta grid_index			; initialize grid animation frame
	sta grid_colour_swap

	sta dli_buffer			; clear dli buffers. (mark end of data screen1 dlis)
	sta dli_buffer+max_dlis		; screen2 dlis
	sta dli_buffer+[max_dlis*2]	; screen3 dlis

	sta sample_status		; ensure sample playing disabled

	sta last_alien			; initialize last alien

	sta last_handler		; initialize last pattern handler

	sta last_missile		; initialize last missile

	sta x_vector_buffer		; make sure vector buffer zero index has zero vectors
	sta y_vector_buffer
	sta z_vector_buffer
	sta z_vector_lo_buffer
	sta frame_change_buffer
	sta ftimer_buffer

	sta music_pat_index		; music off
	

	.local

	ldx #8
?1
	sta sound_stop-1,x		; ensure sounds off

	dex
	bne ?1

	sta sound_queue
	

	.local

	ldx #z_clip
?1
	sta sort_index-1,x		; z clip area of sort index must be zero 

	dex				; to separate from on-screen objects
	bne ?1


	lda #z_clip			; set sort index so z clipped objects separated
	
	sta sort_index-1		; sort index must be set before start (as must sort_index-1)
	sta sort_index


	.local				
?1
	txa
	tay				; setup mask table
	and #~00000011
	bne ?2
	tya
	ora #~00000011
	tay
?2
	txa
	and #~00001100
	bne ?3
	tya
	ora #~00001100
	tay 
?3
	txa
	and #~00110000
	bne ?4
	tya
	ora #~00110000
	tay
?4
	txa
	and #~11000000
	bne ?5
	tya
	ora #~11000000
	tay
?5
	tya		
	sta mask_table,x

	dex
	bne ?1


	.local

	ldx #20				; clear tilt table
?1
	lda #0
	sta tilt_hi,x

	lda #127
	sta tilt_lo,x

	dex
	bpl ?1


	.local

	lda #128

	ldx #max_objects		; initialize object to slot lookup, all objects as unused
?1
	sta object_to_slot,x

	dex
	bpl ?1					

	sta player_index		; man die sequencer code off


	.local

	lda #48-9			; setup x 48 table minus 9 for screen y position
	sta x48min9_table_lo+1		; NB don't use zeroth entry of this table it's not set
	lda #0
	sta x48min9_table_hi+1

	ldx #2
?1
	lda x48min9_table_lo-1,x
	clc
	adc #48
	sta x48min9_table_lo,x
	lda x48min9_table_hi-1,x
	adc #0
	sta x48min9_table_hi,x

	inx
	cpx #140
	bne ?1


	lda #32				; set horizon to half man's y position only 1st time round
	sta horizon		
	lda #32-57
	sta sky_scroll			; set sky_scroll to 57 less than horizon

	jsr init_banks			; setup the memory banks


;
; initialize game
;

init_game

; initialize start of a new game


	lda #<dlist_blank		; setup blank dlist address for os loading of menu screen.
	sta SDLSTL
	lda #>dlist_blank
	sta SDLSTH

; reset game speed depending on NTSC or PAL atari. 12fps=5 NTSC vbi, or 4 PAL vbi
; result to be stored is this less 1
; bits 1-3 zero is PAL, bits 1-3 ones, NTSC
; vcount maximum value is different too.

	.local

	ldx #131			; default to NTSC vcount (0-130)

	lda PAL				; Read for PAL or NTSC
	lsr a		
	lsr a				; bit one into carry
	
	bcs ?1				; branch if NTSC

	ldx #156			; PAL vcount is 0-155
?1
	stx max_vcount			; maximum vcount value (+1)


	.local

	lda #max_lives			; set man lives
	sta man_lives

	tay
	beq ?2				; for testing only check for just 1 life

	lda #20
?1
	sta lives-1,y

	dey
	bne ?1
?2

	sty GRACTL			; disable players

	sty man_status			; initialize man's status
	sty man_flash_timer		; no flashing of man

	sty score_status		; reset score status to not a high score
	sty lives_status		; and man lives to no extra man yet

	sty top_grid_status		; top grid off

	sty background_x		; initialize background position

	sty active_pat_count		; initialize active pattern count

	sty game_status			; ensure game status is reset


	.local

	ldy #2
?1
	lda old_pos_offset,y		; clear old position index
	sta old_pos_index,y

	dey
	bpl ?1


	.local

	ldy #6				; blank previous score
	lda #16
?1
	sta score,y

	dey
	bpl ?1


	lda #0				; initial title screen stage
	sta stage_no


	.local
			
	ldx #max_handlers		; ensure pattern handlers are stopped
?1
	sta pat_kill-1,x

	dex
	bne ?1
	
	lda #128
	sta sum_status			; ensure sum pattern disabled
	

	jsr init_stage			; initialize stage

	lda #start_from_stage
	sta stage_no			; for testing start stage


	lda #<dlist_load		; setup dlist address for os stage loading from now on
	sta 560
	lda #>dlist_load
	sta 561


	jmp title_screen		; do title screen


;
; title screen
;

title_screen

	jsr new_screen

	jsr copy_title_dlis

	jsr draw_dlis

	jsr draw_background

	inc flip_count

	jsr wait_press


	jsr init_stage


;
; main loop	
;

main_loop

	.local



; sound test



;	lda #1
;	jsr init_sound


; music test

;	ldy #song1+1
;	jsr init_song





?1

game_command_rtn

;	dec wait_timer			; Only run this code once every wait timer
;	bpl ?1a				; branch if not time yet
;
;	jsr sequence_events		; check for new patterns
;
;	jsr sequence_scenery		; check for new scenery
;
;	ldx stage_speed
;	lda fps_table+1,x
;	lsr a
;
;	sta wait_timer			; reset the wait timer to approx once every 1/2 second
;?1a
	jsr new_screen			; get a new screen to draw on
;
;	lda man_status
;	bpl ?2				; skip if man alive
;	jsr kill_man			; kill man
;	lda man_lives
;	bpl ?2a				; if more lives left branch to re-draw last screen

;	jmp end_game			; if no lives left end the game
;?2
;	jsr move_objects
;?2a
;	jsr sort
;
;	jsr move_dlis
;
;	jsr draw_dlis
;
;
;	jsr draw_background
;
;
;	jsr draw_screen
;	jsr update_score
;
;	jsr check_object_collisions	; has to be after sort coz it uses sort table




; test

	jsr test_draw





; flip screen

	inc flip_count

; debug code

;	lda active_pat_count
;	sta lives+5			; debug - show active pattern count near lives
;
;	ldx #4
;?temp
;	lda pat_kill,x
;	sta top_score+9,x		; debug - show aliens left in patterns near top score
;	dex
;	bpl ?temp

; debug code end


	ldy game_status
	beq ?1				; loop while game status has no commands waiting

	lda game_command_lo-1,y		; get address of game command required
	sta jump			; (ie init_stage, grid_down, grid_up etc)

	lda game_command_hi-1,y
	sta jump+1

	lda #0
	sta game_status			; reset status to normal "no commands"

	jmp (jump)			; run command routine


; test

test_draw

	lda #0
	sta dest_a
	sta dest_b
	
	lda #>screen3a
	sta dest_a+1
	lda #>screen3b
	sta dest_b+1
	
	jsr draw_test
	
	rts
	






;
; end game
;

end_game

; end of game where man has died

	jsr wait_press

	jmp init_game


;
; Pattern Data
;

	.include patterns.asm


;
; mem remaining help. put all 4000-7fff data before this
;

mem_position_4000-7fff


;
; autorun addr
;

	*=$2e0

	.byte <setup,>setup


; end of program

